Package: Rmfrac
Type: Package
Title: Simulation and Statistical Analysis of Multifractional Processes
Version: 1.0.0
Description: Simulation of several fractional and multifractional processes. Includes Brownian and fractional Brownian motions, bridges and Gaussian Haar-based multifractional processes (GHBMP). Implements the methods from Ayache, Olenko and Samarakoon (2026) <doi:10.1016/j.matcom.2026.01.033> for simulation of GHBMP. Estimation of Hurst functions and local fractal dimension. Clustering realisations based on the Hurst functions. Several functions to estimate and plot geometric statistics of the processes and time series. Provides a 'shiny' application for interactive use of the functions from the package.
License: GPL-3
Encoding: UTF-8
Imports: parallel, parallelly, ggplot2(>= 3.5.0), zoo, matrixStats,
        proxy, rlang, stats, foreach, doParallel, plotly, shiny,
        graphics, shinycssloaders, fields, utils
Suggests: testthat
Config/testthat/edition: 3
RoxygenNote: 7.3.3
URL: https://github.com/Nemini-S/Rmfrac
BugReports: https://github.com/Nemini-S/Rmfrac/issues
Authors@R: c(person("Andriy", "Olenko", email = "a.olenko@latrobe.edu.au", role = "aut", comment = c(ORCID = "0000-0002-0917-7000")),
        person("Nemini", "Samarakoon", email = "neminisamarakoon95@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0009-0000-2107-1973")))
Maintainer: Nemini Samarakoon <neminisamarakoon95@gmail.com>
NeedsCompilation: no
Packaged: 2026-04-17 01:34:21 UTC; 21381851
Author: Andriy Olenko [aut] (ORCID: <https://orcid.org/0000-0002-0917-7000>),
  Nemini Samarakoon [aut, cre] (ORCID:
    <https://orcid.org/0009-0000-2107-1973>)
Repository: CRAN
Date/Publication: 2026-04-17 03:30:07 UTC
Built: R 4.5.2; ; 2026-04-17 09:57:19 UTC; unix

Package: Rmfrac
Type: Package
Title: Simulation and Statistical Analysis of Multifractional Processes
Version: 1.0.0
Description: Simulation of several fractional and multifractional processes. Includes Brownian and fractional Brownian motions, bridges and Gaussian Haar-based multifractional processes (GHBMP). Implements the methods from Ayache, Olenko and Samarakoon (2026) <doi:10.1016/j.matcom.2026.01.033> for simulation of GHBMP. Estimation of Hurst functions and local fractal dimension. Clustering realisations based on the Hurst functions. Several functions to estimate and plot geometric statistics of the processes and time series. Provides a 'shiny' application for interactive use of the functions from the package.
License: GPL-3
Encoding: UTF-8
Imports: parallel, parallelly, ggplot2(>= 3.5.0), zoo, matrixStats,
        proxy, rlang, stats, foreach, doParallel, plotly, shiny,
        graphics, shinycssloaders, fields, utils
Suggests: testthat
Config/testthat/edition: 3
RoxygenNote: 7.3.3
URL: https://github.com/Nemini-S/Rmfrac
BugReports: https://github.com/Nemini-S/Rmfrac/issues
Authors@R: c(person("Andriy", "Olenko", email = "a.olenko@latrobe.edu.au", role = "aut", comment = c(ORCID = "0000-0002-0917-7000")),
        person("Nemini", "Samarakoon", email = "neminisamarakoon95@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0009-0000-2107-1973")))
Maintainer: Nemini Samarakoon <neminisamarakoon95@gmail.com>
NeedsCompilation: no
Packaged: 2026-04-17 01:34:21 UTC; 21381851
Author: Andriy Olenko [aut] (ORCID: <https://orcid.org/0000-0002-0917-7000>),
  Nemini Samarakoon [aut, cre] (ORCID:
    <https://orcid.org/0009-0000-2107-1973>)
Repository: CRAN
Date/Publication: 2026-04-17 03:30:07 UTC
Built: R 4.6.0; ; 2026-04-30 15:57:07 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(Rmfrac::plot,H_LFD)
S3method(Rmfrac::plot,hc_hurst)
S3method(Rmfrac::plot,k_hurst)
S3method(Rmfrac::plot,mp)
S3method(Rmfrac::print,hc_hurst)
S3method(Rmfrac::print,k_hurst)
S3method(autoplot,H_LFD)
S3method(autoplot,hc_hurst)
S3method(autoplot,k_hurst)
S3method(autoplot,mp)
export(Bbridge)
export(Bm)
export(FBbridge)
export(FBm)
export(FGn)
export(GHBMP)
export(H_LFD)
export(Hurst)
export(LFD)
export(RS_Index)
export(X_max)
export(X_min)
export(cov_GHBMP)
export(cross_T)
export(cross_mean)
export(cross_rate)
export(est_cov)
export(exc_Area)
export(hclust_hurst)
export(kmeans_hurst)
export(long_streak)
export(mean_streak)
export(plot_tsest)
export(shinyapp_sim)
export(sojourn)
importFrom(doParallel,registerDoParallel)
importFrom(fields,image.smooth)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(ggplot2,aes)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,sec_axis)
importFrom(ggplot2,theme)
importFrom(ggplot2,vars)
importFrom(graphics,plot)
importFrom(matrixStats,sum2)
importFrom(parallel,clusterExport)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(parallelly,availableCores)
importFrom(parallelly,makeClusterPSOCK)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(proxy,dist)
importFrom(rlang,.data)
importFrom(shiny,runApp)
importFrom(shinycssloaders,withSpinner)
importFrom(stats,IQR)
importFrom(stats,aggregate)
importFrom(stats,approx)
importFrom(stats,as.dendrogram)
importFrom(stats,cutree)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,loess)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,rect.hclust)
importFrom(stats,rnorm)
importFrom(stats,toeplitz)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(zoo,index)
importFrom(zoo,rollapply)

## Simulation and analysis of multifractional processes 
[![Lifecycle: maturing](https://img.shields.io/badge/lifecycle-maturing-blue)](https://lifecycle.r-lib.org/articles/stages.html#maturing)
[![CRAN downloads (total)](https://cranlogs.r-pkg.org/badges/grand-total/Rmfrac)](https://cranlogs.r-pkg.org/badges/grand-total/Rmfrac)
[![CRAN status](https://www.r-pkg.org/badges/version/Rmfrac)](https://CRAN.R-project.org/package=Rmfrac)
<p>
<img width="200" height="220" alt="logo" src="https://github.com/user-attachments/assets/5055ed96-bbed-4907-b609-5411c4f84f92" align="right"align="right" />
Rmfrac provides a collection of tools for simulating, analysing and visualising multifractional processes and time series. The package includes built-in estimation techniques for the Hurst function, Local Fractal Dimension and several other geometric statistics. It provides highly customisable plotting functions for simulated realisations, user-provided time series and their statistics. 
</p>

**Features**
- Simulation of Brownian motion, fractional Brownian motion, fractional Gaussian noise, Brownian bridge and fractional Brownian bridge
- Simulation of Gaussian Haar-based multifractional process (GHBMP)
- Estimation of Hurst function and Local Fractal Dimension
- Customisable plotting functions for GHBMP and user provided time series with estimates of Hurst function and Local Fractal Dimension
- Estimation and visualisation of geometric statistics using realisations of stochastic processes and time series. Clustering based on the Hurst function, sojourn measure, excursion area, etc.
- An interactive Shiny application that provides options to explore and visualise the core functionalities of the package through simulations and user-provided time series.

 
## Installation

To install the development version of [Rmfrac](https://github.com/Nemini-S/Rmfrac) package version from GitHub:
```{r}
# Install devtools if not available
# install.packages("devtools")
devtools::install_github("Nemini-S/Rmfrac")
```

## Getting started 
```{r}
library(Rmfrac)
```
To simulate a Gaussian Haar-based multifractional process for a constant Hurst function
```{r}
t <- seq(0, 1, by = (1/2)^10)
H1 <- function(t) {return(0.5 + 0 * t)}
X1 <- GHBMP(t, H1, J = 12)
```
Oscillating Hurst function
```{r}
H2 <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
X2 <- GHBMP(t, H2, J = 12)
```
Piecewise Hurst function
```{r}
H3 <- function(x) {
ifelse(x >= 0 & x <= 0.8, 0.375 * x + 0.2,
      ifelse(x > 0.8 & x <= 1,-1.5 * x + 1.7, NA))
}
X3 <- GHBMP(t, H3, J = 12)
```
To estimate the Hurst function and Local Fractal Dimension with visualizations
```{r}
Hurst_estimates <- Hurst(X2, N = 100)
LFD_estimates <- LFD(X2, N = 100)

plot(X2, H_EST = TRUE, H_Smooth_Est = TRUE, LFD_Est = TRUE, LFD_Smooth_Est = TRUE)
```
<img width="1918" height="975" alt="image" src="https://github.com/user-attachments/assets/c7df246f-368f-41b2-b108-1b0f1dc002fa" />


## Citation
Ayache A, Olenko A, Samarakoon N (2025). “On Construction, Properties and Simulation
of Haar-Based Multifractional Processes.” doi:10.48550/arXiv.2503.07286. Submitted,
URL https://arxiv.org/abs/2503.07286.

ui <- shiny::navbarPage("",
      shiny::tabPanel("Brownian Motion",
      shiny::fluidRow(
      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Simulation Parameters"),
      shiny::numericInput("x_BM", "Initial value", value = 0),
      shiny::numericInput("t_start_BM", "Initial time point", value = 0, min = 0),
      shiny::numericInput("t_end_BM", "Terminal time point", value = 1, min = 0),
      shiny::numericInput("N_BM", "Number of time steps", value = 1000, min = 0, step = 1),
      shiny::actionButton("submit_BM", "Simulate")),

      shiny::wellPanel(
      shiny::h4("Excursion set and area"),
      shiny::numericInput("N_BMs", "Number of time steps", value = 10000, min = 0, step = 1),
      shiny::numericInput("A_BM", "Constant level", value = 0),
      shiny::selectInput("level_BM",label = "Compare to level",
                          choices = list("Greater" = "greater", "Lower" = "lower"), selected = "greater"),
      shiny::checkboxInput("sm_BM", "Excursion set", value = FALSE),
      shiny::checkboxInput("ea_BM", "Excursion area", value = FALSE))),

      shiny::column(8,
      shinycssloaders::withSpinner((shiny::plotOutput("bmPlot", height = "500px")), type = 8, color = "grey")),

      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Maximum and minimum"),
      shiny::checkboxInput("max_BM", "Maximum", value = FALSE),
      shiny::checkboxInput("min_BM", "Minimum", value = FALSE))),

      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Longest Streak"),
      shiny::checkboxInput("increasing_BM", "Increasing (orange)", value = FALSE),
      shiny::checkboxInput("decreasing_BM", "Decreasing (brown)", value = FALSE))))),



      shiny::tabPanel("Brownian Bridge",
      shiny::fluidRow(
      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Simulation Parameters"),
      shiny::numericInput("xend_BB", "Terminating value", value = 1),
      shiny::numericInput("t_end_BB", "Terminal time point", value = 1, min = 0),
      shiny::numericInput("xstart_BB", "Initial value", value = 0),
      shiny::numericInput("N_BB", "Number of time steps", value = 1000, min = 0, step = 1),
      shiny::actionButton("submit_BB", "Simulate")),

      shiny::wellPanel(
      shiny::h4("Excursion set and area"),
      shiny::numericInput("N_BBs", "Number of time steps", value = 10000, min = 0, step = 1),
      shiny::numericInput("A_BB", "Constant level", value = 0),
      shiny::selectInput("level_BB",label = "Compare to level",
                         choices = list("Greater" = "greater", "Lower" = "lower"),selected = "greater"),
      shiny::checkboxInput("sm_BB", "Excursion set", value = FALSE),
      shiny::checkboxInput("ea_BB", "Excursion area", value = FALSE))),

      shiny::column(8,
      shinycssloaders::withSpinner((shiny::plotOutput("bbPlot", height = "503px")), type = 8, color = "grey")),
      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Maximum and minimum"),
      shiny::checkboxInput("max_BB", "Maximum", value = FALSE),
      shiny::checkboxInput("min_BB", "Minimum", value = FALSE))),

      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Longest Streak"),
      shiny::checkboxInput("increasing_BB", "Increasing (orange)", value = FALSE),
      shiny::checkboxInput("decreasing_BB", "Decreasing (brown)", value = FALSE))))),


      shiny::tabPanel("Fractional Brownian Motion",
      shiny::fluidRow(
      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Simulation Parameters"),
      shiny::numericInput("H_FBM", "Hurst Parameter", value = 0.5, min = 0, max = 1),
      shiny::numericInput("x_FBM", "Initial value", value = 0),
      shiny::numericInput("t_start_FBM", "Initial time point", value = 0, min = 0),
      shiny::numericInput("t_end_FBM", "Terminal time point", value = 1, min = 0),
      shiny::numericInput("N_FBM", "Number of time steps", value = 1000, step = 1, min = 1),
      shiny::actionButton("submit_FBM", "Simulate")),

      shiny::wellPanel(
      shiny::h4("Excursion set and area"),
      shiny::numericInput("N_FBMs", "Number of time steps", value = 10000, min = 0, step = 1),
      shiny::numericInput("A_FBM", "Constant level", value = 0),
      shiny::selectInput("level_FBM",label = "Compare to level",
                          choices = list("Greater" = "greater", "Lower" = "lower"),selected = "greater"),
      shiny::checkboxInput("sm_FBM", "Excursion set", value = FALSE),
      shiny::checkboxInput("ea_FBM", "Excursion area", value = FALSE))),


      shiny::column(8,
      shinycssloaders::withSpinner((shiny::plotOutput("fbmPlot", height = "503px")), type = 8, color = "grey")),
      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Maximum and minimum"),
      shiny::checkboxInput("max_FBM", "Maximum", value = FALSE),
      shiny::checkboxInput("min_FBM", "Minimum", value = FALSE))),
      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Longest Streak"),
      shiny::checkboxInput("increasing_FBM", "Increasing (orange)", value = FALSE),
      shiny::checkboxInput("decreasing_FBM", "Decreasing (brown)", value = FALSE))))),

      shiny::tabPanel("Fractional Brownian Bridge",
      shiny::fluidRow(
      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Simulation Parameters"),
      shiny::numericInput("H_FBB", "Hurst Parameter", value = 0.5, min = 0, max = 1),
      shiny::numericInput("xend_FBB", "Terminating value", value = 1),
      shiny::numericInput("t_end_FBB", "Terminal time point", value = 1, min = 0),
      shiny::numericInput("xstart_FBB", "Initial value", value = 0),
      shiny::numericInput("N_FBB", "Number of time steps", value = 1000, min = 0, step = 1),
      shiny::actionButton("submit_FBB", "Simulate")),

      shiny::wellPanel(
      shiny::h4("Excursion set and area"),
      shiny::numericInput("N_FBBs", "Number of time steps", value = 10000, min = 0, step = 1),
      shiny::numericInput("A_FBB", "Constant level", value = 0),
      shiny::selectInput("level_FBB",label = "Compare to level",
                         choices = list("Greater" = "greater", "Lower" = "lower"), selected = "greater"),
      shiny::checkboxInput("sm_FBB", "Excursion set", value = FALSE),
      shiny::checkboxInput("ea_FBB", "Excursion area", value = FALSE))),
      shiny::column(8,
      shinycssloaders::withSpinner((shiny::plotOutput("fbbPlot",height="576px")), type = 8, color = "grey")),
      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Maximum and minimum"),
      shiny::checkboxInput("max_FBB", "Maximum", value = FALSE),
      shiny::checkboxInput("min_FBB", "Minimum", value = FALSE))),
      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Longest Streak"),
      shiny::checkboxInput("increasing_FBB", "Increasing (orange)", value = FALSE),
      shiny::checkboxInput("decreasing_FBB", "Decreasing (brown)", value = FALSE))))),


      shiny::tabPanel("Fractional Gaussian Noise",
      shiny::fluidRow(
      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Simulation Parameters"),
      shiny::numericInput("H_FGN", "Hurst Parameter", value = 0.5, min = 0, max = 1),
      shiny::numericInput("t_start_FGN", "Initial time point", value = 0, min = 0),
      shiny::numericInput("t_end_FGN", "Terminal time point", value = 1, min = 0),
      shiny::numericInput("N_FGN", "Number of time steps", value = 1000, min = 0, step = 1),
      shiny::actionButton("submit_FGN", "Simulate")),

      shiny::wellPanel(
      shiny::h4("Excursion set and area"),
      shiny::numericInput("N_FGNs", "Number of time steps", value = 10000, min = 0, step = 1),
      shiny::numericInput("A_FGN", "Constant level", value = 0),
      shiny::selectInput("level_FGN",label = "Compare to level",
                         choices = list("Greater" = "greater", "Lower" = "lower"), selected = "greater"),
      shiny::checkboxInput("sm_FGN", "Excursion set", value = FALSE),
      shiny::checkboxInput("ea_FGN", "Excursion area", value = FALSE))),

      shiny::column(8,
      shinycssloaders::withSpinner((shiny::plotOutput("fgnPlot", height = "500px")), type = 8, color = "grey")),
      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Maximum and minimum"),
      shiny::checkboxInput("max_FGN", "Maximum", value = FALSE),
      shiny::checkboxInput("min_FGN", "Minimum", value = FALSE))),
      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Longest Streak"),
      shiny::checkboxInput("increasing_FGN", "Increasing (orange)", value = FALSE),
      shiny::checkboxInput("decreasing_FGN", "Decreasing (brown)", value = FALSE))))),

      shiny::tabPanel("GHBMP",
      shiny::fluidRow(
      shiny::column(4,
      shiny::wellPanel(
      shiny::h4("Simulation"),
      shiny::textInput("func", "Hurst function (in terms of t)", "0.5 + 0*t"),
      shiny::textInput("time", "Time sequence", "seq(0, 1, by = (1/2)^10)"),
      shiny::numericInput("num", "J (positive integer for simulation)", value = 15, min = 1, step = 1),
      shiny::actionButton("submit3", "Simulate")),
      shiny::wellPanel(
      shiny::h4("Estimation"),
      shiny::numericInput("N_int", "Number of sub-intervals for estimation", value = 100, min = 1, step = 1),
      shiny::numericInput("Q", "Q (integer for estimation (>=2))", value = 2, min = 2, step = 1),
      shiny::numericInput("L", "L (integer for estimation (>=2))", value = 2, min = 2, step = 1),
      shiny::checkboxGroupInput("checkbox_group", "Select",
                                choices = list("Theoretical Hurst function" = "H","Raw estimate of Hurst function" = "Raw_Est_H",
                                                "Smoothed estimate of Hurst function" = "Smooth_Est_H",
                                                "Raw estimate of Local Fractal Dimension" = "LFD_Est",
                                                "Smoothed estimate of Local Fractal Dimension" = "LFD_Smooth_Est")))),

     shiny::column(8,
     shinycssloaders::withSpinner((shiny::plotOutput("functionPlot", height = "500px")), type = 8, color = "grey"),
     shiny::fluidRow(
     shiny::column(6,
     shiny::wellPanel(
     shiny::h4("Excursion set and area"),
     shiny::numericInput("N_GHBMP", "Number of time steps", value = 10000, min = 0, step = 1),
     shiny::numericInput("A_GHBMP", "Constant level", value = 0),
     shiny::selectInput("level",label = "Compare to level",
                        choices = list("Greater" = "greater", "Lower" = "lower"), selected = "greater"),
     shiny::checkboxInput("sm", "Excursion set", value = FALSE),
     shiny::checkboxInput("ea", "Excursion area", value = FALSE))),

     shiny::column(5,
     shiny::wellPanel(
     shiny::h4("Maximum and minimum"),
     shiny::checkboxInput("max3", "Maximum", value = FALSE),
     shiny::checkboxInput("min3", "Minimum", value = FALSE))),
     shiny::column(5,
     shiny::wellPanel(
     shiny::h4("Longest Streak"),
     shiny::checkboxInput("increasing3", "Increasing (orange)", value = FALSE),
     shiny::checkboxInput("decreasing3", "Decreasing (brown)", value = FALSE))))))),

     shiny::tabPanel("Input Time Series",
     shiny::fluidRow(
     shiny::column(2,
     shiny::selectInput("panel_select","",choices = c("Estimations", "Excursion set and area"),
                        selected = "Estimations")), column(10)),

     shiny::fluidRow(
     shiny::column(3,
     shiny::conditionalPanel(condition = "input.panel_select == 'Estimations'",
     shiny::wellPanel(
     shiny::h4("Estimation"),
     shiny::numericInput("N_intTS", "Number of sub-intervals for estimation", value = 100, min = 1, step = 1),
     shiny::numericInput("Q_TS", "Q (integer for estimation (>=2))", value = 2, min = 2, step = 1),
     shiny::numericInput("L_TS", "L (integer for estimation (>=2))", value = 2, min = 2, step = 1),
     shiny::checkboxGroupInput("checkbox_group_TS", "Select",
                                choices = list("Raw estimate of Hurst function" = "Raw_Est_H",
                                               "Smoothed estimate of Hurst function" = "Smooth_Est_H",
                                               "Raw estimate of Local Fractal Dimension" = "LFD_Est",
                                               "Smoothed estimate of Local Fractal Dimension" = "LFD_Smooth_Est")))),

     shiny::conditionalPanel(condition = "input.panel_select=='Excursion set and area'",
     shiny::wellPanel(
     shiny::h4("Excursion set and area"),
     shiny::numericInput("N_TS", "Number of time steps", value = 10000, min = 0, step = 1),
     shiny::numericInput("A_TS", "Constant level", value = 0),
     shiny::selectInput("level_TS", label = "Compare to level",
                        choices = list("Greater" = "greater", "Lower" = "lower"), selected = "greater"),
     shiny::checkboxInput("sm_TS", "Excursion set", value = FALSE),
     shiny::checkboxInput("ea_TS", "Excursion area", value = FALSE))),


     shiny::wellPanel(
     shiny::h4("Maximum and minimum"),
     shiny::checkboxInput("max_TS", "Maximum", value = FALSE),
     shiny::checkboxInput("min_TS", "Minimum", value = FALSE)),

     shiny::wellPanel(
     shiny::h4("Longest Streak"),
     shiny::checkboxInput("increasing_TS", "Increasing (orange)", value = FALSE),
     shiny::checkboxInput("decreasing_TS", "Decreasing (brown)", value = FALSE))),

     shiny::column(9,
     shiny::wellPanel(fileInput("file", "Upload CSV File", accept = ".csv"), uiOutput("column_ui")),

     shiny::conditionalPanel(condition = "input.panel_select =='Estimations'",
     shinycssloaders::withSpinner((shiny::plotOutput("tsplot1", height = "500px")), type = 8, color = "grey")),
     shiny::conditionalPanel(condition = "input.panel_select =='Excursion set and area'",
     shinycssloaders::withSpinner((shiny::plotOutput("tsplot2", height = "500px")), type = 8, color = "grey"))

     ))),
)


server <- function(input, output, session) {

  data_input <- shiny::reactive({
    req(input$file)
    DF_Input <- utils::read.csv(input$file$datapath, header = TRUE)
    DF_Input
  })

  output$tsplot1 <- shiny::renderPlot({

    InputTS <- data_input()
    shiny::req(InputTS)
    InputTS <- na.omit(InputTS)
    InputTS <- InputTS[order(InputTS[,1]),]

    p6 <- ggplot2::ggplot(InputTS, ggplot2::aes(x = InputTS[,1], y = InputTS[,2])) + ggplot2::geom_line() +
      ggplot2::labs(y = "Time Series", x = "Time", color = "")

    N <- input$N_intTS
    Q <- input$Q_TS
    L <- input$L_TS

    H_est <- Hurst(InputTS, N, Q, L)
    colnames(H_est) <- c("x", "y")
    IQR_H <- stats::IQR(InputTS[,2])
    range_H <- range(InputTS[,2])
    q1_H <-stats::quantile(InputTS[,2], 0.25)

    t1<-InputTS[,1]

    LFD_est <- LFD(InputTS, N, Q, L)
    colnames(LFD_est) <- c("x1", "y1")
    IQR_L <- stats::IQR(InputTS[,2])
    range_L <- range(InputTS[,2])
    q1_L <- stats::quantile(InputTS[,2], 0.25)

    if("Raw_Est_H" %in% input$checkbox_group_TS){
      p6 <- p6 + ggplot2::geom_line(data = H_est,ggplot2::aes(x = .data$x, y = ((.data$y * (IQR_H)) + q1_H), color = factor("Raw Estimate H")), linewidth = 1) +
        ggplot2::scale_y_continuous(name = "Time Series", limits = range_H,
                                    sec.axis = ggplot2::sec_axis(transform = function(x){(x - q1_H) / (IQR_H)}, name = "Estimator"))
    }

    if("Smooth_Est_H" %in% input$checkbox_group_TS){
      p6 <- p6 + ggplot2::geom_smooth(data = H_est, ggplot2::aes(x = .data$x, y = ((.data$y * (IQR_H)) + q1_H), color = factor("Smoothed Estimate H"))
                                      , method = "loess", se = FALSE, span = 0.3, linewidth = 1) +
        ggplot2::scale_y_continuous(name = "Time Series", limits = range_H,
                                    sec.axis = ggplot2::sec_axis(transform = function(x){(x - q1_H) / (IQR_H)}, name = "Estimator"))
    }

    if("LFD_Est" %in% input$checkbox_group_TS){
      p6 <- p6 + ggplot2::geom_line(data = LFD_est, ggplot2::aes(x = .data$x1, y = ((.data$y1 * (IQR_L)) + q1_L), color = factor("Raw Estimate LFD")), linewidth = 1) +
        ggplot2::scale_y_continuous(name = "Time Series", limits = range_L,
                                    sec.axis = ggplot2::sec_axis(transform = function(x){(x - q1_L) / (IQR_L)}, name = "Estimator"))

    }

    if("LFD_Smooth_Est" %in% input$checkbox_group_TS){
      p6 <- p6 + ggplot2::geom_smooth(data = LFD_est, ggplot2::aes(x = .data$x1, y = ((.data$y1 * (IQR_L)) + q1_L), color = factor("Smoothed Estimate LFD"))
                                      , method = "loess", se = FALSE, span = 0.3, linewidth = 1) +
        ggplot2::scale_y_continuous(name = "Time Series", limits = range_L,
                                    sec.axis = ggplot2::sec_axis(transform = function(x){(x - q1_L) / (IQR_L)}, name = "Estimator"))
    }



    if (!is.null(input$min_TS) && input$min_TS){

      X.minimum <- min(InputTS[,2])
      t.X.minimum <- ((InputTS[,1])[which(InputTS[,2] == X.minimum)])
      min_points_df <- data.frame(t = t.X.minimum, x = rep(X.minimum, length(t.X.minimum)))

      p6 <- p6 + ggplot2::geom_point(data = min_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5)+
        ggplot2::geom_text(data = min_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Min = (",round(t, 2),",",round(x, 2),")")), vjust = 1.5, color = "red")
    }

    if (!is.null(input$max_TS) && input$max_TS){

      X.maximum <- max(InputTS[,2])
      t.X.maximum <- ((InputTS[,1])[which(InputTS[,2] == X.maximum)])
      max_points_df <- data.frame(t = t.X.maximum, x = rep(X.maximum, length(t.X.maximum)))

      p6 <- p6 + ggplot2::geom_point(data = max_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5)+
        ggplot2::geom_text(data = max_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Max=(",round(t, 2),",",round(x, 2),")")),vjust = -0.5, color = "red")
    }


    if(!is.null(input$increasing_TS) && input$increasing_TS){
      t <- InputTS[,1]
      x <- InputTS[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] > x[i-1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] > x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p6 <- p6 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "orange", linewidth = 1)

      }



    }

    if (!is.null(input$decreasing_TS) && input$decreasing_TS){
      t <- InputTS[,1]
      x <- InputTS[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] < x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] < x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }


      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p6 <- p6 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "brown", linewidth = 1)

      }

    }

    if (!is.null(input$sm_TS) && (input$sm_TS) && ("greater" %in% input$level_TS)){

      colnames(InputTS) <- c("x","y")
      N <- input$N_TS
      A <- input$A_TS

      ag_df <- stats::aggregate(InputTS[,2] ~ InputTS[,1], FUN = mean)
      t <- seq(ag_df[1, 1], ag_df[nrow(ag_df), 1], length.out = N + 1)
      int_X <- stats::approx(x = ag_df[,1], y = ag_df[,2], xout = t)$y
      diff <- ((ag_df[nrow(ag_df), 1] - ag_df[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 >= A) && (x2 < A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (x1 - A) / (x1 - x2))

        } else if ((x1 < A) && (x2 >= A)){

          seg$T_start[i] <- t2 - (diff * (A - x2) / (x1 - x2))
          seg$T_end[i] <- t2

        } else {

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }

      seg <- na.omit(seg)

      if (nrow(seg) > 0){

        p6 <- p6 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0), color = "red")
      }
    }

    p6

  })

  output$tsplot2 <- shiny::renderPlot({

    InputTS <- data_input()
    shiny::req(InputTS)
    InputTS <- na.omit(InputTS)
    InputTS <- InputTS[order(InputTS[,1]),]

    p7 <- ggplot2::ggplot(InputTS, ggplot2::aes(x = InputTS[,1], y = InputTS[,2])) + ggplot2::geom_line() +
      ggplot2::labs(y = "Time Series", x = "Time", color = "")


    if (!is.null(input$min_TS) && input$min_TS){

      X.minimum <- min(InputTS[,2])
      t.X.minimum <- ((InputTS[,1])[which(InputTS[,2] == X.minimum)])
      min_points_df <- data.frame(t = t.X.minimum, x = rep(X.minimum, length(t.X.minimum)))

      p7 <- p7 + ggplot2::geom_point(data = min_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5)+
        ggplot2::geom_text(data = min_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Min = (",round(t, 2),",",round(x, 2),")")), vjust = 1.5, color = "red")
    }

    if (!is.null(input$max_TS) && input$max_TS){

      X.maximum <- max(InputTS[,2])
      t.X.maximum <- ((InputTS[,1])[which(InputTS[,2] == X.maximum)])
      max_points_df <- data.frame(t = t.X.maximum, x = rep(X.maximum, length(t.X.maximum)))

      p7 <- p7 + ggplot2::geom_point(data = max_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5) +
        ggplot2::geom_text(data = max_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Max = (",round(t, 2),",",round(x, 2),")")), vjust = -0.5, color = "red")
    }


    if(!is.null(input$increasing_TS) && input$increasing_TS){
      t <- InputTS[,1]
      x <- InputTS[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] > x[i-1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] > x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p7 <- p7 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "orange", linewidth = 1)

      }



    }

    if (!is.null(input$decreasing_TS) && input$decreasing_TS){
      t <- InputTS[,1]
      x <- InputTS[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] < x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] < x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }


      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p7 <- p7 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "brown", linewidth = 1)

      }

    }

    if (!is.null(input$sm_TS) && (input$sm_TS) && ("greater" %in% input$level_TS)){

      colnames(InputTS) <- c("x", "y")
      N <- input$N_TS
      A <- input$A_TS

      ag_df <- stats::aggregate(InputTS[,2] ~ InputTS[,1], FUN = mean)
      t <- seq(ag_df[1, 1], ag_df[nrow(ag_df), 1], length.out=N + 1)
      int_X <- stats::approx(x = ag_df[,1], y = ag_df[,2], xout = t)$y
      diff <- ((ag_df[nrow(ag_df), 1] - ag_df[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 >= A) && (x2 < A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (x1 - A) / (x1 - x2))

        } else if ((x1 < A) && (x2 >= A)){

          seg$T_start[i] <- t2 - (diff * (A - x2) / (x1 - x2))
          seg$T_end[i] <- t2

        } else {

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }

      seg <- na.omit(seg)

      if (nrow(seg) > 0){

        p7 <- p7 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0), color = "red")
      }
    }


    if (!is.null(input$sm_TS) && (input$sm_TS) && ("lower" %in% input$level_TS)){

      colnames(InputTS) <- c("x", "y")

      N <- input$N_TS
      A <- input$A_TS

      ag_df <- stats::aggregate(InputTS[,2] ~ InputTS[,1], FUN = mean)
      t <- seq(ag_df[1, 1], ag_df[nrow(ag_df), 1], length.out = N + 1)
      int_X <- stats::approx(x = ag_df[,1], y = ag_df[,2], xout = t)$y
      diff <-((ag_df[nrow(ag_df), 1] - ag_df[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){

          S <- S + diff

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 <= A) && (x2 > A)){

          S <- S + (diff * (A - x1) / (x2 - x1))

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (A - x1) / (x2 - x1))

        } else if ((x1 > A) && (x2 <= A)){

          S <- S + (diff * (x2 - A) / (x2 - x1))

          seg$T_start[i] <- t2 - (diff * (x2 - A) / (x2 - x1))
          seg$T_end[i] <- t2

        } else {

          S <- S

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }


      seg <- na.omit(seg)

      if (nrow(seg) > 0){

        p7 <- p7 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0), color = "red")
      }

    }


    if (!is.null(input$ea_TS) && (input$ea_TS) && ("greater" %in% input$level_TS)){

      colnames(InputTS) <- c("x", "y")

      N <- input$N_TS
      A <- input$A_TS

      ag_df <- stats::aggregate(InputTS[,2] ~ InputTS[,1], FUN = mean)
      t <- seq(ag_df[1, 1], ag_df[nrow(ag_df), 1], length.out = N + 1)
      int_X <- stats::approx(x = ag_df[,1], y = ag_df[,2], xout = t)$y
      diff <- ((ag_df[nrow(ag_df), 1] - ag_df[1, 1]) / N)

      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          DF_Area <- rbind(DF_Area,data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area,data.frame(t = t2, X_t = x2))}

        } else if ((x1 >= A) && (x2 < A)){

          x_cross <- t1+(diff * (x1 - A) / (x1 - x2))
          DF_Area <- rbind(DF_Area,data.frame(t = t1, X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area,data.frame(t = rev(DF_Area$t), X_t = rep(A,nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 < A) && (x2 >= A)){


          x_cross <- t1 + (diff * (A - x2) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = ag_df[nrow(ag_df), 1], X_t = ag_df[nrow(ag_df), 2]))
        DF_Area <- rbind(DF_Area,data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)
      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area)> 0){

        p7 <- p7 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data = DF_Area, ggplot2::aes(x = t ,y = X_t, group = .data$G), fill = "lightblue")
      }

    }

    if (!is.null(input$ea_TS) && (input$ea_TS) && ("lower" %in% input$level_TS)){

      colnames(InputTS) <- c("x", "y")

      N <- input$N_TS
      A <- input$A_TS

      ag_df <- stats::aggregate(InputTS[,2] ~ InputTS[,1], FUN = mean)
      t <- seq(ag_df[1, 1], ag_df[nrow(ag_df), 1], length.out = N + 1)
      int_X <- stats::approx(x = ag_df[,1], y = ag_df[,2], xout = t)$y
      diff <- ((ag_df[nrow(ag_df), 1] - ag_df[1, 1]) / N)

      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){


          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 <= A) && (x2 > A)){

          x_cross <- t1 + (diff * (A - x1) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area,data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 > A) && (x2 <= A)){

          x_cross <- t1 + (diff * (x2 - A) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = ag_df[nrow(ag_df), 1], X_t = ag_df[nrow(ag_df), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)

      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area) > 0){

        p7 <- p7 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data = DF_Area, ggplot2::aes(x = t, y = X_t, group = .data$G), fill = "lightblue")
      }
    }

    p7

  })

  PR <- shiny::eventReactive(input$submit3, {

    H <- parse(text = input$func)
    Time <- tryCatch(eval(eval(parse(text = input$time))), error = function(e) NULL)
    J <- input$num

    func.H <- function(t) {
      eval(parse(text = H))
    }

    process <- GHBMP(Time, func.H, J)
    process <- process[order(process[,1]),]
    colnames(process) <- c("t1", "PP")
    process

  })


  output$functionPlot <- shiny::renderPlot({

    simPR <- PR()
    shiny::req(simPR)

    p<-ggplot2::ggplot(simPR, ggplot2::aes(x =t1, y =PP)) + ggplot2::geom_line() +
      ggplot2::labs(y = "X(t)", x = "t", color = "")

    N <- input$N_int
    Q <- input$Q
    L <- input$L

    H <- parse(text = input$func)
    func.H <- function(t) {
      eval(parse(text = H))
    }

    H_est <- Hurst(simPR, N, Q, L)
    colnames(H_est) <- c("x", "y")
    t1 <- simPR[,1]

    LFD_est <- LFD(simPR, N, Q, L)
    colnames(LFD_est) <- c("x1", "y1")

    if("H" %in% input$checkbox_group){

      H1 <- sapply(t1, func.H)
      data1 <- data.frame(t1, H1)

      p <- p + ggplot2::geom_line(data = data1, ggplot2::aes(x = .data$t1, y = .data$H1,color = factor("Theoretical H")), linewidth = 1)

    }

    if("Raw_Est_H" %in% input$checkbox_group){
      p <- p + ggplot2::geom_line(data = H_est, ggplot2::aes(x = .data$x, y = .data$y, color = factor("Raw Estimate H")), linewidth = 1)

    }

    if("Smooth_Est_H" %in% input$checkbox_group){
      p <- p + ggplot2::geom_smooth(data = H_est, ggplot2::aes(x = .data$x, y = .data$y, color = factor("Smoothed Estimate H"))
                                    , method = "loess", se = FALSE, span = 0.3, linewidth = 1)

    }

    if("LFD_Est" %in% input$checkbox_group){

      p <- p + ggplot2::geom_line(data = LFD_est, ggplot2::aes(x = .data$x1, y = .data$y1, color = factor("Raw Estimate LFD")), linewidth = 1)

    }

    if("LFD_Smooth_Est"%in% input$checkbox_group){

      p <- p + ggplot2::geom_smooth(data = LFD_est, ggplot2::aes(x = .data$x1, y = .data$y1, color = factor("Smoothed Estimate LFD"))
                                    , method = "loess", se = FALSE, span = 0.3, linewidth = 1)

    }

    if (!is.null(input$max3) && input$max3){

      X.maximum <- max(simPR[,2])
      t.X.maximum <- ((simPR[,1])[which(simPR[,2] == X.maximum)])
      max_points_df <- data.frame(t = t.X.maximum, x = rep(X.maximum, length(t.X.maximum)))

      p <- p+ggplot2::geom_point(data = max_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5)+
        ggplot2::geom_text(data = max_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Max = (",round(t, 2),",",round(x, 2),")")),vjust = -0.5,color = "red")
    }

    if (!is.null(input$min3) && input$min3){

      X.minimum <- min(simPR[,2])
      t.X.minimum <- ((simPR[,1])[which(simPR[,2] == X.minimum)])
      min_points_df <- data.frame(t = t.X.minimum, x = rep(X.minimum, length(t.X.minimum)))

      p <- p + ggplot2::geom_point(data = min_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5) +
        ggplot2::geom_text(data = min_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Min = (",round(t,2),",",round(x,2),")")), vjust = 1.5, color = "red")
    }

    if(!is.null(input$increasing3) && input$increasing3){
      t <- simPR[,1]
      x <- simPR[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] > x[i-1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] > x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p <- p + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "orange", linewidth = 1)

      }



    }

    if (!is.null(input$decreasing3) && input$decreasing3){
      t <- simPR[,1]
      x <- simPR[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] < x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] < x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }


      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p <- p + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "brown", linewidth = 1)

      }

    }


    if (!is.null(input$sm) && (input$sm) && ("greater" %in% input$level)){

      colnames(simPR) <- c("x", "y")
      N <- input$N_GHBMP
      A <- input$A_GHBMP

      t <- seq(simPR[1, 1], simPR[nrow(simPR), 1], length.out = N + 1)
      int_X <- stats::approx(simPR[,1], simPR[,2], xout = t)$y
      diff <- ((simPR[nrow(simPR), 1] - simPR[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 >= A) && (x2 < A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (x1 - A) / (x1 - x2))

        } else if ((x1 < A) && (x2 >= A)){

          seg$T_start[i] <- t2 - (diff * (A - x2) / (x1 - x2))
          seg$T_end[i] <- t2

        } else {

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }

      seg <- na.omit(seg)

      if (nrow(seg) > 0){

        p <- p + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0) ,color = "red")
      }
    }


    if (!is.null(input$sm) && (input$sm) && ("lower" %in% input$level)){

      colnames(simPR) <- c("x", "y")

      N <- input$N_GHBMP
      A <- input$A_GHBMP

      t <- seq(simPR[1, 1], simPR[nrow(simPR), 1], length.out = N + 1)
      int_X <- stats::approx(simPR[,1], simPR[,2], xout = t)$y
      diff<-((simPR[nrow(simPR), 1] - simPR[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){

          S <- S + diff

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 <= A) && (x2 > A)){

          S <- S + (diff * (A - x1) / (x2 - x1))

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (A - x1) / (x2 - x1))

        } else if ((x1 > A) && (x2 <= A)){

          S <- S + (diff * (x2 - A) / (x2 - x1))

          seg$T_start[i] <- t2 - (diff * (x2 - A) / (x2 - x1))
          seg$T_end[i] <- t2

        } else {

          S <- S

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }


      seg <- na.omit(seg)

      if (nrow(seg) > 0){

        p <- p + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0), color = "red")
      }

    }


    if (!is.null(input$ea) && (input$ea) && ("greater" %in% input$level)){

      colnames(simPR) <- c("x", "y")

      N <- input$N_GHBMP
      A <- input$A_GHBMP

      t <- seq(simPR[1, 1], simPR[nrow(simPR), 1], length.out = N + 1)
      int_X <- stats::approx(simPR[,1], simPR[,2], xout = t)$y
      diff <- ((simPR[nrow(simPR), 1] - simPR[1, 1]) / N)

      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 >= A) && (x2 < A)){

          x_cross <- t1 + (diff * (x1 - A) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 < A) && (x2 >= A)){


          x_cross <- t1 + (diff * (A - x2) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = simPR[nrow(simPR), 1],X_t = simPR[nrow(simPR), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)
      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area) > 0){

        p <- p + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data=DF_Area, ggplot2::aes(x = t, y = X_t, group = .data$G), fill = "lightblue")
      }

    }

    if (!is.null(input$ea) && (input$ea) && ("lower" %in% input$level)){

      colnames(simPR) <- c("x", "y")

      N <- input$N_GHBMP
      A <- input$A_GHBMP

      t <- seq(simPR[1, 1], simPR[nrow(simPR), 1], length.out = N + 1)
      int_X <- stats::approx(simPR[,1], simPR[,2], xout = t)$y
      diff<-((simPR[nrow(simPR), 1] - simPR[1, 1]) / N)


      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){


          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 <= A) && (x2 > A)){

          x_cross <- t1 + (diff * (A - x1) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area,data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 > A) && (x2 <= A)){

          x_cross <- t1 + (diff * (x2 - A) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = simPR[nrow(simPR), 1], X_t = simPR[nrow(simPR), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)

      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area) > 0){

        p <- p + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data = DF_Area,ggplot2::aes(x = t ,y = X_t, group = .data$G), fill = "lightblue")
      }
    }

    p

  })


  simBM <- shiny::eventReactive(input$submit_BM, {

    x1 <- input$x_BM
    t_start <- input$t_start_BM
    t_end <- input$t_end_BM
    N <- input$N_BM
    Bm(x_start = x1, t_start = t_start, t_end = t_end, N = N)

  })

  output$bmPlot <- shiny::renderPlot({
    simBMdf <- simBM()
    shiny::req(simBMdf)
    simBMdf <- simBMdf[order(simBMdf[,1]), ]

    p1 <- ggplot2::ggplot(simBMdf, ggplot2::aes(x = t, y = X)) +
      ggplot2::geom_line() +
      ggplot2::labs(y = "X(t)", x = "t")

    if (!is.null(input$max_BM) && input$max_BM){

      X.maximum<-max(simBMdf[,2])
      t.X.maximum<-((simBMdf[,1])[which(simBMdf[,2] == X.maximum)])
      max_points_df <- data.frame(t = t.X.maximum, x = rep(X.maximum, length(t.X.maximum)))

      p1 <- p1 + ggplot2::geom_point(data = max_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5) +
        ggplot2::geom_text(data = max_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Max = (",round(t, 2),",",round(x, 2),")")),vjust = -0.5, color = "red")
    }

    if (!is.null(input$min_BM) && input$min_BM){

      X.minimum <- min(simBMdf[,2])
      t.X.minimum <- ((simBMdf[,1])[which(simBMdf[,2] == X.minimum)])
      min_points_df <- data.frame(t = t.X.minimum, x = rep(X.minimum, length(t.X.minimum)))

      p1 <- p1+ggplot2::geom_point(data = min_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5) +
        ggplot2::geom_text(data = min_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Min = (",round(t, 2),",",round(x, 2),")")), vjust = 1.5, color = "red")
    }


    if(!is.null(input$increasing_BM) && input$increasing_BM){
      t <- simBMdf[,1]
      x <- simBMdf[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] > x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] > x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p1 <- p1 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "orange", linewidth = 1)

      }



    }

    if (!is.null(input$decreasing_BM) && input$decreasing_BM){
      t <- simBMdf[,1]
      x <- simBMdf[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] < x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] < x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }


      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p1 <- p1 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "brown", linewidth = 1)

      }

    }


    if (!is.null(input$sm_BM) && (input$sm_BM) && ("greater" %in% input$level_BM)){

      colnames(simBMdf) <- c("x", "y")
      N <- input$N_BMs
      A <- input$A_BM

      t <- seq(simBMdf[1, 1], simBMdf[nrow(simBMdf), 1], length.out = N + 1)
      int_X <- stats::approx(simBMdf[,1], simBMdf[,2], xout = t)$y
      diff<-((simBMdf[nrow(simBMdf), 1] - simBMdf[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 >= A) && (x2 < A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (x1 - A) / (x1 - x2))

        } else if ((x1 < A) && (x2 >= A)){

          seg$T_start[i] <- t2 - (diff * (A - x2) / (x1 - x2))
          seg$T_end[i] <- t2

        } else {

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }

      seg <- na.omit(seg)

      if (nrow(seg)>0){

        p1 <- p1 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0) , color = "red")
      }
    }


    if (!is.null(input$sm_BM) && (input$sm_BM) && ("lower" %in% input$level_BM)){

      colnames(simBMdf) <- c("x","y")

      N <- input$N_BMs
      A <- input$A_BM

      t <- seq(simBMdf[1, 1], simBMdf[nrow(simBMdf), 1], length.out = N + 1)
      int_X <- stats::approx(simBMdf[,1], simBMdf[,2], xout = t)$y
      diff <- ((simBMdf[nrow(simBMdf), 1] - simBMdf[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){

          S <- S + diff

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 <= A) && (x2 > A)){

          S <- S + (diff * (A - x1) / (x2 - x1))

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (A-x1)/(x2-x1))

        } else if ((x1 > A) && (x2 <= A)){

          S <- S + (diff * (x2 - A) / (x2 - x1))

          seg$T_start[i] <- t2 - (diff * (x2 - A) / (x2 - x1))
          seg$T_end[i] <- t2

        } else {

          S <- S

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }


      seg <- na.omit(seg)

      if (nrow(seg) > 0){

        p1 <- p1 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0), color = "red")
      }

    }


    if (!is.null(input$ea_BM) && (input$ea_BM) && ("greater" %in% input$level_BM)){

      colnames(simBMdf) <- c("x", "y")

      N <- input$N_BMs
      A <- input$A_BM

      t <- seq(simBMdf[1, 1], simBMdf[nrow(simBMdf), 1], length.out=N + 1)
      int_X <- stats::approx(simBMdf[,1], simBMdf[,2], xout = t)$y
      diff<-((simBMdf[nrow(simBMdf), 1] - simBMdf[1, 1]) / N)

      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 >= A) && (x2 < A)){

          x_cross <- t1 + (diff * (x1 - A) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = t1,X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 < A) && (x2 >= A)){


          x_cross <- t1 + (diff * (A - x2) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = simBMdf[nrow(simBMdf), 1], X_t = simBMdf[nrow(simBMdf), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)
      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area) > 0){

        p1 <- p1 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data = DF_Area, ggplot2::aes(x = t,y = X_t, group = .data$G), fill = "lightblue")
      }

    }

    if (!is.null(input$ea_BM) && (input$ea_BM) && ("lower" %in% input$level_BM)){

      colnames(simBMdf) <- c("x", "y")

      N <- input$N_BMs
      A <- input$A_BM

      t <- seq(simBMdf[1, 1], simBMdf[nrow(simBMdf), 1],length.out = N + 1)
      int_X <- stats::approx(simBMdf[,1],simBMdf[,2], xout = t)$y
      diff <- ((simBMdf[nrow(simBMdf), 1]-simBMdf[1, 1]) / N)


      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){


          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i==N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 <= A) && (x2 > A)){

          x_cross <- t1 + (diff * (A - x1) / (x2 - x1))
          DF_Area <- rbind(DF_Area,data.frame(t = t1, X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0) ,X_t = numeric(0))

        } else if ((x1 > A) && (x2 <= A)){

          x_cross <- t1 + (diff * (x2 - A) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = simBMdf[nrow(simBMdf), 1], X_t = simBMdf[nrow(simBMdf), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)

      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area) > 0){

        p1 <- p1 + ggplot2::geom_hline(yintercept = A,color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data = DF_Area, ggplot2::aes(x = t,y = X_t, group = .data$G), fill = "lightblue")
      }
    }

    p1
  })

  #Brownian Bridge

  simBB <- shiny::eventReactive(input$submit_BB, {

    x_end <- input$xend_BB
    t_end <- input$t_end_BB
    x_start <- input$xstart_BB
    N <- input$N_BB
    Bbridge(x_end = x_end, t_end = t_end, x_start = x_start, N = N)
  })

  output$bbPlot <- shiny::renderPlot({
    simBBdf <- simBB()
    shiny::req(simBBdf)
    simBBdf <- simBBdf[order(simBBdf[,1]), ]

    p2 <- ggplot2::ggplot(simBBdf, ggplot2::aes(x = t, y = X)) +
      ggplot2::geom_line() +
      ggplot2::labs(y = "X(t)", x = "t")

    if (!is.null(input$max_BB) && input$max_BB){

      X.maximum <- max(simBBdf[,2])
      t.X.maximum <- ((simBBdf[,1])[which(simBBdf[,2] == X.maximum)])
      max_points_df <- data.frame(t = t.X.maximum, x = rep(X.maximum, length(t.X.maximum)))

      p2 <- p2+ggplot2::geom_point(data = max_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5)+
        ggplot2::geom_text(data = max_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Max = (",round(t, 2),",",round(x, 2),")")), vjust = -0.5, color = "red")
    }

    if (!is.null(input$min_BB) && input$min_BB){

      X.minimum <- min(simBBdf[,2])
      t.X.minimum <- ((simBBdf[,1])[which(simBBdf[,2] == X.minimum)])
      min_points_df <- data.frame(t = t.X.minimum, x = rep(X.minimum, length(t.X.minimum)))

      p2 <- p2 + ggplot2::geom_point(data = min_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5) +
        ggplot2::geom_text(data = min_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Min = (",round(t, 2),",",round(x, 2),")")), vjust = 1.5, color = "red")
    }


    if(!is.null(input$increasing_BB) && input$increasing_BB){
      t <- simBBdf[,1]
      x <- simBBdf[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] > x[i-1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] > x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p2 <- p2 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "orange", linewidth = 1)

      }



    }

    if (!is.null(input$decreasing_BB) && input$decreasing_BB){
      t <- simBBdf[,1]
      x <- simBBdf[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] < x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] < x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }


      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p2 <- p2 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "brown", linewidth = 1)

      }

    }



    if (!is.null(input$sm_BB) && (input$sm_BB) && ("greater" %in% input$level_BB)){

      colnames(simBBdf) <- c("x","y")
      N <- input$N_BBs
      A <- input$A_BB

      t <- seq(simBBdf[1, 1], simBBdf[nrow(simBBdf), 1], length.out = N + 1)
      int_X <- stats::approx(simBBdf[,1], simBBdf[,2], xout = t)$y
      diff <- ((simBBdf[nrow(simBBdf), 1] - simBBdf[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 >= A) && (x2 < A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (x1 - A) / (x1 - x2))

        } else if ((x1 < A) && (x2 >= A)){

          seg$T_start[i] <- t2 - (diff * (A - x2) / (x1 - x2))
          seg$T_end[i] <- t2

        } else {

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }

      seg <- na.omit(seg)

      if (nrow(seg) > 0){

        p2 <- p2 + ggplot2::geom_hline(yintercept = A, color="blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0), color = "red")
      }
    }


    if (!is.null(input$sm_BB) && (input$sm_BB) && ("lower" %in% input$level_BB)){

      colnames(simBBdf) <- c("x", "y")

      N <- input$N_BBs
      A <- input$A_BB

      t <- seq(simBBdf[1, 1], simBBdf[nrow(simBBdf), 1], length.out = N + 1)
      int_X <- stats::approx(simBBdf[,1], simBBdf[,2], xout = t)$y
      diff <- ((simBBdf[nrow(simBBdf), 1] - simBBdf[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){

          S <- S + diff

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 <= A) && (x2 > A)){

          S <- S + (diff * (A - x1) / (x2 - x1))

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (A - x1) / (x2 - x1))

        } else if ((x1 > A) && (x2 <= A)){

          S <- S + (diff * (x2 - A) / (x2 - x1))

          seg$T_start[i] <- t2 - (diff * (x2 - A) / (x2 - x1))
          seg$T_end[i] <- t2

        } else {

          S <- S

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }


      seg <- na.omit(seg)

      if (nrow(seg)>0){

        p2 <- p2 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0), color = "red")
      }

    }


    if (!is.null(input$ea_BB) && (input$ea_BB) && ("greater" %in% input$level_BB)){

      colnames(simBBdf) <- c("x", "y")

      N <- input$N_BBs
      A <- input$A_BB

      t <- seq(simBBdf[1, 1], simBBdf[nrow(simBBdf), 1], length.out = N + 1)
      int_X <- stats::approx(simBBdf[,1], simBBdf[,2], xout = t)$y
      diff <- ((simBBdf[nrow(simBBdf) ,1] - simBBdf[1, 1]) / N)

      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area,data.frame(t = t2, X_t = x2))}

        } else if ((x1 >= A) && (x2 < A)){

          x_cross <- t1 + (diff * (x1 - A) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 < A) && (x2 >= A)){


          x_cross <- t1 + (diff * (A - x2) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = simBBdf[nrow(simBBdf), 1], X_t = simBBdf[nrow(simBBdf), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)
      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area) > 0){

        p2 <- p2 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data = DF_Area, ggplot2::aes(x = t, y = X_t, group = .data$G), fill = "lightblue")
      }

    }

    if (!is.null(input$ea_BB) && (input$ea_BB) && ("lower" %in% input$level_BB)){

      colnames(simBBdf) <- c("x", "y")

      N <- input$N_BBs
      A <- input$A_BB

      t <- seq(simBBdf[1, 1], simBBdf[nrow(simBBdf), 1], length.out = N + 1)
      int_X <- stats::approx(simBBdf[,1], simBBdf[,2], xout = t)$y
      diff <- ((simBBdf[nrow(simBBdf), 1] - simBBdf[1, 1]) / N)


      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){


          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 <= A) && (x2 > A)){

          x_cross <- t1 + (diff * (A - x1) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 > A) && (x2 <= A)){

          x_cross <- t1 + (diff * (x2 - A) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = simBBdf[nrow(simBBdf), 1], X_t = simBBdf[nrow(simBBdf), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)

      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area) > 0){

        p2 <- p2 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data = DF_Area, ggplot2::aes(x = t, y = X_t, group = .data$G), fill = "lightblue")
      }
    }
    p2
  })

  #Fractional Brownian Motion

  simFBM <- shiny::eventReactive(input$submit_FBM, {

    H <- input$H_FBM
    x_start <- input$x_FBM
    t_start <- input$t_start_FBM
    t_end <- input$t_end_FBM
    N <- input$N_FBM
    FBm(H = H, x_start = x_start, t_start = t_start, t_end = t_end, N = N)
  })

  output$fbmPlot <- shiny::renderPlot({

    simFBMdf <- simFBM()
    shiny::req(simFBMdf)
    simFBMdf <- simFBMdf[order(simFBMdf[,1]),]

    p3 <- ggplot2::ggplot(simFBMdf, ggplot2::aes(x = t, y = X)) +
      ggplot2::geom_line() +
      ggplot2::labs(y = "X(t)", x = "t")

    if (!is.null(input$max_FBM) && input$max_FBM){

      X.maximum <- max(simFBMdf[,2])
      t.X.maximum <- ((simFBMdf[,1])[which(simFBMdf[,2] == X.maximum)])
      max_points_df <- data.frame(t = t.X.maximum, x = rep(X.maximum, length(t.X.maximum)))

      p3 <- p3+ggplot2::geom_point(data = max_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5) +
        ggplot2::geom_text(data = max_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Max = (",round(t, 2),",",round(x, 2),")")), vjust = -0.5, color = "red")
    }

    if (!is.null(input$min_FBM) && input$min_FBM){

      X.minimum <- min(simFBMdf[,2])
      t.X.minimum <- ((simFBMdf[,1])[which(simFBMdf[,2] == X.minimum)])
      min_points_df <- data.frame(t = t.X.minimum, x = rep(X.minimum, length(t.X.minimum)))

      p3 <- p3 + ggplot2::geom_point(data = min_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5) +
        ggplot2::geom_text(data = min_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Min = (",round(t, 2),",",round(x, 2),")")), vjust = 1.5, color = "red")
    }


    if(!is.null(input$increasing_FBM) && input$increasing_FBM){
      t <- simFBMdf[,1]
      x <- simFBMdf[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] > x[i-1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] > x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p3 <- p3 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "orange", linewidth = 1)

      }



    }

    if (!is.null(input$decreasing_FBM) && input$decreasing_FBM){

      t <- simFBMdf[,1]
      x <- simFBMdf[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] < x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] < x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }


      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p3 <- p3 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "brown", linewidth = 1)

      }

    }



    if (!is.null(input$sm_FBM) && (input$sm_FBM) && ("greater" %in% input$level_FBM)){

      colnames(simFBMdf) <- c("x", "y")
      N <- input$N_FBMs
      A <- input$A_FBM

      t <- seq(simFBMdf[1, 1], simFBMdf[nrow(simFBMdf), 1], length.out = N + 1)
      int_X <- stats::approx(simFBMdf[,1], simFBMdf[,2], xout = t)$y
      diff <- ((simFBMdf[nrow(simFBMdf), 1] - simFBMdf[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 >= A) && (x2 < A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (x1 - A) / (x1 - x2))

        } else if ((x1 < A) && (x2 >= A)){

          seg$T_start[i] <- t2 - (diff * (A - x2) / (x1 - x2))
          seg$T_end[i] <- t2

        } else {

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }

      seg <- na.omit(seg)

      if (nrow(seg) > 0){

        p3 <- p3 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0), color = "red")
      }
    }


    if (!is.null(input$sm_FBM) && (input$sm_FBM) && ("lower" %in% input$level_FBM)){

      colnames(simFBMdf) <- c("x", "y")

      N <- input$N_FBMs
      A <- input$A_FBM

      t <- seq(simFBMdf[1, 1], simFBMdf[nrow(simFBMdf), 1], length.out = N + 1)
      int_X <- stats::approx(simFBMdf[,1], simFBMdf[,2], xout = t)$y
      diff<-((simFBMdf[nrow(simFBMdf), 1] - simFBMdf[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){

          S <- S + diff

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 <= A) && (x2 > A)){

          S <- S + (diff * (A - x1) / (x2 - x1))

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (A - x1) / (x2 - x1))

        } else if ((x1 > A) && (x2 <= A)){

          S <- S + (diff * (x2 - A) / (x2 - x1))

          seg$T_start[i] <- t2 - (diff * (x2 - A) / (x2 - x1))
          seg$T_end[i] <- t2

        } else {

          S <- S

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }


      seg <- na.omit(seg)

      if (nrow(seg)>0){

        p3 <- p3 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0), color = "red")
      }

    }


    if (!is.null(input$ea_FBM) && (input$ea_FBM) && ("greater" %in% input$level_FBM)){

      colnames(simFBMdf) <- c("x", "y")

      N <- input$N_FBMs
      A <- input$A_FBM

      t <- seq(simFBMdf[1, 1], simFBMdf[nrow(simFBMdf), 1], length.out = N + 1)
      int_X <- stats::approx(simFBMdf[,1], simFBMdf[,2], xout = t)$y
      diff <- ((simFBMdf[nrow(simFBMdf), 1] - simFBMdf[1, 1]) / N)

      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 >= A) && (x2 < A)){

          x_cross <- t1 + (diff * (x1 - A) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 < A) && (x2 >= A)){


          x_cross <- t1 + (diff * (A - x2) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = simFBMdf[nrow(simFBMdf), 1], X_t = simFBMdf[nrow(simFBMdf), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)
      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area) > 0){

        p3 <- p3 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data = DF_Area, ggplot2::aes(x = t, y = X_t, group = .data$G), fill = "lightblue")
      }

    }

    if (!is.null(input$ea_FBM) && (input$ea_FBM) && ("lower" %in% input$level_FBM)){

      colnames(simFBMdf) <- c("x", "y")

      N <- input$N_FBMs
      A <- input$A_FBM

      t <- seq(simFBMdf[1, 1], simFBMdf[nrow(simFBMdf), 1], length.out = N + 1)
      int_X <- stats::approx(simFBMdf[,1], simFBMdf[,2], xout = t)$y
      diff<-((simFBMdf[nrow(simFBMdf), 1] - simFBMdf[1, 1]) / N)


      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){


          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 <= A) && (x2 > A)){

          x_cross <- t1 + (diff * (A - x1) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 > A) && (x2 <= A)){

          x_cross <- t1 + (diff * (x2 - A) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = simFBMdf[nrow(simFBMdf), 1], X_t = simFBMdf[nrow(simFBMdf), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)

      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area) > 0){

        p3 <- p3 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data = DF_Area, ggplot2::aes(x = t, y = X_t, group = .data$G), fill = "lightblue")
      }
    }

    p3
  })

  #Fractional Brownian bridge

  simFBB <- shiny::eventReactive(input$submit_FBB, {

    H <- input$H_FBB
    x_end <- input$xend_FBB
    t_end <- input$t_end_FBB
    x_start <- input$xstart_FBB
    N <- input$N_FBB
    FBbridge(H = H, x_end = x_end, t_end = t_end, x_start =x_start, N = N)
  })

  output$fbbPlot <- shiny::renderPlot({


    simFBBdf <- simFBB()
    shiny::req(simFBBdf)
    simFBBdf <- simFBBdf[order(simFBBdf[,1]), ]

    p4 <- ggplot2::ggplot(simFBBdf, ggplot2::aes(x = t, y = X)) +
      ggplot2::geom_line() +
      ggplot2::labs(y = "X(t)", x = "t")

    if (!is.null(input$max_FBB) && input$max_FBB){

      X.maximum <- max(simFBBdf[,2])
      t.X.maximum <- ((simFBBdf[,1])[which(simFBBdf[,2] == X.maximum)])
      max_points_df <- data.frame(t = t.X.maximum, x = rep(X.maximum, length(t.X.maximum)))

      p4 <- p4 + ggplot2::geom_point(data = max_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5) +
        ggplot2::geom_text(data = max_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Max = (",round(t, 2),",",round(x, 2),")")), vjust = -0.5, color = "red")
    }

    if (!is.null(input$min_FBB) && input$min_FBB){

      X.minimum <- min(simFBBdf[,2])
      t.X.minimum <- ((simFBBdf[,1])[which(simFBBdf[,2] == X.minimum)])
      min_points_df <- data.frame(t = t.X.minimum, x = rep(X.minimum, length(t.X.minimum)))

      p4 <- p4 + ggplot2::geom_point(data = min_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5) +
        ggplot2::geom_text(data = min_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Min = (",round(t, 2),",",round(x, 2),")")), vjust = 1.5, color = "red")
    }


    if(!is.null(input$increasing_FBB) && input$increasing_FBB){
      t <- simFBBdf[,1]
      x <- simFBBdf[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] > x[i-1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] > x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p4 <- p4 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "orange", linewidth = 1)

      }



    }

    if (!is.null(input$decreasing_FBB) && input$decreasing_FBB){

      t <- simFBBdf[,1]
      x <- simFBBdf[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] < x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] < x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }


      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p4 <- p4 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "brown", linewidth = 1)

      }

    }

    if (!is.null(input$sm_FBB) && (input$sm_FBB) && ("greater" %in% input$level_FBB)){

      colnames(simFBBdf) <- c("x", "y")
      N <- input$N_FBBs
      A <- input$A_FBB

      t <- seq(simFBBdf[1, 1], simFBBdf[nrow(simFBBdf), 1], length.out = N + 1)
      int_X <- stats::approx(simFBBdf[,1], simFBBdf[,2], xout = t)$y
      diff <- ((simFBBdf[nrow(simFBBdf),1] - simFBBdf[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 >= A) && (x2 < A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (x1 - A) / (x1 - x2))

        } else if ((x1 < A) && (x2 >= A)){

          seg$T_start[i] <- t2 - (diff * (A - x2) / (x1 - x2))
          seg$T_end[i] <- t2

        } else {

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }

      seg <- na.omit(seg)

      if (nrow(seg) > 0){

        p4 <- p4 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0), color = "red")
      }
    }


    if (!is.null(input$sm_FBB) && (input$sm_FBB) && ("lower" %in% input$level_FBB)){

      colnames(simFBBdf) <- c("x", "y")

      N <- input$N_FBBs
      A <- input$A_FBB

      t <- seq(simFBBdf[1, 1], simFBBdf[nrow(simFBBdf), 1], length.out = N + 1)
      int_X <- stats::approx(simFBBdf[,1], simFBBdf[,2], xout = t)$y
      diff <- ((simFBBdf[nrow(simFBBdf), 1] - simFBBdf[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){

          S <- S + diff

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 <= A) && (x2 > A)){

          S <- S + (diff * (A - x1) / (x2 - x1))

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (A - x1) / (x2 - x1))

        } else if ((x1 > A) && (x2 <= A)){

          S <- S + (diff * (x2 - A) / (x2 - x1))

          seg$T_start[i] <- t2 - (diff * (x2 - A) / (x2 - x1))
          seg$T_end[i] <- t2

        } else {

          S <- S

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }


      seg <- na.omit(seg)

      if (nrow(seg) > 0){

        p4 <- p4 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0), color = "red")
      }

    }


    if (!is.null(input$ea_FBB) && (input$ea_FBB) && ("greater" %in% input$level_FBB)){

      colnames(simFBBdf) <- c("x","y")

      N <- input$N_FBBs
      A <- input$A_FBB

      t <- seq(simFBBdf[1, 1], simFBBdf[nrow(simFBBdf), 1], length.out = N + 1)
      int_X <- stats::approx(simFBBdf[,1], simFBBdf[,2], xout = t)$y
      diff<-((simFBBdf[nrow(simFBBdf), 1] - simFBBdf[1, 1]) / N)

      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 >= A) && (x2 < A)){

          x_cross <- t1 + (diff * (x1 - A) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 < A) && (x2 >= A)){


          x_cross <- t1 + (diff * (A - x2) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = simFBBdf[nrow(simFBBdf), 1], X_t = simFBBdf[nrow(simFBBdf), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)
      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area) > 0){

        p4 <- p4 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data = DF_Area, ggplot2::aes(x = t, y = X_t, group = .data$G), fill = "lightblue")
      }

    }

    if (!is.null(input$ea_FBB) && (input$ea_FBB) && ("lower" %in% input$level_FBB)){

      colnames(simFBBdf) <- c("x", "y")

      N <- input$N_FBBs
      A <- input$A_FBB

      t <- seq(simFBBdf[1, 1], simFBBdf[nrow(simFBBdf), 1], length.out = N + 1)
      int_X <- stats::approx(simFBBdf[,1], simFBBdf[,2], xout = t)$y
      diff <- ((simFBBdf[nrow(simFBBdf), 1] - simFBBdf[1, 1]) / N)


      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){


          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 <= A) && (x2 > A)){

          x_cross <- t1 + (diff * (A - x1) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 > A) && (x2 <= A)){

          x_cross <- t1 + (diff * (x2 - A) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = simFBBdf[nrow(simFBBdf), 1], X_t = simFBBdf[nrow(simFBBdf), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)

      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area)>0){

        p4 <- p4 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data = DF_Area, ggplot2::aes(x = t, y = X_t, group = .data$G), fill = "lightblue")
      }
    }

    p4
  })


  simFGN <- shiny::eventReactive(input$submit_FGN, {

    H <- input$H_FGN
    t_start <- input$t_start_FGN
    t_end <- input$t_end_FGN
    n <- input$N_FGN
    FGn(H = H, t_start = t_start, t_end = t_end, N = n)
  })

  output$fgnPlot <- shiny::renderPlot({


    simFGNdf <- simFGN()
    shiny::req(simFGNdf)
    simFGNdf <- simFGNdf[order(simFGNdf[,1]), ]

    p5 <- ggplot2::ggplot(simFGNdf,ggplot2::aes(x = t, y = X)) +
      ggplot2::geom_line() +
      ggplot2::labs(y = "X(t)", x = "t")

    if (!is.null(input$max_FGN) && input$max_FGN){

      X.maximum <- max(simFGNdf[,2])
      t.X.maximum <- ((simFGNdf[,1])[which(simFGNdf[,2] == X.maximum)])
      max_points_df <- data.frame(t = t.X.maximum, x = rep(X.maximum, length(t.X.maximum)))

      p5 <- p5 + ggplot2::geom_point(data = max_points_df, ggplot2::aes(x = t, y = x), color = "red", size = 1.5) +
        ggplot2::geom_text(data = max_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Max = (",round(t, 2),",",round(x, 2),")")), vjust = -0.5, color = "red")
    }

    if (!is.null(input$min_FGN) && input$min_FGN){

      X.minimum <- min(simFGNdf[,2])
      t.X.minimum <- ((simFGNdf[,1])[which(simFGNdf[,2] == X.minimum)])
      min_points_df <- data.frame(t = t.X.minimum, x = rep(X.minimum, length(t.X.minimum)))

      p5 <- p5 + ggplot2::geom_point(data = min_points_df, ggplot2::aes(x = t, y = x), color="red", size = 1.5) +
        ggplot2::geom_text(data = min_points_df,
                           ggplot2::aes(x = t, y = x, label = paste0("Min = (",round(t, 2),",",round(x, 2),")")), vjust = 1.5, color = "red")
    }


    if(!is.null(input$increasing_FGN) && input$increasing_FGN){
      t <- simFGNdf[,1]
      x <- simFGNdf[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] > x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] > x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p5 <- p5 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "orange", linewidth = 1)

      }

    }

    if (!is.null(input$decreasing_FGN) && input$decreasing_FGN){

      t <- simFGNdf[,1]
      x <- simFGNdf[,2]

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] < x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] < x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }


      if (length(streak) > 0) {

        time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

        max_int <- max(time_int)

        longest_streak <- streak[time_int == max_int]

        long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

        for (i in 1:length(longest_streak)) {
          id <- longest_streak[[i]][1]:longest_streak[[i]][2]
          df1 <- data.frame(t = t[id], x = x[id], group = i)
          long_streak_df <- rbind(long_streak_df, df1)
        } }

      else{
        long_streak_df <- NULL
      }


      if (!is.null(long_streak_df)) {
        p5 <- p5 + ggplot2::geom_line(data = long_streak_df, ggplot2::aes(x = t, y = x, group = group), color = "brown", linewidth = 1)

      }

    }


    if (!is.null(input$sm_FGN) && (input$sm_FGN) && ("greater" %in% input$level_FGN)){

      colnames(simFGNdf) <- c("x", "y")
      N <- input$N_FGNs
      A <- input$A_FGN

      t <- seq(simFGNdf[1, 1], simFGNdf[nrow(simFGNdf), 1], length.out = N + 1)
      int_X <- stats::approx(simFGNdf[,1], simFGNdf[,2], xout = t)$y
      diff <- ((simFGNdf[nrow(simFGNdf), 1] - simFGNdf[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 >= A) && (x2 < A)){

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (x1 - A) / (x1 - x2))

        } else if ((x1 < A) && (x2 >= A)){

          seg$T_start[i] <- t2 - (diff * (A - x2) / (x1 - x2))
          seg$T_end[i] <- t2

        } else {

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }

      seg <- na.omit(seg)

      if (nrow(seg)>0){

        p5 <- p5 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0), color = "red")
      }
    }


    if (!is.null(input$sm_FGN) && (input$sm_FGN) && ("lower" %in% input$level_FGN)){

      colnames(simFGNdf) <- c("x", "y")

      N <- input$N_FGNs
      A <- input$A_FGN

      t <- seq(simFGNdf[1, 1], simFGNdf[nrow(simFGNdf), 1], length.out = N + 1)
      int_X <- stats::approx(simFGNdf[,1], simFGNdf[,2], xout = t)$y
      diff<-((simFGNdf[nrow(simFGNdf), 1] - simFGNdf[1, 1]) / N)

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){

          S <- S + diff

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 <= A) && (x2 > A)){

          S <- S + (diff * (A - x1) / (x2 - x1))

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (A - x1) / (x2 - x1))

        } else if ((x1 > A) && (x2 <= A)){

          S <- S + (diff * (x2 - A) / (x2 - x1))

          seg$T_start[i] <- t2 - (diff * (x2 - A) / (x2 - x1))
          seg$T_end[i] <- t2

        } else {

          S <- S

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }


      seg <- na.omit(seg)

      if (nrow(seg) > 0){

        p5 <- p5 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_segment(data = seg, ggplot2::aes(x = T_start, xend = T_end, y = 0, yend = 0), color = "red")
      }

    }


    if (!is.null(input$ea_FGN) && (input$ea_FGN) && ("greater" %in% input$level_FGN)){

      colnames(simFGNdf) <- c("x", "y")

      N <- input$N_FGNs
      A <- input$A_FGN

      t <- seq(simFGNdf[1, 1], simFGNdf[nrow(simFGNdf), 1], length.out = N + 1)
      int_X <- stats::approx(simFGNdf[,1], simFGNdf[,2], xout = t)$y
      diff<-((simFGNdf[nrow(simFGNdf), 1] - simFGNdf[1, 1]) / N)

      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 >= A) && (x2 < A)){

          x_cross <- t1 + (diff * (x1 - A) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 < A) && (x2 >= A)){


          x_cross <- t1 + (diff * (A - x2) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = simFGNdf[nrow(simFGNdf), 1], X_t = simFGNdf[nrow(simFGNdf), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)
      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area)>0){

        p5 <- p5 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data = DF_Area, ggplot2::aes(x = t,y = X_t, group = .data$G), fill = "lightblue")
      }

    }

    if (!is.null(input$ea_FGN) && (input$ea_FGN) && ("lower" %in% input$level_FGN)){

      colnames(simFGNdf) <- c("x", "y")

      N <- input$N_FGNs
      A <- input$A_FGN

      t <- seq(simFGNdf[1, 1], simFGNdf[nrow(simFGNdf), 1], length.out = N + 1)
      int_X <- stats::approx(simFGNdf[,1], simFGNdf[,2], xout = t)$y
      diff <- ((simFGNdf[nrow(simFGNdf), 1] - simFGNdf[1, 1]) / N)


      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){


          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 <= A) && (x2 > A)){

          x_cross <- t1 + (diff * (A - x1) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1),
                           data.frame(t = x_cross, X_t = A),
                           data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 > A) && (x2 <= A)){

          x_cross <- t1 + (diff * (x2 - A) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = simBMdf[nrow(simBMdf), 1], X_t = simBMdf[nrow(simBMdf), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)

      DF_Area <- na.omit(DF_Area)

      if (!is.null(DF_Area) && nrow(DF_Area) > 0){

        p5 <- p5 + ggplot2::geom_hline(yintercept = A, color = "blue", linetype = "dashed") +
          ggplot2::geom_polygon(data = DF_Area, ggplot2::aes(x = t, y = X_t, group = .data$G), fill = "lightblue")
      }
    }

    p5
  })


}

shiny::shinyApp(ui = ui, server = server)

#' Shiny app to visualise and analyse processes
#'
#' @description
#' Launches a Shiny app to visualise and analyse realisations of Brownian motion (see \code{\link{Bm}}),
#' Brownian bridge (see \code{\link{Bbridge}}), fractional Brownian motion (see \code{\link{FBm}}),
#' fractional Brownian bridge (see \code{\link{FBbridge}}), fractional Gaussian noise (see \code{\link{FGn}}),
#' Gaussian Haar-based multifractional processes (see \code{\link{GHBMP}}) and user-provided time series data.
#'
#' @return An interactive Shiny app with the following user interface controls.
#'
#' **GHBMP simulation**
#' \describe{
#'   \item{\code{Hurst function}}{Input the Hurst function in terms of \code{t}.
#'   The default is set to \code{0.5 + 0*t}.}
#'   \item{\code{Time sequence}}{Input the time sequence which belongs to the interval \eqn{[0,1]}. The default is set to \code{seq(0, 1, by = (1/2)^10)}.}
#'   \item{\code{J}}{Input or select a positive integer. For large \code{J} could be rather time consuming. Default is set to 15.}}
#'**Hurst function and LFD estimation**
#'   \describe{
#'   \item{\code{Number of sub-intervals for estimation}}{Default is set to 100.}
#'   \item{\code{Q}}{Input or select an integer greater than or equal to 2. Default is set to 2.}
#'   \item{\code{L}}{Input or select an integer greater than or equal to 2. Default is set to 2.}
#'   \item{\code{Plot}}{Choose the required from: \code{Theoretical Hurst function}, \code{Raw estimate of Hurst function}, \code{Smoothed estimate of Hurst function}, \code{Raw estimate of Local Fractal Dimension}, \code{Smoothed estimate of Local Fractal Dimension}.}}
#' **Excursion set and area**
#'  \describe{
#'   \item{\code{Number of time steps}}{Input the number of steps the time interval needs to be split into.}
#'   \item{\code{Constant level}}{Input the constant level.}
#'   \item{\code{Compare to level}}{\code{Greater}, \code{Lower}.}
#'   \item{\code{Plot}}{Select: \code{Excursion set}, \code{Excursion area}.}}
#' **Longest streak**
#'  \describe{
#'   \item{\code{Longest streak to plot}}{Select: \code{Increasing}, \code{Decreasing}.}}
#' **Maximum and minimum**
#'  \describe{
#'   \item{\code{Plot}}{Select: \code{Maximum}, \code{Minimum}.}}
#' @importFrom shiny runApp
#' @importFrom shinycssloaders withSpinner
#' @details
#' For \code{Input time series}, provide a \code{.csv} file that has headers of its columns.
#' It must have two columns: the first column should contain the numeric time sequence \eqn{t},
#' and the second the corresponding time series values \eqn{X(t)}. To get reliable estimation
#' results for time series, it is recommended to use at least 500 time points.
#' Make sure there are no extra header rows or footnotes.
#'
#' @export shinyapp_sim
#'
#' @seealso \code{\link{Bm}}, \code{\link{FBm}}, \code{\link{FGn}}, \code{\link{Bbridge}}, \code{\link{FBbridge}},
#' \code{\link{GHBMP}}, \code{\link{Hurst}}, \code{\link{LFD}},  \code{\link{sojourn}},  \code{\link{exc_Area}}
#' \code{\link{long_streak}}, \code{\link{X_max}}, \code{\link{X_min}}
#'
#' @examples
#' if (interactive()) {
#'   shinyapp_sim()
#' }
#'
shinyapp_sim<- function() {
  appDir <- system.file("GHBMP_app", package = "Rmfrac")
  if (appDir == "") {
    stop("Could not find example directory. Try re-installing `Rmultifractional` package.", call. = FALSE)
  }

  runApp(appDir, display.mode = "normal")
}

#' Empirical covariance function
#'
#' @description
#' Computes the empirical covariance function of a process, for each pair of time points in the
#' time sequence using M realisations of the process.
#' @param theta Smoothing parameter.
#' @param X A data frame where the first column is the numeric time sequence and the remaining
#' columns are the values of each realisation of the process.
#' @param plot Logical: If TRUE, a 3D surface plot of the covariance function is plotted in interactive sessions.
#'
#' @return
#' An \eqn{m \times m} matrix, where \eqn{m} is the number of time points.
#' Each element represents the estimated value of covariance function for the
#' corresponding time points. Time points are arranged in ascending order.
#'
#' @details
#' The smoothing parameter \code{theta} can help to better visualise changes between
#' neighbour estimated values.
#'
#' @importFrom fields image.smooth
#' @importFrom plotly plot_ly layout
#' @export est_cov
#' @seealso \code{\link{cov_GHBMP}}
#' @examples
#' \donttest{
#' #Matrix of empirical covariance estimates of the GHBMP with Hurst function H.
#' t <- seq(0, 1, by = (1/2)^8)
#' H <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
#' #Only 5 realisations of GHBMP are used in this example to reduce the computational time.
#' X.t <- replicate(5, GHBMP(t, H), simplify = FALSE)
#' X <- do.call(rbind, lapply(X.t, function(df) df[, 2]))
#' Data <- data.frame(t, t(X))
#' cov.mat <- est_cov(Data, theta = 0.2, plot = TRUE)
#' cov.mat
#' }
est_cov<-function(X, theta = 0.1, plot = FALSE)
{

  if (!is.data.frame(X) | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0))) {
    stop("X must be a numeric data frame with time sequence given as the first column")
  }

  if (!is.numeric(theta)) {
    stop("theta must be numeric")
  } else if (!(theta > 0)){
    stop("theta must be positive")
  }

  if (!is.logical(plot)) {
    stop("Plot must have logical inputs either TRUE or FALSE")
  }

  X <- na.omit(X)
  X <- X[order(X[[1]]),]

  X.data <- t(X[,-1])
  t <- X[,1]

  m <- ncol(X.data)

  X_mean <- colMeans(X.data)

  C <- matrix(0, m, m)

  for (i in 1:m) {
    for (j in 1:i) {  # Compute only for j ≤ i (lower)
      C[i, j] <- mean(X.data[,i] * X.data[,j]) - X_mean[i] * X_mean[j]
    }
  }

  C[upper.tri(C)] <- t(C)[upper.tri(C)]

  if(!is.null(theta))
  {
    D <- mean(diff(t))
    Smooth_data <- image.smooth(C, theta = theta, dx = D,dy = D)
    C <- Smooth_data$z
  }

  if (isTRUE(plot) && interactive())
  {
    cov.fig <- plot_ly(x = ~t, y = ~t, z = ~C, type = 'surface', colorbar = list(title = "Covariance"))

    cov.fig <- layout(cov.fig, scene = list (xaxis = list(title = "t"), yaxis = list(title = "s"), zaxis = list(title = "Covariance")))

    print(cov.fig)
  }

  return(C)

}


#' Covariance of Gaussian Haar-based multifractional processes
#'
#' @description
#' Computes the theoretical covariance matrix of a Gaussian Haar-based multifractional process.
#'
#' @param t Time point or time sequence on the interval \eqn{[0,1]}.
#' @param H Hurst function \eqn{H(t)} which depends on \code{t}.
#' @param J Positive integer. For large J values could be rather time consuming. Default is set to 8.
#' @param theta Optional: Smoothing parameter.
#' @param plot Logical: If TRUE, a 3D surface plot of the covariance function is plotted in interactive sessions.
#' @param num.cores Number of cores to set up the clusters for parallel computing.
#'
#' @return An \eqn{m \times m} matrix, where \eqn{m} is the length of \code{t}.
#'
#' @details
#' To make it comparable with the empirical covariance function the same smoothing parameter
#' \code{theta} can be used if needed.
#'
#' @importFrom parallel makeCluster stopCluster
#' @importFrom parallelly availableCores
#' @importFrom doParallel registerDoParallel
#' @importFrom foreach foreach %dopar%
#' @importFrom plotly plot_ly layout
#' @importFrom fields image.smooth
#' @references Ayache, A., Olenko, A. and Samarakoon, N. (2026).
#' On construction, properties and simulation of Haar-based multifractional processes.
#' Mathematics and Computers in Simulation. 246:311-332. \doi{doi:10.1016/j.matcom.2026.01.033}.
#'
#' @export cov_GHBMP
#'
#' @seealso \code{\link{GHBMP}}, \code{\link{est_cov}}
#'
#' @examples
#' \donttest{
#' t <- seq(0, 1, by = 0.01)
#' H <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
#'
#' #Smoothed covariance function
#' cov_GHBMP(t, H, theta = 0.1, plot = TRUE)
#'
#' #Non-smoothed covariance function
#' cov_GHBMP(t, H, plot = TRUE)
#' }
cov_GHBMP<-function(t, H, J = 8, theta = NULL, plot = FALSE, num.cores = availableCores(omit = 1))
{

  if (!is.numeric(t)|!all(t >= 0 & t<= 1)) {
    stop("t must be a numeric sequence between 0 and 1")
  }

  H.t<-sapply(t, H)
  if (!is.numeric(H.t) | !all(H.t > 0 & H.t< 1)) {
    stop("H must be a function which returns a numeric list between 0 and 1")
  }

  if (!is.numeric(J)) {
    stop("J must be numeric")
  } else if (!(J > 0) | !(J %% 1 == 0)){
    stop("J must be a positive integer")
  }

  if (!is.logical(plot)) {
    stop("Plot must have logical inputs either TRUE or FALSE")
  }

  if (!is.numeric(num.cores)) {
    stop("num.cores must be numeric")
  } else if (!(num.cores %% 1 == 0) | !(num.cores > 0)) {
    stop("num.cores must be a positive integer")
  }

  h<-function(x, lambda)
  {
    ((lambda + 0.5)^(-1)) * ((x * (x>0))^(lambda + 0.5)-(2 * (((x-0.5) * ((x - 0.5) > 0))^(lambda + 0.5))) + (((x - 1) * ((x - 1) > 0)) ^ (lambda + 0.5)))
  }

  int<-function(j, k, t, H1){
    H11 <- H1(t)
    return((2^(-j * H11)) * h(((2^j) * t) - k, H11))
  }

  x1 <- 0:J
  x2 <- 0:(2^(J) - 1)

  X<-function(t,s){
    A <- outer(x1, x2, function(j, k){int(j, k, t, H)*int(j, k, s, H)})
    return(sum(A))
  }

  cl <- makeCluster(num.cores)
  registerDoParallel(cl)

  i <- NULL
  cov.mat <- foreach(i = 1:length(t), .combine = rbind) %dopar% {sapply(t, function(j) X(t[i], j))}

  stopCluster(cl)

  rownames(cov.mat) <- NULL

  if(!is.null(theta))
  {
    if (!is.numeric(theta)) {
      stop("theta must be numeric")
    } else if (!(theta > 0)){
      stop("theta must be positive")
    }

    D <- mean(diff(t))
    Smooth_data <- image.smooth(cov.mat, theta = theta, dx = D,dy = D)
    cov.mat <- Smooth_data$z
  }

  if (isTRUE(plot) && interactive())
  {

    cov.fig <- plot_ly(x = ~t, y = ~t, z = ~cov.mat, type = 'surface', colorbar = list(title = "Covariance"))

    cov.fig <- layout(cov.fig, scene = list (xaxis = list(title = "t"), yaxis = list(title = "s"), zaxis = list(title = "Covariance")))
    print(cov.fig)
  }

  return(cov.mat)

}

#est_cov

test_that("est_cov works", {
  expected_cov_est <- c(0.004097762, 0.042825573, 0.125753473, 0.197736999, 0.299328563,
    0.487519993, 0.695189740, 0.809483046, 0.896743403, 0.887964940, 0.781976102)
  expect_equal(diag(est_cov(Data_Bm)), expected_cov_est, tolerance = 1e-6)
})

test_that("est_cov fails if X is not a data frame", {
  expect_error(est_cov(c(0.2788,0.1365,0.5567)))
})

test_that("est_cov fails if X is a nonnumeric data frame", {
  expect_error(est_cov(data.frame(t = c("A", "B", "C"), x = c(0.2788, 0.1365, 0.5567))))
  expect_error(est_cov(data.frame(t = c(1, 2, 3), x = c(0.2788, "A", 0.5567))))
})

test_that("est_cov fails if X[,1] have negative values", {
  expect_error(est_cov(data.frame(t = c(1, -2, 3), x = c(0.2788, 0.1365, 0.5567))))
})

test_that("est_cov fails if theta is nonnumeric", {
  expect_error(est_cov(Data_Bm, theta = "A"))
})

test_that("est_cov fails for negative theta", {
  expect_error(est_cov(Data_Bm, theta = -10))
})

test_that("est_cov fails for non logical plot", {
  expect_error(est_cov(Data_Bm, plot = 1))
  expect_error(est_cov(Data_Bm, plot = "TRUE"))
})

#cov_GHBMP

test_that("cov_GHBMP fails for nonnumeric t", {
  expect_error(cov_GHBMP("a", H, J = 4))
})

test_that("cov_GHBMP fails for t not in [0,1]", {
  expect_error(cov_GHBMP(2:3, H, J = 4))
  expect_error(cov_GHBMP(-1, H, J = 4))
})

test_that("cov_GHBMP fails for empty t ", {
  expect_error(cov_GHBMP(c(), H, J = 4))
})

test_that("cov_GHBMP fails for H(t) not in (0,1)", {
  expect_error(cov_GHBMP(t, function(t) 2 + 0.5 * t, J = 4))
  expect_error(cov_GHBMP(t, function(t) -2 - 0.5 * t, J = 4))
})

test_that("cov_GHBMP fails for nonnumeric J ", {
  expect_error(cov_GHBMP(t, H, J = "a"))
})

test_that("cov_GHBMP fails for negative J", {
  expect_error(cov_GHBMP(t, H, J = -1))
})

test_that("cov_GHBMP fails for non integer J", {
  expect_error(cov_GHBMP(t, H, J = 1.5))
})

test_that("cov_GHBMP fails for nonnumeric num.cores", {
  expect_error(cov_GHBMP(t, H, J = 4, num.cores = "a"))
})

test_that("cov_GHBMP fails for negative num.cores", {
  expect_error(cov_GHBMP(t, H, J = 4, num.cores = -1))
})

test_that("cov_GHBMP fails for non integer num.cores", {
  expect_error(cov_GHBMP(t, H, J = 4, num.cores = 1.5))
})

test_that("cov_GHBMP fails if theta is nonnumeric", {
  expect_error(cov_GHBMP(t, H, J = 4, theta = "A"))
})

test_that("cov_GHBMP fails for negative theta", {
  expect_error(cov_GHBMP(t, H, J = 4, theta = -10))
})

test_that("cov_GHBMP fails for non logical plot", {
  expect_error(cov_GHBMP(t, H, J = 4, plot = 1))
  expect_error(cov_GHBMP(t, H, J = 4, plot = "TRUE"))
})

#' Estimated crossing times
#'
#' @description
#' Computes the estimated \eqn{t} value(s), in which a time series crosses a specific
#' constant level for the provided time interval or its sub-interval.
#'
#' @param X Data frame where the first column is a numeric time sequence \eqn{t}
#' and the second one is the values of the time series \eqn{X(t)}.
#' @param A Constant level as a numeric value.
#' @param subI Time sub-interval as a vector, where the lower bound is
#' the first element and the upper bound is the second. Optional: If provided estimated crossing
#' times of the sub-interval is returned, otherwise the whole time interval is considered.
#' @param plot Logical: If \code{TRUE}, the time series, the constant level and corresponding \eqn{t} values are plotted
#' in interactive sessions.
#' @param vline Logical: If \code{TRUE}, a vertical line is plotted at the crossing point(s).
#'
#' @return The estimated crossing times at a given level.
#' @importFrom ggplot2 ggplot geom_line geom_point geom_vline geom_hline labs aes
#' @importFrom rlang .data
#' @seealso \code{\link{cross_rate}}, \code{\link{cross_mean}}
#' @export cross_T
#'
#' @examples
#' t <- seq(0, 1, length = 100)
#' TS <- data.frame("t" = t, "X(t)" = rnorm(100))
#' cross_T(TS, 0.1, subI = c(0.2, 0.8), plot = TRUE, vline = TRUE)
#'
cross_T <- function(X, A, subI = NULL, plot = FALSE, vline = FALSE){

  if (!is.data.frame(X) | !ncol(X) == 2 | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0))){
    stop("X must be a numeric data frame with time sequence given as the first column")
  }

  if (!is.numeric(A)){
    stop("A must be numeric")
  }

  if (!is.logical(plot))
  {
    stop("plot should have logical inputs either TRUE or FALSE")
  }

  if (!is.logical(vline))
  {
    stop("vline should have logical inputs either TRUE or FALSE")
  }

  X <- na.omit(X)
  X <- X[order(X[,1]),]
  colnames(X) <- c("C1", "C2")

  if (is.null(subI)){

    X.I <- X
    t <- X.I[,1]
    x <- X.I[,2]

    cross_df <- data.frame(cross_t = numeric(0), cross_x = numeric(0))

    for (i in 1:(nrow(X.I) - 1)) {

      if ((x[i] < A && x[i+1] > A) || (x[i] > A && x[i+1] < A)) {
        cross_t <- t[i] + (A - x[i])*(t[i + 1 ]- t[i]) / (x[i + 1] - x[i])
        cross_df <- rbind(cross_df, data.frame(cross_t = cross_t, cross_x = A))
      }

      if (x[i + 1] == A) {
        if (i>1 && (i + 2) <= length(x)) {
          if ((x[i] < A && x[i + 2] > A) || (x[i] > A && x[i + 2] < A)) {
            cross_df <- rbind(cross_df, data.frame(cross_t = t[i + 1], cross_x = A))
          }
        }
      }
    }

    if (isTRUE(plot) && interactive()){
      p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
        geom_line() +
        geom_hline(yintercept = A, linetype = "dashed", color = "blue") +
        geom_point(data = cross_df, aes(x = .data$cross_t, y = .data$cross_x), color = "red", size = 1.5) +
        labs(x = "t",y = "X(t)")

      if (vline){
        p <- p + geom_vline(xintercept = cross_df$cross_t, linetype = "dashed", color = "blue")
      }

      print(p)
    }
    return(cross_df$cross_t)
  }
  else{

    if (!is.numeric(subI) | !is.vector(subI) | !length(subI) == 2 | !(all(subI[1] >= X[1,1] & subI[2] <= X[nrow(X),1]))){
      stop("subI must be a numeric vector")
    }

    Time <- X[,1]
    X.I <- subset(X, Time >= subI[1] & Time <= subI[2])

    t <- X.I[,1]
    x <- X.I[,2]

    cross_df <- data.frame(cross_t = numeric(0), cross_x = numeric(0))

    for (i in 1:(nrow(X.I) - 1)) {

      if ((x[i] < A && x[i + 1] > A) || (x[i] > A && x[i + 1] < A)) {
        cross_t <- t[i] + (A - x[i]) * (t[i + 1] - t[i]) / (x[i + 1] - x[i])
        cross_df <- rbind(cross_df, data.frame(cross_t = cross_t, cross_x = A))
      }

      if (x[i + 1] == A) {
        if (i > 1 && (i + 2) <= length(x)) {
          if ((x[i] < A && x[i + 2] > A) || (x[i] > A && x[i + 2] < A)) {
            cross_df <- rbind(cross_df, data.frame(cross_t = t[i + 1], cross_x = A))
          }
        }
      }
    }

    if (isTRUE(plot) && interactive()){
      p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
        geom_line() +
        geom_hline(yintercept = A, linetype = "dashed", color = "blue") +
        geom_point(data = cross_df, aes(x = .data$cross_t, y = .data$cross_x), color = "red", size = 2) +
        labs(x = "t",y = "X(t)")

      if (vline){
        p <- p + geom_vline(xintercept = cross_df$cross_t, linetype = "dashed", color = "blue")
      }

      print(p)
    }
    return(cross_df$cross_t)
  }

}


#' Crossing rate
#' @description
#' Computes the rate at which a time series crosses a specific
#' constant level for the provided time interval or its sub-interval.
#'
#' @param X Data frame where the first column is a numeric time sequence \eqn{t}
#' and the second one is the values of the time series \eqn{X(t)}.
#' @param A Constant level as a numeric value.
#' @param subI Time sub-interval as a vector, where the lower bound is
#' the first element and the upper bound is the second. Optional: If provided crossing rate
#' for the sub-interval is returned, otherwise the whole time interval is considered.
#' @param plot Logical: If \code{TRUE}, the time series, the constant level and crossing
#' points are plotted in interactive sessions.
#'
#' @return The crossing rate, which gives average number of crossings per time unit.
#' @export cross_rate
#' @importFrom ggplot2 ggplot geom_line geom_point geom_hline labs aes
#' @importFrom rlang .data
#' @seealso \code{\link{cross_T}}, \code{\link{cross_mean}}
#' @examples
#' t <- seq(0, 1, length = 100)
#' TS <- data.frame("t" = t, "X(t)" = rnorm(100))
#' cross_rate(TS, 0.1, subI = c(0.2, 0.8), plot = TRUE)
cross_rate <- function(X, A, subI = NULL, plot = FALSE){

  if (!is.data.frame(X) | !ncol(X) == 2 | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0))){
    stop("X must be a numeric data frame with time sequence given as the first column")
  }

  if (!is.numeric(A)){
    stop("A must be numeric")
  }

  if (!is.logical(plot))
  {
  stop("plot should have logical inputs either TRUE or FALSE")
  }

  X <- na.omit(X)
  X <- X[order(X[,1]), ]
  colnames(X) <- c("C1", "C2")

if (is.null(subI)){

  X.I <- X
  t <- X.I[,1]
  x <- X.I[,2]

  cross_df <- data.frame(cross_t = numeric(0), cross_x = numeric(0))

  for (i in 1:(nrow(X.I) - 1)) {

    if ((x[i] < A && x[i + 1] > A) || (x[i] > A && x[i + 1] < A)) {
      cross_t <- t[i] + (A - x[i]) * (t[i + 1] - t[i]) / (x[i + 1] - x[i])
      cross_df <- rbind(cross_df, data.frame(cross_t = cross_t, cross_x = A))
    }

    if (x[i+1] == A) {
      if (i > 1 && (i + 2) <= length(x)) {
        if ((x[i] < A && x[i + 2] > A) || (x[i] > A && x[i + 2] < A)) {
          cross_df <- rbind(cross_df, data.frame(cross_t = t[i + 1], cross_x = A))
        }
      }
    }
  }

  total_t <- X.I[nrow(X.I), 1] - X.I[1, 1]

  crossing_rate <- nrow(cross_df) / total_t

  if (isTRUE(plot) && interactive()){
    p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
      geom_line() +
      geom_hline(yintercept = A, linetype = "dashed", color = "blue") +
      geom_point(data = cross_df, aes(x = .data$cross_t, y = .data$cross_x), color = "red", size = 1.5) +
      labs(x = "t",y = "X(t)")

  print(p)
}
  return(crossing_rate)
}
  else{

    if (!is.numeric(subI) | !is.vector(subI) | !length(subI) == 2 | !(all(subI[1] >= X[1,1] & subI[2] <= X[nrow(X),1]))){
      stop("subI must be a numeric vector")
    }

    Time <- X[,1]
    X.I <- subset(X, Time >= subI[1] & Time <= subI[2])

    t <- X.I[,1]
    x <- X.I[,2]

    cross_df <- data.frame(cross_t = numeric(0), cross_x = numeric(0))

    for (i in 1:(nrow(X.I) - 1)) {

      if ((x[i] < A && x[i + 1] > A) || (x[i] > A && x[i + 1] < A)) {
        cross_t <- t[i] + (A - x[i]) * (t[i + 1] - t[i]) / (x[i + 1] - x[i])
        cross_df <- rbind(cross_df, data.frame(cross_t = cross_t, cross_x = A))
      }

      if (x[i+1] == A) {
        if (i > 1 && (i + 2) <= length(x)) {
          if ((x[i] < A && x[i + 2] > A) || (x[i] > A && x[i + 2] < A)) {
            cross_df <- rbind(cross_df, data.frame(cross_t = t[i + 1], cross_x = A))
          }
        }
      }
    }

    total_t <- X.I[nrow(X.I), 1] - X.I[1, 1]

    crossing_rate <- nrow(cross_df) / total_t

    if (isTRUE(plot) && interactive()){
      p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
        geom_line() +
        geom_hline(yintercept = A, linetype = "dashed", color = "blue") +
        geom_point(data = cross_df, aes(x = .data$cross_t, y = .data$cross_x), color = "red", size = 1.5) +
        labs(x = "t",y = "X(t)")

      print(p)
    }
    return(crossing_rate)
  }

}


#' Mean time between crossings
#' @description
#' Computes the mean duration between crossings of a time series
#' at a specified constant level for the provided time interval or its sub-interval.
#'
#' @param X Data frame where the first column is a numeric time sequence \eqn{t}
#' and the second one is the values of the time series \eqn{X(t)}.
#' @param A Constant level as a numeric value.
#' @param subI Time sub-interval is a vector, where the lower bound is
#' the first element and the upper bound is the second. Optional: If provided mean crossing
#' times for the sub-interval is returned, otherwise the whole time interval is considered.
#' @param plot Logical: If \code{TRUE}, the time series, the constant level and crossing
#' points are plotted in interactive sessions.
#'
#' @return The estimated mean time between crossings.
#' @export cross_mean
#' @importFrom ggplot2 ggplot geom_line geom_point geom_hline labs aes
#' @importFrom rlang .data
#' @seealso \code{\link{cross_T}}, \code{\link{cross_rate}}
#' @examples
#' t <- seq(0, 1, length = 100)
#' TS <- data.frame("t" = t, "X(t)" = rnorm(100))
#' cross_mean(TS, 0.1, subI = c(0.2, 0.8), plot = TRUE)
cross_mean <- function(X, A, subI = NULL, plot = FALSE){

  if (!is.data.frame(X) | !ncol(X) == 2 | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0))){
    stop("X must be a numeric data frame with time sequence given as the first column")
  }

  if (!is.numeric(A)){
    stop("A must be numeric")
  }

  if (!is.logical(plot))
  {
    stop("plot should have logical inputs either TRUE or FALSE")
  }

  X <- na.omit(X)
  X <- X[order(X[,1]), ]
  colnames(X) <- c("C1", "C2")

  if (is.null(subI)){

    X.I <- X
    t <- X.I[,1]
    x <- X.I[,2]

    cross_df <- data.frame(cross_t = numeric(0), cross_x = numeric(0))
    for (i in 1:(nrow(X.I) - 1)) {

      if ((x[i] < A && x[i + 1] > A) || (x[i] > A && x[i + 1] < A)) {
        cross_t <- t[i] + (A - x[i]) * (t[i+1] - t[i]) / (x[i+1] - x[i])
        cross_df <- rbind(cross_df, data.frame(cross_t = cross_t, cross_x = A))
      }

      if (x[i + 1] == A && i > 1 && i + 1 <= nrow(X.I)) {
        if ((x[i] < A && x[i + 2] > A) || (x[i] > A && x[i + 2] < A)) {
          cross_df <- rbind(cross_df, data.frame(cross_t = t[i + 1], cross_x = A))
        }
      }
    }

    if (nrow(cross_df) < 2) {

      message("Insufficient number of crossings to compute the mean time between crossings.\n")

      if (isTRUE(plot) && interactive()){
        p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
          geom_line() +
          geom_hline(yintercept = A, linetype = "dashed", color = "blue") +
          geom_point(data = cross_df, aes(x = .data$cross_t, y = .data$cross_x), color = "red", size = 2) +
          labs(x = "t",y = "X(t)")

        print(p)
      }

    } else {

      if (isTRUE(plot) && interactive()){
        p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
          geom_line() +
          geom_hline(yintercept = A, linetype = "dashed", color = "blue") +
          geom_point(data = cross_df, aes(x = .data$cross_t, y = .data$cross_x), color = "red", size = 2) +
          labs(x = "t",y = "X(t)")

        print(p)
      }

      time_diff <- diff(cross_df$cross_t)
      mean_t <- mean(time_diff)
      return(mean_t)
    }

  }
  else{

    if (!is.numeric(subI) | !is.vector(subI) | !length(subI) == 2 | !(all(subI[1] >= X[1,1] & subI[2] <= X[nrow(X),1]))){
      stop("subI must be a numeric vector")
    }

    Time <- X[,1]
    X.I <- subset(X, Time >= subI[1] & Time <= subI[2])

    t <- X.I[,1]
    x <- X.I[,2]

    cross_df <- data.frame(cross_t = numeric(0), cross_x = numeric(0))
    for (i in 1:(nrow(X.I) - 1)) {

      if ((x[i] < A && x[i + 1] > A) || (x[i] > A && x[i + 1] < A)) {
        cross_t <- t[i] + (A - x[i]) * (t[i + 1] - t[i]) / (x[i + 1] - x[i])
        cross_df <- rbind(cross_df, data.frame(cross_t = cross_t, cross_x = A))
      }

      if (x[i+1] == A && i > 1 && i + 1 <= nrow(X.I)) {
        if ((x[i] < A && x[i + 2] > A) || (x[i] > A && x[i + 2] < A)) {
          cross_df <- rbind(cross_df, data.frame(cross_t = t[i + 1], cross_x = A))
        }
      }
    }

    if (nrow(cross_df) < 2) {

      message("Insufficient number of crossings to compute the mean time between crossings.\n")

      if (isTRUE(plot) && interactive()){
        p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
          geom_line() +
          geom_hline(yintercept = A, linetype = "dashed", color = "blue") +
          geom_point(data = cross_df, aes(x = .data$cross_t, y = .data$cross_x), color = "red", size = 2) +
          labs(x = "t",y = "X(t)")

        print(p)
      }

      } else {

        if (isTRUE(plot) && interactive()){
        p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
          geom_line() +
          geom_hline(yintercept = A, linetype = "dashed", color = "blue") +
          geom_point(data = cross_df, aes(x = .data$cross_t, y = .data$cross_x), color = "red", size = 2) +
          labs(x = "t",y = "X(t)")

        print(p)
      }

      time_diff <- diff(cross_df$cross_t)
      mean_t <- mean(time_diff)
      return(mean_t)

    }

  }

}



#' Longest increasing/decreasing streak
#' @description
#' Computes the time span of the longest increasing or decreasing streak(s)
#' of a time series for the provided time interval or its sub-interval.
#'
#' @param X Data frame where the first column is a numeric time sequence \eqn{t}
#' and the second one is the values of the time series \eqn{X(t)}.
#' @param direction A character string which specifies the direction of
#' the streak: \code{"increasing"} or \code{"decreasing"}.
#' @param subI Time sub-interval is a vector, where the lower bound is
#' the first element and the upper bound is the second. Optional: If provided
#' longest streak(s) time span of the sub-interval is returned, otherwise the whole time interval is considered.
#' @param plot Logical: If \code{TRUE}, the time series and the longest streaks(s) of
#' increasing/decreasing is plotted in interactive sessions.
#'
#' @return A data frame with one row for each longest streak, containing the time span
#' and the corresponding values of \eqn{X(t)} at the streak endpoints.
#'
#' @export long_streak
#' @importFrom ggplot2 ggplot geom_line labs aes
#' @importFrom rlang .data
#' @importFrom stats aggregate
#' @importFrom utils head tail
#' @seealso \code{\link{mean_streak}}
#' @examples
#' t <- seq(0, 1, length = 100)
#' TS <- data.frame("t" = t,"X(t)" = rnorm(100))
#' long_streak(TS, direction = 'decreasing', subI = c(0.2, 0.8), plot = TRUE)
long_streak <- function(X, direction = 'increasing', subI = NULL, plot = FALSE){

  if (!is.data.frame(X) | !ncol(X) == 2 | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0))){
    stop("X must be a numeric data frame with time sequence given as the first column")
  }

  if (!is.logical(plot))
  {
    stop("plot should have logical inputs either TRUE or FALSE")
  }

  if (!direction %in% c("increasing", "decreasing")) {
    stop("streak should be 'increasing' or 'decreasing' ")
  }

  X <- na.omit(X)
  X <- X[order(X[,1]), ]
  colnames(X) <- c("C1", "C2")

  if (is.null(subI)){

    X.I <- X
    t <- X.I[,1]
    x <- X.I[,2]

    if(direction == 'increasing'){

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] > x[i-1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] > x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

    }

    if (direction == 'decreasing'){

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] < x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] < x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

    }

    if (length(streak) > 0) {

      time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

      max_int <- max(time_int)

      longest_streak <- streak[time_int == max_int]

      long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

      for (i in 1:length(longest_streak)) {
        id <- longest_streak[[i]][1]:longest_streak[[i]][2]
        df1 <- data.frame(t = t[id], x = x[id], group = i)
        long_streak_df <- rbind(long_streak_df, df1)
      }

      t_start <- aggregate(t ~ group, long_streak_df, head, 1)
      t_end <- aggregate(t ~ group, long_streak_df, tail, 1)
      X_start <- aggregate(x ~ group, long_streak_df, head, 1)
      X_end <- aggregate(x ~ group, long_streak_df, tail, 1)

      Return_DF <- data.frame(streak_no = t_start$group, t_start = t_start$t, t_end = t_end$t, X_start = X_start$x, X_end = X_end$x)
      row.names(Return_DF) <- NULL

      if (isTRUE(plot) && interactive()){
        p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
             geom_line() +
             geom_line(data = long_streak_df, aes(x = .data$t, y = .data$x, group = .data$group), color = "blue",linewidth = 1) +
             labs(x = "t",y = "X(t)")

        print(p)
      }

      return(Return_DF)

    }

    else{

      if (isTRUE(plot) && interactive()){

          p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
               geom_line() +
               labs(x = "t",y = "X(t)")

        print(p)
      }

      message(sprintf("No %s streaks", direction))

    }


  }
  else{

    if (!is.numeric(subI) | !is.vector(subI) | !length(subI) == 2 | !(all(subI[1] >= X[1, 1] & subI[2] <= X[nrow(X), 1]))){
      stop("subI must be a numeric vector")
    }

    Time <- X[,1]
    X.I <- subset(X, Time >= subI[1] & Time <= subI[2])

    t <- X.I[,1]
    x <- X.I[,2]

    if(direction == 'increasing'){

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] > x[i-1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] > x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

    }

    if (direction == 'decreasing'){

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] < x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] < x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

    }

    if (length(streak) > 0) {

      time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

      max_int <- max(time_int)

      longest_streak <- streak[time_int == max_int]

      long_streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

      for (i in 1:length(longest_streak)) {
        id <- longest_streak[[i]][1]:longest_streak[[i]][2]
        df1 <- data.frame(t = t[id], x = x[id], group = i)
        long_streak_df <- rbind(long_streak_df, df1)
      }

      t_start <- aggregate(t ~ group, long_streak_df, head, 1)
      t_end <- aggregate(t ~ group, long_streak_df, tail, 1)
      X_start <- aggregate(x ~ group, long_streak_df, head, 1)
      X_end <- aggregate(x ~ group, long_streak_df, tail, 1)

      Return_DF <- data.frame(streak_no = t_start$group, t_start = t_start$t, t_end = t_end$t, X_start = X_start$x, X_end = X_end$x)
      row.names(Return_DF) <- NULL

      if (isTRUE(plot) && interactive()){
        p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
          geom_line() +
          geom_line(data = long_streak_df, aes(x = .data$t, y = .data$x, group = .data$group), color = "blue",linewidth = 1) +
          labs(x = "t", y = "X(t)")

        print(p)
      }

      return(Return_DF)

    }

    else{

      if (isTRUE(plot) && interactive()){
        p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
          geom_line() +
          labs(x = "t", y = "X(t)")

        print(p)
      }

      message(sprintf("No %s streaks", direction))

    }

  }

}


#' Mean time span of increasing/decreasing streaks
#' @description
#' Computes the mean time span of the increasing/decreasing streaks
#' for the provided time interval or its sub-interval.
#'
#' @param X Data frame where the first column is a numeric time sequence \eqn{t}
#' and the second one is the values of the time series \eqn{X(t)}.
#' @param direction A character string which specifies the direction of
#' the streak: \code{"increasing"} or \code{"decreasing"}.
#' @param subI Time sub-interval is a vector, where the lower bound is
#' the first element and the upper bound is the second. Optional: If provided mean time span
#' of the sub-interval is returned, otherwise the whole time interval is considered.
#' @param plot Logical: If \code{TRUE}, the time series and the increasing/decreasing
#' streaks are plotted in interactive sessions.
#'
#' @return Mean time span of the increasing/decreasing streaks.
#' @export mean_streak
#' @importFrom ggplot2 ggplot geom_line labs aes
#' @importFrom rlang .data
#' @seealso \code{\link{long_streak}}
#' @examples
#' t <- seq(0, 1 ,length = 100)
#' TS <- data.frame("t" = t,"X(t)" = rnorm(100))
#' mean_streak(TS, direction = 'decreasing', subI = c(0.2, 0.8), plot = TRUE)
#'
mean_streak <- function(X, direction = 'increasing', subI = NULL, plot = FALSE){

  if (!is.data.frame(X) | !ncol(X) == 2 | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0))){
    stop("X must be a numeric data frame with time sequence given as the first column")
  }

  if (!is.logical(plot))
  {
    stop("plot should have logical inputs either TRUE or FALSE")
  }

  if (!direction %in% c("increasing", "decreasing")) {
    stop("streak should be 'increasing' or 'decreasing' ")
  }

  X <- na.omit(X)
  X <- X[order(X[,1]), ]
  colnames(X) <- c("C1", "C2")

  if (is.null(subI)){

    X.I <- X
    t <- X.I[,1]
    x <- X.I[,2]

    if(direction == 'increasing'){

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] > x[i-1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] > x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

    }

    if (direction == 'decreasing'){

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] < x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] < x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

    }

    if (length(streak) > 0) {

      time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

      mean_streak <- mean(time_int)

      streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

      for (i in 1:length(streak)) {
        id <- streak[[i]][1]:streak[[i]][2]
        df1 <- data.frame(t = t[id], x = x[id], group = i)
        streak_df <- rbind(streak_df, df1)
      }

      if (isTRUE(plot) && interactive()){
        p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
          geom_line() +
          geom_line(data = streak_df, aes(x = .data$t, y = .data$x, group = .data$group), color = "blue",linewidth = 1)+
          labs(x = "t", y = "X(t)")

        print(p)
      }

      return(mean_streak)

    }
    else{
      if (isTRUE(plot) && interactive()){
        p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
          geom_line() +
          labs(x = "t", y = "X(t)")

        print(p)
      }

      message(sprintf("No %s streaks", direction))
    }

  }

  else{

    if (!is.numeric(subI) | !is.vector(subI) | !length(subI) == 2 | !(all(subI[1] >= X[1, 1] & subI[2] <= X[nrow(X), 1]))){
      stop("subI must be a numeric vector")
    }

    Time <- X[,1]
    X.I <- subset(X, Time >= subI[1] & Time <= subI[2])

    t <- X.I[,1]
    x <- X.I[,2]

    if(direction == 'increasing'){

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] > x[i-1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] > x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

    }

    if (direction == 'decreasing'){

      streak <- list()
      start <- 1

      for (i in 2:length(x)) {
        if (x[i] < x[i - 1]) {
          next
        } else {
          if (i - 1 > start) {
            streak[[length(streak) + 1]] <- c(start, i - 1)
          }
          start <- i
        }
      }

      if (length(x) > 1 && x[length(x)] < x[length(x) - 1]) {
        streak[[length(streak) + 1]] <- c(start, length(x))
      }

    }

    if (length(streak) > 0) {

      time_int <- sapply(streak, function(n) t[n[2]] - t[n[1]])

      mean_streak <- mean(time_int)

      streak_df <- data.frame(t = numeric(0), x = numeric(0), group = integer(0))

      for (i in 1:length(streak)) {
        id <- streak[[i]][1]:streak[[i]][2]
        df1 <- data.frame(t = t[id], x = x[id], group = i)
        streak_df <- rbind(streak_df, df1)
      }

      if (isTRUE(plot) && interactive()){
        p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
          geom_line() +
          geom_line(data = streak_df, aes(x = .data$t, y = .data$x, group = .data$group), color = "blue",linewidth = 1)+
          labs(x = "t", y = "X(t)")

        print(p)
      }

      return(mean_streak)

    }
    else{
      if (isTRUE(plot) && interactive()){
        p <- ggplot(X.I, aes(x = .data$C1, y = .data$C2)) +
          geom_line() +
          labs(x = "t", y = "X(t)")

        print(p)
      }

      message(sprintf("No %s streaks", direction))
    }


  }
}


#' Relative strength index
#' @description
#' This function computes the Relative Strength Index (RSI) for a time series.
#'
#' @param X A numeric vector.
#' @param period Period length used for smoothing. Default is set to 14.
#' @param plot Logical: If \code{TRUE}, the time series and the
#' RSI are plotted (with \code{overbought} and
#' \code{oversold} levels) in the same window in interactive sessions.
#' @param overbought Horizontal line which indicates an overbought level in the RSI plot. Default is set to 70.
#' @param oversold Horizontal line which indicates an oversold level in the RSI plot. Default is set to 30.
#'
#' @return A list or vector of the RSI values.
#' @details To compute the RSI,
#'
#' \deqn{\text{RSI} = 100 \dfrac{\text{Average\_gain}}{\text{Average\_gain}+\text{Average\_loss}}}
#'
#' formula is used. Average gain and average loss are computed using the Wilders's smoothing method.
#' @references Wilder, J. W. (1978). New concepts in technical trading systems. Greensboro, NC.
#' @importFrom ggplot2 ggplot geom_line geom_hline labs aes facet_grid vars
#' @importFrom rlang .data
#' @importFrom zoo index
#'
#' @export RS_Index
#'
#' @examples
#' X <- c(74.44,74.19,74.25,73.65,74.37,74.73,75.15,75.46,75.88,76.78,
#'             75.81,76.53,75.11,76.28,76.68,76.08,76.53,76.11,76.42,75.58,
#'             75.44,75.46,74.98)
#' RS_Index(X, plot = TRUE)
#'
RS_Index <- function(X, period = 14, plot = FALSE, overbought = 70, oversold = 30)
{

  if(!is.numeric(X) | !is.vector(X)){
    stop("X must be a numeric vector")
  } else if (!(length(X) >= (period + 1))){
    stop("Not enough data to compute RSI for the given period")
  }

  if (!is.numeric(period)) {
    stop("period must be numeric")
  } else if (!(period %% 1 == 0) | !(period > 0)) {
    stop("period must be a positive integer")
  }

  if (!is.logical(plot))
  {
    stop("plot should have logical inputs either TRUE or FALSE")
  }

  if (!is.numeric(overbought) | !is.numeric(oversold)) {
    stop("overbought and oversold must be numeric")
  } else if ((overbought < 0 | overbought > 100) | (oversold < 0 | oversold > 100)) {
    stop("overbought and oversold should range from 0 to 100")
  }

  X <- na.omit(X)
  t <- 1:length(X)

  N <- length(X)
  diff <- diff(X)

  gain <- ifelse(diff > 0, diff, 0)
  loss <- ifelse(diff < 0, -diff, 0)

  gain_mean <- rep(NA, length(X))
  loss_mean <- rep(NA, length(X))

  gain_mean[period + 1] <- mean(gain[1:period], na.rm = TRUE)
  loss_mean[period + 1] <- mean(loss[1:period], na.rm = TRUE)

  for (i in (period + 2):N) {
    gain_mean[i] <- (gain_mean[i - 1] * (period - 1) + gain[i - 1]) / period
    loss_mean[i] <- (loss_mean[i - 1] * (period - 1) + loss[i - 1]) / period
  }

  RSI <- 100 * (gain_mean) / (gain_mean + loss_mean)

  RSI_df <- data.frame(t = rep(t, 2), value = c(X, RSI),group = rep(c("X", "RSI"), each = N))

  if (isTRUE(plot) && interactive())
  {
    df1 <- data.frame(group = "RSI",hline = c(overbought, oversold))

    p <- ggplot(RSI_df, aes(x = .data$t, y = .data$value)) +
      geom_line(na.rm = TRUE) +
      geom_hline(data = df1, aes(yintercept = .data$hline),color = c("blue","blue"), linetype = "dashed") +
      facet_wrap(~group, ncol = 1, scales = "free_y") +
      labs(title = "X and Relative Strength Index", y = "X(t)", x = "t")

    print(p)

  }

  return(RSI)

}

#cross_T

test_that("cross_T works", {
  expect_equal(cross_T(TS_Ana_2, 0.1), c(0.1316191, 0.3311314, 0.3890816, 0.6899009), tolerance = 1e-6)
  expect_equal(cross_T(TS_Ana_2, 0.1, subI = c(0.2,0.5)), c(0.3311314, 0.3890816), tolerance = 1e-6)
})

test_that("cross_T fails if X is not a data frame", {
  expect_error(cross_T(c(0.2788, 0.1365, 0.5567), 0.1))
})

test_that("cross_T fails if X is a data frame with more than or less than 2 columns", {
  expect_error(cross_T(data.frame(x = c(0.2788, 0.1365, 0.5567)), 0.1))
  expect_error(cross_T(data.frame(t = c(1, 2, 3), x = c(0.2788, 0.1365, 0.5567), y = c(0.2788,0.1365,0.5567)), 0.1))
})

test_that("cross_T fails if X is a nonnumeric data frame", {
  expect_error(cross_T(data.frame(t = c("A", "B", "C"), x = c(0.2788, 0.1365, 0.5567)), 0.1))
  expect_error(cross_T(data.frame(t = c(1, 2, 3), x = c(0.2788, "A", 0.5567)), 0.1))
})

test_that("cross_T fails if X[,1] have negative values", {
  expect_error(cross_T(data.frame(t = c(1, -2, 3), x = c(0.2788, 0.1365, 0.5567)), 0.1))
})

test_that("cross_T fails if A is nonnumeric", {
  expect_error(cross_T(TS_Ana_2, "A"))
})

test_that("cross_T fails if subI is not a numeric vector with 2 elemenTS_Ana_2", {
  expect_error(cross_T(TS_Ana_2, 0.1, subI = c(0,1, "A")))
  expect_error(cross_T(TS_Ana_2, 0.1, subI = c(0,1, 0.2, 0.3)))
})

test_that("cross_T fails if the second element of subI is larger than the first", {
  expect_error(cross_T(TS_Ana_2, 0.1, subI = c(0,5, 0.2)))
})

test_that("cross_T fails for non logical plot", {
  expect_error(cross_T(TS_Ana_2, 0.1, plot = 1))
  expect_error(cross_T(TS_Ana_2, 0.1, plot = "TRUE"))
})

test_that("cross_T fails for non logical vline", {
  expect_error(cross_T(TS_Ana_2, 0.1, vline = 1))
  expect_error(cross_T(TS_Ana_2, 0.1, vline = "TRUE"))
})

#cross_rate

test_that("cross_rate works", {
  expect_equal(cross_rate(TS_Ana_2, 0.1), 4, tolerance = 1e-0)
  expect_equal(cross_rate(TS_Ana_2, 0.1, subI = c(0.2,0.5)), 9, tolerance = 1e-0)
})

test_that("cross_rate fails if X is not a data frame", {
  expect_error(cross_rate(c(0.2788, 0.1365, 0.5567), 0.1))
})

test_that("cross_rate fails if X is a data frame with more than or less than 2 columns", {
  expect_error(cross_rate(data.frame(x = c(0.2788, 0.1365, 0.5567)), 0.1))
  expect_error(cross_rate(data.frame(t = c(1, 2, 3), x = c(0.2788, 0.1365, 0.5567), y = c(0.2788,0.1365,0.5567)), 0.1))
})

test_that("cross_rate fails if X is a nonnumeric data frame", {
  expect_error(cross_rate(data.frame(t = c("A", "B", "C"), x = c(0.2788, 0.1365, 0.5567)), 0.1))
  expect_error(cross_rate(data.frame(t = c(1, 2, 3), x = c(0.2788, "A", 0.5567)), 0.1))
})

test_that("cross_rate fails if X[,1] have negative values", {
  expect_error(cross_rate(data.frame(t = c(1, -2, 3), x = c(0.2788, 0.1365, 0.5567)), 0.1))
})

test_that("cross_rate fails if A is nonnumeric", {
  expect_error(cross_rate(TS_Ana_2, "A"))
})

test_that("cross_rate fails if subI is not a numeric vector with 2 elemenTS_Ana_2", {
  expect_error(cross_rate(TS_Ana_2, 0.1, subI = c(0,1, "A")))
  expect_error(cross_rate(TS_Ana_2, 0.1, subI = c(0,1, 0.2, 0.3)))
})

test_that("cross_rate fails if the second element of subI is larger than the first", {
  expect_error(cross_rate(TS_Ana_2, 0.1, subI = c(0,5, 0.2)))
})

test_that("cross_rate fails for non logical plot", {
  expect_error(cross_rate(TS_Ana_2, 0.1, plot = 1))
  expect_error(cross_rate(TS_Ana_2, 0.1, plot = "TRUE"))
})

#cross_mean

test_that("cross_mean works", {
  expect_equal(cross_mean(TS_Ana_2, 0.1), 0.186094, tolerance = 1e-6)
  expect_equal(cross_mean(TS_Ana_2, 0.1, subI = c(0.2,0.5)), 0.05795013, tolerance = 1e-6)
})

test_that("cross_mean fails if X is not a data frame", {
  expect_error(cross_mean(c(0.2788, 0.1365, 0.5567), 0.1))
})

test_that("cross_mean fails if X is a data frame with more than or less than 2 columns", {
  expect_error(cross_mean(data.frame(x = c(0.2788, 0.1365, 0.5567)), 0.1))
  expect_error(cross_mean(data.frame(t = c(1, 2, 3), x = c(0.2788, 0.1365, 0.5567), y = c(0.2788,0.1365,0.5567)), 0.1))
})

test_that("cross_mean fails if X is a nonnumeric data frame", {
  expect_error(cross_mean(data.frame(t = c("A", "B", "C"), x = c(0.2788, 0.1365, 0.5567)), 0.1))
  expect_error(cross_mean(data.frame(t = c(1, 2, 3), x = c(0.2788, "A", 0.5567)), 0.1))
})

test_that("cross_mean fails if X[,1] have negative values", {
  expect_error(cross_mean(data.frame(t = c(1, -2, 3), x = c(0.2788, 0.1365, 0.5567)), 0.1))
})

test_that("cross_mean fails if A is nonnumeric", {
  expect_error(cross_mean(TS_Ana_2, "A"))
})

test_that("cross_mean fails if subI is not a numeric vector with 2 elemenTS_Ana_2", {
  expect_error(cross_mean(TS_Ana_2, 0.1, subI = c(0,1, "A")))
  expect_error(cross_mean(TS_Ana_2, 0.1, subI = c(0,1, 0.2, 0.3)))
})

test_that("cross_mean fails if the second element of subI is larger than the first", {
  expect_error(cross_mean(TS_Ana_2, 0.1, subI = c(0,5, 0.2)))
})

test_that("cross_mean fails for non logical plot", {
  expect_error(cross_mean(TS_Ana_2, 0.1, plot = 1))
  expect_error(cross_mean(TS_Ana_2, 0.1, plot = "TRUE"))
})

#long_streak

test_that("long_streak works", {
  expected_long_streak <- data.frame(streak_no = 1, t_start = 0.2222222, t_end = 0.3333333, X_start = 1.558708, X_end = 0.07050839)
  expect_equal(long_streak(TS_Ana_2, direction = 'decreasing', subI = c(0.2,0.5)), expected_long_streak, tolerance = 1e-6)
})

test_that("long_streak fails if X is not a data frame", {
  expect_error(long_streak(c(0.2788, 0.1365, 0.5567)))
})

test_that("long_streak fails if X is a data frame with more than or less than 2 columns", {
  expect_error(long_streak(data.frame(x = c(0.2788, 0.1365, 0.5567))))
  expect_error(long_streak(data.frame(t = c(1, 2, 3), x = c(0.2788, 0.1365, 0.5567), y = c(0.2788,0.1365,0.5567))))
})

test_that("long_streak fails if X is a nonnumeric data frame", {
  expect_error(long_streak(data.frame(t = c("A", "B", "C"), x = c(0.2788, 0.1365, 0.5567))))
  expect_error(long_streak(data.frame(t = c(1, 2, 3), x = c(0.2788, "A", 0.5567))))
})

test_that("long_streak fails if X[,1] have negative values", {
  expect_error(long_streak(data.frame(t = c(1, -2, 3), x = c(0.2788, 0.1365, 0.5567))))
})

test_that("long_streak fails for irrelavant inpuTS_Ana_2 for level", {
  expect_error(long_streak(TS_Ana_2, direction = "increase"))
})

test_that("long_streak fails if subI is not a numeric vector with 2 elemenTS_Ana_2", {
  expect_error(long_streak(TS_Ana_2, subI = c(0,1, "A")))
  expect_error(long_streak(TS_Ana_2, subI = c(0,1, 0.2, 0.3)))
})

test_that("long_streak fails if the second element of subI is larger than the first", {
  expect_error(long_streak(TS_Ana_2, subI = c(0,5, 0.2)))
})

test_that("long_streak fails for non logical plot", {
  expect_error(long_streak(TS_Ana_2, plot = 1))
  expect_error(long_streak(TS_Ana_2, plot = "TRUE"))
})

#mean_streak

test_that("mean_streak works", {
  expect_equal(mean_streak(TS_Ana_2, direction = 'increasing'), 0.2222222, tolerance = 1e-6)
  expect_equal(mean_streak(TS_Ana_2, direction = 'increasing', subI = c(0.2,0.5)), 0.1111111, tolerance = 1e-6)
  expect_equal(mean_streak(TS_Ana_2, direction = 'decreasing'), 0.1666667, tolerance = 1e-6)
  expect_equal(mean_streak(TS_Ana_2, direction = 'decreasing', subI = c(0.2,0.5)), 0.1111111, tolerance = 1e-6)
})

test_that("mean_streak fails if X is not a data frame", {
  expect_error(mean_streak(c(0.2788, 0.1365, 0.5567)))
})

test_that("mean_streak fails if X is a data frame with more than or less than 2 columns", {
  expect_error(mean_streak(data.frame(x = c(0.2788, 0.1365, 0.5567))))
  expect_error(mean_streak(data.frame(t = c(1, 2, 3), x = c(0.2788, 0.1365, 0.5567), y = c(0.2788,0.1365,0.5567))))
})

test_that("mean_streak fails if X is a nonnumeric data frame", {
  expect_error(mean_streak(data.frame(t = c("A", "B", "C"), x = c(0.2788, 0.1365, 0.5567))))
  expect_error(mean_streak(data.frame(t = c(1, 2, 3), x = c(0.2788, "A", 0.5567))))
})

test_that("mean_streak fails if X[,1] have negative values", {
  expect_error(mean_streak(data.frame(t = c(1, -2, 3), x = c(0.2788, 0.1365, 0.5567))))
})

test_that("mean_streak fails for irrelavant inpuTS_Ana_2 for level", {
  expect_error(mean_streak(TS_Ana_2, direction = "increase"))
})

test_that("mean_streak fails if subI is not a numeric vector with 2 elemenTS_Ana_2", {
  expect_error(mean_streak(TS_Ana_2, subI = c(0,1, "A")))
  expect_error(mean_streak(TS_Ana_2, subI = c(0,1, 0.2, 0.3)))
})

test_that("mean_streak fails if the second element of subI is larger than the first", {
  expect_error(mean_streak(TS_Ana_2, subI = c(0,5, 0.2)))
})

test_that("mean_streak fails for non logical plot", {
  expect_error(mean_streak(TS_Ana_2, plot = 1))
  expect_error(mean_streak(TS_Ana_2, plot = "TRUE"))
})


#RS_Index

test_that("RS_Index works", {
  expect_equal(RS_Index(c(74.44, 74.19, 74.25, 73.65, 74.37, 74.73, 75.15, 75.46, 75.88, 76.78), period = 7),
               c(NA, NA, NA, NA, NA, NA, NA, 68.75000, 73.52025, 80.83427), tolerance = 1e-4)
})

test_that("RS_Index fails if X is not a numeric vector with enough data", {
  expect_error(RS_Index(c(74.44, "A", 74.25, "B", 74.37, 74.73, "C", 75.46, 75.88, 76.78), period = 7))
  expect_error(RS_Index(c(74.44, 74.19), period = 5))
})

test_that("RS_Index fails if period is nonnumeric", {
  expect_error(RS_Index(c(74.44, 74.19, 74.25, 73.65, 74.37, 74.73, 75.15, 75.46, 75.88, 76.78), period = "A"))
})

test_that("RS_Index fails for negative period", {
  expect_error(RS_Index(c(74.44, 74.19, 74.25, 73.65, 74.37, 74.73, 75.15, 75.46, 75.88, 76.78), period = -10))
})

test_that("RS_Index fails for non integer period", {
  expect_error(RS_Index(c(74.44, 74.19, 74.25, 73.65, 74.37, 74.73, 75.15, 75.46, 75.88, 76.78), period = 2.5))
})

test_that("mean_streak fails for non logical plot", {
  expect_error(RS_Index(c(74.44, 74.19, 74.25, 73.65, 74.37, 74.73, 75.15, 75.46, 75.88, 76.78), period = 7, plot = 1))
  expect_error(RS_Index(c(74.44, 74.19, 74.25, 73.65, 74.37, 74.73, 75.15, 75.46, 75.88, 76.78), period = 7, plot = "TRUE"))
})

test_that("RS_Index fails if overbought is nonnumeric and not in the correct range", {
  expect_error(RS_Index(c(74.44, 74.19, 74.25, 73.65, 74.37, 74.73, 75.15, 75.46, 75.88, 76.78), period = 7, overbought = TRUE))
  expect_error(RS_Index(c(74.44, 74.19, 74.25, 73.65, 74.37, 74.73, 75.15, 75.46, 75.88, 76.78), period = 7, overbought = 150))
})

test_that("RS_Index fails if oversold is nonnumeric and not in the correct range", {
  expect_error(RS_Index(c(74.44, 74.19, 74.25, 73.65, 74.37, 74.73, 75.15, 75.46, 75.88, 76.78), period = 7, oversold = TRUE))
  expect_error(RS_Index(c(74.44, 74.19, 74.25, 73.65, 74.37, 74.73, 75.15, 75.46, 75.88, 76.78), period = 7, oversold = 150))
})

#' @importFrom ggplot2 autoplot ggplot geom_line geom_smooth scale_color_manual labs aes
#' @importFrom rlang .data
#' @export
autoplot.mp <- function(object, ..., H = NULL, H_Est = TRUE, H_Smooth_Est = TRUE,LFD_Est = TRUE, LFD_Smooth_Est = TRUE, N = 100, Q = 2, L = 2)
{

  if (!is.logical(H_Est) | !is.logical(H_Smooth_Est) | !is.logical(LFD_Est) | !is.logical(LFD_Smooth_Est)) {
    stop("H_Est, H_Smooth_Est, LFD_Est and LFD_Smooth_Est should have logical inputs either TRUE or FALSE")
  }

  object <- object[order(object[,1]),]
  H_est <- Hurst(object, N, Q, L)
  colnames(H_est)  <- c("x","y")
  colnames(object) <- c("t1", "PP")
  t1<-object[,1]

  LFD_EST <- LFD(object, N, Q, L)
  colnames(LFD_EST) <- c("x1", "y1")

  p <- ggplot(object, aes(x =.data$t1, y =.data$PP)) + geom_line() +
       labs(y = "X(t)", x = "t",color = "")

  if(H_Est){
    p <- p + geom_line(data = H_est, aes(x =.data$x, y =.data$y, color = factor("Raw Estimate H")), linewidth = 1)

  }

  if(H_Smooth_Est){
    p <- p + geom_smooth(data = H_est, aes(x = .data$x, y = .data$y, color = factor("Smoothed Estimate H")),
                         method = "loess", se = FALSE, span = 0.3, linewidth = 1)

  }

  if (!is.null(H)){

    H1 <- sapply(t1, H)
    data1 <- data.frame(t1, H1) #Data for the theoretical Hurst function

    if (!is.numeric(H1) | !all(H1 >= 0 & H1 <= 1)) {
      stop("H must be a function which returns a numeric list between 0 and 1")
    }

    p <- p + geom_line(data = data1, aes(x = .data$t1, y = .data$H1, color = factor("Theoretical H")), linewidth = 1)


  }

  if(LFD_Est){

    p <- p + geom_line(data = LFD_EST, aes(x = .data$x1, y = .data$y1, color = factor("Raw Estimate LFD")), linewidth = 1)

  }

  if(LFD_Smooth_Est){

    p <- p + geom_smooth(data = LFD_EST, aes(x = .data$x1, y =.data$y1, color = factor("Smoothed Estimate LFD"))
                         , method = "loess", se = FALSE, span = 0.3, linewidth = 1)

  }

}


#' Plot Gaussian Haar-based multifractional processes with their
#' theoretical and estimated Hurst functions and local fractal dimension
#'
#' @description
#' Creates a plot of the Gaussian Haar-based multifractional process
#' simulated by using \code{\link{GHBMP}} with theoretical Hurst function (if provided),
#' Hurst function estimated using \code{\link{Hurst}}, the smoothed estimated
#' Hurst function and local fractal dimension estimated using \code{\link{LFD}}
#' and smoothed estimates of local fractal dimension.
#'
#' @param x Return from \code{\link{GHBMP}}. To get reliable results for simulated trajectories, it is recommended
#' to use at least 500 time points.
#' @param H Theoretical Hurst function. Optional: If provided, the theoretical Hurst function is plotted.
#' @param H_Est Logical: If \code{TRUE}, the Hurst function estimated by using \code{\link{Hurst}} is plotted.
#' @param H_Smooth_Est Logical: If \code{TRUE}, the smoothed estimated Hurst function is plotted.
#' The estimated Hurst function is smoothed using the LOESS method.
#' @param LFD_Est Logical: If \code{TRUE}, the local fractal dimension estimates are plotted.
#' @param LFD_Smooth_Est Logical: If \code{TRUE}, the smoothed estimates of local fractal dimension is plotted.
#' Smoothed using the LOESS method.
#' @param N Argument used for the estimation of Hurst functions and LFD. Number of sub-intervals on which the estimation is performed on. Default is set to 100 sub-intervals.
#' @param Q Argument used for the estimation of Hurst functions and LFD. Fixed integer greater than or equal to 2. Default is set to 2.
#' @param L Argument used for the estimation of Hurst functions and LFD. Fixed integer greater than or equal to 2. Default is set to 2.
#' @param ... Other arguments.
#' @return A ggplot object which is used to plot the multifractional process with theoretical, raw and smoothed estimates of Hurst function
#' and raw and smoothed estimates of local fractal dimension.
#' @exportS3Method Rmfrac::plot
#'
#' @importFrom graphics plot
#' @seealso \code{\link{GHBMP}}, \code{\link{Hurst}}, \code{\link{LFD}}
#'
#' @examples
#' \donttest{
#' #Simulation of the multifractional process and estimation of the Hurst function
#' t <- seq(0, 1, by = (1/2)^10)
#' H <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
#' X <- GHBMP(t, H)
#'
#' #Plot of process, theoretical Hurst function, estimated and smoothed Hurst and LFD estimates
#' plot(X, H = H)
#'
#' #Plot of process, estimated and smoothed Hurst and LFD estimates
#' plot(X)
#' }
plot.mp <- function(x, H = NULL, H_Est = TRUE, H_Smooth_Est = TRUE, LFD_Est = TRUE, LFD_Smooth_Est = TRUE, N = 100, Q = 2,L = 2, ...) {

  pp <- autoplot(x, H = H, H_Est = H_Est, H_Smooth_Est = H_Smooth_Est, LFD_Est = LFD_Est, LFD_Smooth_Est = LFD_Smooth_Est, N = N, Q = Q, L = L)

  if (interactive()) {
    print(pp)
  }

  invisible(pp)
}

#' Creates objects of class \code{H_LFD}
#' @description
#' For user provided time series creates objects of class \code{"H_LFD"} with the
#' Hurst function estimated using \code{\link{Hurst}}, local fractal dimension
#' estimated using \code{\link{LFD}} and smoothed estimated Hurst function and LFD added.
#'
#' @param X Data frame where the first column is a numeric time sequence \eqn{t}
#' and the second is the values of the time series \eqn{X(t)}.
#' To get reliable results, it is recommended to use at least 500 time points.
#' @param N Argument used for the estimation of Hurst functions and LFD. Number of sub-intervals on which the estimation is performed on. Default is set to 100 sub-intervals.
#' @param Q Argument used for the estimation of Hurst functions and LFD. Fixed integer greater than or equal to 2. Default is set to 2.
#' @param L Argument used for the estimation of Hurst functions and LFD. Fixed integer greater than or equal to 2. Default is set to 2.
#' @return The return from \code{\link{H_LFD}} is an object list of class \code{"H_LFD"} with the following components:
#' \describe{
#' \item{\code{Raw_Hurst_estimates}}{A data frame of where the first column is a time sequence and second column is estimated values of the Hurst function.}
#' \item{\code{Smoothed_Hurst_estimates}}{A data frame of where the first column is a time sequence and second column is smoothed estimates of the Hurst function. Smoothed using the LOESS method.}
#' \item{\code{Raw_LFD_estimates}}{A data frame of where the first column is a time sequence and second column is Local fractal dimension estimates.}
#' \item{\code{Smoothed_LFD_estimates}}{A data frame of where the first column is a time sequence and second column is smoothed estimates of Local fractal dimension. Smoothed using the LOESS method.}
#' \item{\code{Data}}{User provided time series.}}
#' @export H_LFD
#' @note
#' Since these are estimators of local characteristics, reliable results can only be obtained when a sufficiently large number of points is used.
#' @seealso \code{\link{plot.H_LFD}}, \code{\link{Hurst}}, \code{\link{LFD}}
#' @examples
#' TS <- data.frame("t" = seq(0, 1, length = 1000),"X(t)" = rnorm(1000))
#' Object <- H_LFD(TS)
#' #Plot of time series, estimated and smoothed Hurst and LFD estimates
#' plot(Object)
#'
H_LFD <- function(X, N = 100, Q = 2, L = 2){

  X<-na.omit(X)

  if (!is.data.frame(X) | !ncol(X) == 2 | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0))) {
    stop("X must be a numeric data frame with time sequence from 0 to 1 given as the first column")
  }

  X <- X[order(X[,1]),]

  H_est <- Hurst(X, N, Q, L)

  smoothed_H_est <- data.frame(time = H_est[,1], smoothed_Hurst = (loess(H_est[,2] ~ H_est[,1], data = H_est, span = 0.3))$fitted)

  smoothed_H_est[,2] <- pmax(pmin(smoothed_H_est[,2], 1), 0)

  LFD_est <- LFD(X, N, Q, L)

  LFD_smoothed_est <- data.frame(time = LFD_est[,1], smoothed_LFD = (loess(LFD_est[,2]~LFD_est[,1], data = LFD_est, span = 0.3))$fitted)

  structure(list(Raw_Hurst_estimates = H_est, Smoothed_Hurst_estimates = smoothed_H_est, Raw_LFD_estimates = LFD_est,Smoothed_LFD_estimates = LFD_smoothed_est, Data = X),class = "H_LFD")

}

#' @importFrom ggplot2 autoplot ggplot geom_line geom_smooth scale_color_manual labs aes sec_axis scale_y_continuous
#' @importFrom rlang .data
#' @importFrom stats IQR quantile
#' @export
autoplot.H_LFD<-function(object, ..., H_Est = TRUE, H_Smooth_Est = TRUE, LFD_Est = TRUE, LFD_Smooth_Est = TRUE){

  if (!is.logical(H_Est) | !is.logical(H_Smooth_Est) | !is.logical(LFD_Est) | !is.logical(LFD_Smooth_Est)) {
    stop("H_Est, H_Smooth_Est, LFD_Est and LFD_Smooth_Est should have logical inputs either TRUE or FALSE")
  }

  X <- object$Data
  H_est <- object$Raw_Hurst_estimates

  X <- na.omit(X)
  X <- X[order(X[,1]),]
  colnames(H_est) <- c("x", "y")
  colnames(X) <- c("t1", "PP")
  t1 <- X[,1]

  LFD_EST <- object$Raw_LFD_estimates
  colnames(LFD_EST) <- c("x1", "y1")

  IQR_H <- IQR(X[,2])
  range_H <- range(X[,2])
  q1_H <-quantile(X[,2], 0.25)

  IQR_L <- IQR(X[,2])
  range_L <- range(X[,2])
  q1_L <- quantile(X[,2], 0.25)

  p <- ggplot(X, aes(x = .data$t1, y = .data$PP)) + geom_line() +
       labs(y = "Time series", x = "Time", color = "")


  if(H_Est){
    p <- p + geom_line(data = H_est, aes(x = .data$x, y = ((.data$y * (IQR_H)) + q1_H), color = factor("Raw Estimate H")), linewidth = 1)+
             scale_y_continuous(name = "Time Series", limits = range_H,
                       sec.axis = sec_axis(transform = function(x){(x - q1_H) / (IQR_H)}, name = "Estimator"))


  }

  if(H_Smooth_Est){
    p <- p + geom_smooth(data = H_est, aes(x = .data$x,y=((.data$y*(IQR_H))+q1_H),color = factor("Smoothed Estimate H"))
                         ,method = "loess", se=FALSE, span = 0.3, linewidth = 1) +
             scale_y_continuous(name = "Time Series", limits = range_H,
                         sec.axis = sec_axis(transform = function(x){(x - q1_H) / (IQR_H)}, name = "Estimator"))

  }

  if(LFD_Est){

    p <- p + geom_line(data = LFD_EST, aes(x = .data$x1, y = ((.data$y1 * (IQR_L)) + q1_L),color = factor("Raw Estimate LFD")), linewidth = 1)+
             scale_y_continuous(name = "Time Series",limits = range_L,
                         sec.axis = sec_axis(transform = function(x){(x - q1_L) / (IQR_L)},name = "Estimator"))

  }

  if(LFD_Smooth_Est){

    p <- p + geom_smooth(data = LFD_EST, aes(x = .data$x1, y = ((.data$y1 * (IQR_L)) + q1_L), color = factor("Smoothed Estimate LFD"))
                         , method = "loess", se = FALSE, span = 0.3, linewidth = 1) +
             scale_y_continuous(name = "Time Series", limits = range_L,
                         sec.axis = sec_axis(transform = function(x){(x - q1_L) / (IQR_L)}, name = "Estimator"))
  }

}


#' Plot the estimated Hurst functions and local fractal dimension estimates
#' for objects of class \code{H_LFD}
#'
#' @description
#' Creates a plot of the user provided time series with the
#' Hurst function estimated using \code{\link{Hurst}}, the smoothed estimated
#' Hurst function and local fractal dimension estimated using \code{\link{LFD}}
#' and smoothed estimates of local fractal dimension for objects of class \code{"H_LFD"}.
#'
#' @param x Return from \code{\link{H_LFD}}.
#' @param H_Est Logical: If \code{TRUE}, the Hurst function estimated by using \code{\link{Hurst}} is plotted.
#' @param H_Smooth_Est Logical: If \code{TRUE}, the smoothed estimated Hurst function is plotted.
#' The estimated Hurst function is smoothed using the LOESS method.
#' @param LFD_Est Logical: If \code{TRUE}, the local fractal dimension estimates are plotted.
#' @param LFD_Smooth_Est Logical: If \code{TRUE}, the smoothed estimates of local fractal dimension is plotted.
#' Smoothed using the LOESS method.
#' @param ... Other arguments.
#' @return A ggplot object which is used to plot the time series with theoretical, raw and smoothed estimates of Hurst function
#' and raw and smoothed estimates of local fractal dimension.
#' @details Compared to \code{\link{plot_tsest}}, the function's argument is a \code{"H_LFD"}
#' object, not a time series.
#' @exportS3Method Rmfrac::plot
#' @importFrom graphics plot
#' @seealso \code{\link{H_LFD}}, \code{\link{Hurst}}, \code{\link{LFD}}, \code{\link{plot_tsest}}
#'
#' @examples
#' TS <- data.frame("t" = seq(0, 1, length = 1000), "X(t)" = rnorm(1000))
#' Object <- H_LFD(TS)
#' #Plot of time series, estimated and smoothed Hurst and LFD estimates
#' plot(Object)
#'
plot.H_LFD <- function(x, H_Est = TRUE,H_Smooth_Est = TRUE, LFD_Est = TRUE, LFD_Smooth_Est = TRUE,...) {

  pp <- (autoplot(x, H_Est = H_Est, H_Smooth_Est = H_Smooth_Est, LFD_Est = LFD_Est, LFD_Smooth_Est = LFD_Smooth_Est))

  if (interactive()) {
    print(pp)
  }

  invisible(pp)
}


#' Plot the estimated Hurst functions and local fractal dimension estimates
#' for a user provided time series
#'
#' @description
#' Creates a plot of the user provided time series with the
#' Hurst function estimated using \code{\link{Hurst}}, the smoothed estimated
#' Hurst function and local fractal dimension estimated using \code{\link{LFD}}
#' and smoothed estimates of local fractal dimension.
#'
#' @param X Data frame where the first column is a numeric time sequence \eqn{t}
#' and the second one is the values of the time series \eqn{X(t)}.
#' To get reliable results for time series, it is recommended to use at least 500 time points.
#' @param H_Est Logical: If \code{TRUE}, the Hurst function estimated by using \code{\link{Hurst}} is plotted.
#' @param H_Smooth_Est Logical: If \code{TRUE}, the smoothed estimated Hurst function is plotted.
#' The estimated Hurst function is smoothed using the LOESS method.
#' @param LFD_Est Logical: If \code{TRUE}, the local fractal dimension estimates are plotted.
#' @param LFD_Smooth_Est Logical: If \code{TRUE}, the smoothed estimates of local fractal dimension is plotted.
#' Smoothed using the LOESS method.
#' @param N Argument used for the estimation of Hurst functions and LFD. Number of sub-intervals on which the estimation is performed on. Default is set to 100 sub-intervals.
#' @param Q Argument used for the estimation of Hurst functions and LFD. Fixed integer greater than or equal to 2. Default is set to 2.
#' @param L Argument used for the estimation of Hurst functions and LFD. Fixed integer greater than or equal to 2. Default is set to 2.
#'
#' @return A ggplot object which is used to plot the time series with raw and smoothed estimates of Hurst function
#' and local fractal dimension.
#' @details
#' Compared to \code{\link{plot.H_LFD}} the function's first argument is a time series,
#' not \code{H_LFD} object.
#'
#' @importFrom ggplot2 ggplot geom_line geom_smooth scale_color_manual labs aes sec_axis scale_y_continuous
#' @importFrom rlang .data
#' @importFrom stats IQR quantile
#' @seealso \code{\link{Hurst}}, \code{\link{LFD}}, \code{\link{plot.H_LFD}}
#' @export plot_tsest
#'
#' @examples
#' TS <- data.frame("t" = seq(0, 1, length = 1000), "X(t)" = rnorm(1000))
#' #Plot of time series, estimated and smoothed Hurst and LFD estimates
#' plot_tsest(TS)
#'
plot_tsest<-function(X, H_Est = TRUE, H_Smooth_Est = TRUE, LFD_Est = TRUE, LFD_Smooth_Est = TRUE, N = 100, Q = 2, L = 2){

  X <- na.omit(X)

  if (!is.logical(H_Est) | !is.logical(H_Smooth_Est) | !is.logical(LFD_Est) | !is.logical(LFD_Smooth_Est)) {
    stop("H_Est, H_Smooth_Est, LFD_Est and LFD_Smooth_Est should have logical inputs either TRUE or FALSE")
  }

  if (!is.data.frame(X) | !ncol(X) == 2 | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0))) {
    stop("X must be a numeric data frame with time sequence from 0 to 1 given as the first column")
  }

  X <- X[order(X[,1]),]
  H_est <- Hurst(X, N, Q, L)
  colnames(H_est) <- c("x", "y")
  colnames(X) <- c("t1", "PP")
  t1 <- X[,1]

  LFD_EST <- LFD(X, N, Q, L)
  colnames(LFD_EST) <- c("x1", "y1")

  IQR_H <- IQR(X[,2])
  range_H <- range(X[,2])
  q1_H <- quantile(X[,2], 0.25)

  IQR_L <- IQR(X[,2])
  range_L <- range(X[,2])
  q1_L <- quantile(X[,2], 0.25)

  p <- ggplot(X, aes(x = .data$t1, y = .data$PP)) + geom_line() +
       labs(y = "Time series", x = "Time", color = "")


  if(H_Est){
    p <- p + geom_line(data = H_est, aes(x = .data$x, y = ((.data$y * (IQR_H)) + q1_H), color = factor("Raw Estimate H")), linewidth = 1) +
             scale_y_continuous(name = "Time Series", limits = range_H,
                         sec.axis = sec_axis(transform = function(x){(x - q1_H) / (IQR_H)}, name = "Estimator"))

  }

  if(H_Smooth_Est){
    p <- p + geom_smooth(data = H_est, aes(x = .data$x, y = ((.data$y * (IQR_H)) + q1_H), color = factor("Smoothed Estimate H"))
                         , method = "loess", se = FALSE, span = 0.3, linewidth = 1) +
             scale_y_continuous(name = "Time Series", limits = range_H,
                         sec.axis = sec_axis(transform = function(x){(x - q1_H) / (IQR_H)}, name="Estimator"))

  }

  if(LFD_Est){


    p <- p + geom_line(data = LFD_EST, aes(x = .data$x1, y = ((.data$y1 * (IQR_H)) + q1_H), color = factor("Raw Estimate LFD")), linewidth = 1) +
      scale_y_continuous(name = "Time Series", limits=range_H,
                         sec.axis = sec_axis(transform = function(x){(x - q1_H) / (IQR_H)}, name = "Estimator"))

  }

  if(LFD_Smooth_Est){

    p <- p + geom_smooth(data = LFD_EST, aes(x =.data$x1,y=((.data$y1*(IQR_H))+q1_H),color = factor("Smoothed Estimate LFD"))
                         , method="loess", se = FALSE,span = 0.3, linewidth = 1) +
      scale_y_continuous(name = "Time Series", limits = range_H,
                         sec.axis = sec_axis(transform = function(x){(x - q1_H) / (IQR_H)} ,name = "Estimator"))
  }

  if(interactive()){
    print(p)
  }

  invisible(p)

}

test_that("H_LFD works", {
  Exp_1 <- c(0.6276124, 1.0000000, 1.0000000, 0.0000000, 1.0000000, 1.0000000,
    1.0000000, 0.0000000, 1.0000000, 1.0000000, 1.0000000, 0.0000000,
    1.0000000, 1.0000000, 0.0000000, 1.0000000, 1.0000000, 1.0000000, 0.0000000)
  Exp_2 <- c(1.372388, 1.000000, 1.000000, 2.000000, 1.000000, 1.000000, 1.000000,
    2.000000, 1.000000, 1.000000, 1.000000, 2.000000, 1.000000, 1.000000,
    2.000000, 1.000000, 1.000000, 1.000000, 2.000000)

  expect_equal(Actual_H_LFD$Raw_Hurst_estimates[,2], Exp_1, tolerance = 1e-5)
  expect_equal(Actual_H_LFD$Raw_LFD_estimates[,2], Exp_2, tolerance = 1e-5)
})

test_that("H_LFD has the specified class", {
  expect_s3_class(Actual_H_LFD, c("H_LFD", "list"))
})

test_that("H_LFD fails if X is not a data frame", {
  expect_error(H_LFD(c(0.2788,0.1365,0.5567)))
})

test_that("H_LFD fails if X is a data frame with more than or less than 2 columns", {
  expect_error(H_LFD(data.frame(x = c(0.2788,0.1365,0.5567))))
  expect_error(H_LFD(data.frame(t = c(1, 2, 3), x = c(0.2788,0.1365,0.5567), y =c(0.2788,0.1365,0.5567))))
})

test_that("H_LFD fails if X is a nonnumeric data frame", {
  expect_error(H_LFD(data.frame(t = c("A", "B", "C"), x = c(0.2788,0.1365,0.5567))))
  expect_error(H_LFD(data.frame(t = c(1, 2, 3), x = c(0.2788,"B",0.5567))))
})

test_that("H_LFD fails if X[,1] have negative values", {
  expect_error(H_LFD(data.frame(t = c(1, -2, 3), x = c(0.2788,0.1365,0.5567))))
})

test_that("H_LFD fails if N is nonnumeric", {
  expect_error(H_LFD(TS, "a"))
})

test_that("H_LFD fails if N is negative", {
  expect_error(H_LFD(TS, -100))
})

test_that("H_LFD fails if N is non integer", {
  expect_error(H_LFD(TS, 100.5))
})

test_that("H_LFD fails if Q is nonnumeric", {
  expect_error(H_LFD(TS, Q = "a"))
})

test_that("H_LFD fails if Q is less than 2", {
  expect_error(H_LFD(TS, Q = -2))
  expect_error(H_LFD(TS, Q = 1))
})

test_that("H_LFD fails if Q is non integer", {
  expect_error(H_LFD(TS, Q = 5.5))
})

test_that("H_LFD fails if L is nonnumeric", {
  expect_error(H_LFD(TS, L = "a"))
})

test_that("H_LFD fails if L is less than 2", {
  expect_error(H_LFD(TS, L = -2))
  expect_error(H_LFD(TS, L = 1))
})

test_that("H_LFD fails if L is non integer", {
  expect_error(H_LFD(TS, L = 5.5))
})

#plot_tsest

test_that("plot_tsest fails if X is not a data frame", {
  expect_error(plot_tsest(c(0.2788,0.1365,0.5567)))
})

test_that("plot_tsest fails if X is a data frame with more than or less than 2 columns", {
  expect_error(plot_tsest(data.frame(x = c(0.2788,0.1365,0.5567))))
  expect_error(plot_tsest(data.frame(t = c(1, 2, 3), x = c(0.2788,0.1365,0.5567), y =c(0.2788,0.1365,0.5567))))
})

test_that("plot_tsest fails if X is a nonnumeric data frame", {
  expect_error(plot_tsest(data.frame(t = c("A", "B", "C"), x = c(0.2788,0.1365,0.5567))))
  expect_error(plot_tsest(data.frame(t = c(1, 2, 3), x = c(0.2788,"B",0.5567))))
})

test_that("plot_tsest fails if X[,1] have negative values", {
  expect_error(plot_tsest(data.frame(t = c(1, -2, 3), x = c(0.2788,0.1365,0.5567))))
})

test_that("plot_tsest fails if H_Est or H_Smooth_Est or LFD_Est or LFD_Smooth_Est is non logical", {
  expect_error(plot_tsest(TS, H_Est = 1))
  expect_error(plot_tsest(TS, H_Smooth_Est = 1))
  expect_error(plot_tsest(TS, LFD_Est = 1))
  expect_error(plot_tsest(TS, LFD_Smooth_Est = 1))
})

test_that("plot_tsest fails if N is nonnumeric", {
  expect_error(plot_tsest(TS, "a"))
})

test_that("plot_tsest fails if N is negative", {
  expect_error(plot_tsest(TS, -100))
})

test_that("plot_tsest fails if N is non integer", {
  expect_error(plot_tsest(TS, 100.5))
})

test_that("plot_tsest fails if Q is nonnumeric", {
  expect_error(plot_tsest(TS, Q = "a"))
})

test_that("plot_tsest fails if Q is less than 2", {
  expect_error(plot_tsest(TS, Q = -2))
  expect_error(plot_tsest(TS, Q = 1))
})

test_that("plot_tsest fails if Q is non integer", {
  expect_error(plot_tsest(TS, Q = 5.5))
})

test_that("plot_tsest fails if L is nonnumeric", {
  expect_error(plot_tsest(TS, L = "a"))
})

test_that("plot_tsest fails if L is less than 2", {
  expect_error(plot_tsest(TS, L = -2))
  expect_error(plot_tsest(TS, L = 1))
})

test_that("plot_tsest fails if L is non integer", {
  expect_error(plot_tsest(TS, L = 5.5))
})

set.seed(123)
Bm_sim <- Bm(N = 10)

set.seed(123)
fBm_sim <- FBm(H = 0.5, N = 10)

set.seed(123)
fGn_sim <- FGn(H = 0.5, N = 10)

set.seed(123)
Bbridge_sim <- Bbridge(x_end = 1, t_end = 1, N = 10)

set.seed(123)
FBbridge_sim <- FBbridge(H = 0.5, x_end = 2, t_end = 1, N = 10)

#' Simulation of Gaussian Haar-based multifractional processes
#'
#' @description
#' This function simulates a realisation of a Gaussian Haar-based multifractional process at any
#' time point or time sequence on the interval \eqn{[0,1]}.
#'
#' @param t Time point or time sequence on the interval \eqn{[0,1]}.
#' @param H Hurst function which depends on \code{t} \eqn{(H(t))}. See Examples for usage.
#' @param J Positive integer. \code{J} is recommended to be greater than \eqn{\log_2(length(t))}. For large \code{J} values could
#' be rather time consuming. Default is set to 15.
#' @param num.cores Number of cores to set up the clusters for parallel computing.
#'
#' @return A data frame of class \code{"mp"} where the first column is time moments \code{t} and second column is simulated values of \eqn{X(t)}.
#'
#' @details
#' The following formula defined in Ayache, A., Olenko, A. & Samarakoon, N. (2026) was used in simulating Gaussian Haar-based multifractional process.
#'
#' \deqn{X(t) := \sum_{j=0}^{+\infty}  \sum_{k=0}^{2^{j}-1}\left(\int_{0}^{1} (t-s)_{+}^{H_{j}(k/{2^j})-{1}/{2}} h_{j,k}(s)ds \right)\varepsilon_{j,k},}
#'
#' where
#'
#' \deqn{  \int_{0}^{1} (t-s)_{+}^{H_{j,k}-\frac{1}{2}} h_{j,k} (s) ds = 2^{-j H_{j,k}} h^{[H_{j,k}]} (2^jt-k)}
#'
#' with \eqn{h^{[\lambda]} (x) =  \int_{\mathbb{R}} (x-s)_{+}^{\lambda-\frac{1}{2}} h(s) ds}.
#' \eqn{h} is the Haar mother wavelet, \eqn{j} and \eqn{k} are positive integers, \eqn{t} is time, \eqn{H} is the Hurst function and
#' \eqn{\varepsilon_{j,k}} is a sequence of independent \eqn{\mathcal{N}(0,1)} Gaussian random variables.
#' For simulations, the truncated version of this formula with first summation up to J is used.
#'
#' @note
#' See Examples for the usage of constant, time-varying, piecewise or step Hurst functions.
#'
#' @export GHBMP
#'
#' @importFrom parallel clusterExport parLapply stopCluster
#' @importFrom parallelly availableCores makeClusterPSOCK
#' @importFrom stats rnorm
#'
#' @references Ayache, A., Olenko, A. and Samarakoon, N. (2026).
#' On construction, properties and simulation of Haar-based multifractional processes.
#' Mathematics and Computers in Simulation. 246:311-332. \doi{doi:10.1016/j.matcom.2026.01.033}.
#'
#' @seealso \code{\link{Hurst}}, \code{\link{plot.mp}}, \code{\link{Bm}}, \code{\link{FBm}},
#' \code{\link{FGn}}, \code{\link{Bbridge}}, \code{\link{FBbridge}}
#' @examples
#' \donttest{
#' #Constant Hurst function
#' t <- seq(0, 1, by = (1/2)^10)
#' H <- function(t) {return(0.4 + 0*t)}
#' GHBMP(t, H)
#'
#' #Linear Hurst function
#' t <- seq(0, 1, by = (1/2)^10)
#' H <- function(t) {return(0.2 + 0.45*t)}
#' GHBMP(t, H)
#'
#' #Oscillating Hurst function
#' t <- seq(0, 1, by = (1/2)^10)
#' H <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
#' GHBMP(t, H)
#'
#' #Piecewise Hurst function
#' t <- seq(0, 1, by = (1/2)^10)
#' H <- function(x) {
#' ifelse(x >= 0 & x <= 0.8, 0.375 * x + 0.2,
#'       ifelse(x > 0.8 & x <= 1,-1.5 * x + 1.7, NA))
#' }
#' GHBMP(t, H)
#' }
GHBMP <- function(t, H, J = 15, num.cores = availableCores(omit = 1))
{
  if (!is.numeric(t)|!all(t >= 0 & t<= 1)) {
    stop("t must be a numeric sequence between 0 and 1")
  }

  H.t<-sapply(t, H)
  if (!is.numeric(H.t) | !all(H.t > 0 & H.t< 1)) {
    stop("H must be a function which returns a numeric list between 0 and 1")
  }

  if (!is.numeric(J)) {
    stop("J must be numeric")
   } else if (!(J > 0) | !(J %% 1 == 0)){
      stop("J must be a positive integer")
    }

  if (!is.numeric(num.cores)) {
    stop("num.cores must be numeric")
   } else if (!(num.cores %% 1 == 0) | !(num.cores > 0)) {
      stop("num.cores must be a positive integer")
   }

  cl <- makeClusterPSOCK(num.cores) #Creation of a cluster using PSOCK connections for parallel computing

  t <- sort(t)

  x1 <- 0:J
  x2 <- 0:(2^(J) - 1)

  Aind <- outer(x1, x2, function(x,y) as.integer(as.logical(y < 2^x)))
  Aindv <- unlist(asplit(Aind, 2))
  ind0 <- which(Aindv > 0)

  A1 <- outer(x1, x2, function(x, y) 2^x)
  A2 <- outer(x1, x2, function(x, y) y)

  m <- rnorm((J + 1) * 2^(J))
  dim(m) <- c(J + 1, 2^(J))

  H1 <- outer(x1, x2, function(x, y) H(y / 2^x))

  A3 <- outer(x1, x2, function(x, y) 2^(-x * H(y / 2^x)))

  m <- ((H1 + 0.5)^(-1)) * m * A3

  A1v <- unlist(asplit(A1, 2))[ind0]
  A2v <- unlist(asplit(A2, 2))[ind0]
  mv <- unlist(asplit(m, 2))[ind0]
  Hv <- unlist(asplit(H1, 2))[ind0]

  Xt<-function(t)
  {
    tv <- A1v * t - A2v
    ind1 <- which(tv > 0)
    ind2 <- which(tv > 0.5)
    ind3 <- which(tv > 1)
    return(sum(mv[ind1] * (tv[ind1]^(Hv[ind1] + 0.5))) - 2 * sum(mv[ind2] * ((tv[ind2] - 0.5)^(Hv[ind2] + 0.5))) + sum(mv[ind3] * ((tv[ind3] - 1)^(Hv[ind3] + 0.5))))
  }

  clusterExport(cl, c("t", "A1v", "A2v", "Hv", "mv"), envir = environment())
  #XN <- do.call(c, parLapply(cl, t, Xt))
  XN <- unlist(parLapply(cl, t, Xt))

  stopCluster(cl)

  sim_data <- data.frame("t" = t, "X" = XN)
  class(sim_data) <- c("mp", class(sim_data))
  return(sim_data)

}




#' Simulation of Brownian motion
#'
#' @description
#' This function simulates a realisation of the Brownian motion over the
#' time interval \code{[t_start,t_end]} with \code{N} time steps and initial value \code{x_start}.
#'
#' @param x_start Value of the process at the initial time point (additive constant mean).
#' @param t_start Initial time point.
#' @param t_end Terminal time point.
#' @param N Number of time steps on the interval \code{[t_start,t_end]}.
#' Default set to 1000.
#' @param plot Logical: If \code{TRUE}, the realisation of the Brownian
#' motion is plotted in interactive sessions.
#'
#' @return A data frame where the first column is \code{t} and second
#' column is simulated values of the realisation of Brownian motion with added constant mean.
#'
#' @importFrom ggplot2 ggplot geom_line labs ggtitle aes
#' @importFrom stats rnorm
#'
#' @export Bm
#' @seealso \code{\link{GHBMP}}, \code{\link{FBm}}, \code{\link{FGn}}, \code{\link{Bbridge}}, \code{\link{FBbridge}}
#'
#' @examples
#' Bm(t_end = 2, plot = TRUE)
Bm <- function(x_start = 0, t_start = 0, t_end = 1, N = 1000, plot = FALSE){

  if (!is.numeric(x_start)) {
    stop("x_start must be numeric")
  }

  if (!is.numeric(t_start)) {
    stop("t_start must be numeric")
  } else if ( !(t_start >=0)) {
    stop("Incorrect input for t_start")
  }

  if (!is.numeric(t_end)) {
    stop("t_end must be numeric")
  }

  if (!(t_start < t_end)) {
    stop("Incorrect inputs for t_start and t_end")
  }

  if (!is.numeric(N)) {
    stop("N must be numeric")
  } else if (!(N %% 1 == 0) | !(N > 0)) {
    stop("N must be a positive integer")
  }

  if (!is.logical(plot)) {
    stop("Plot must have logical inputs either TRUE or FALSE")
  }

  diff <- (t_end - t_start) / N
  increments <- rnorm(N, mean = 0, sd = sqrt(diff))
  X <- c(x_start, x_start + cumsum(increments))
  t <- seq(t_start, t_end, length.out = N + 1)
  sim_data <- data.frame(t = t, X = X)

  if (isTRUE(plot) && interactive()) {
    p<- ggplot(sim_data, aes(x = .data$t, y = .data$X)) +
      geom_line() +
      labs(y="X(t)",x="t") +
      ggtitle("Realisation of Brownian motion")

    print(p)
  }

  return(sim_data)
}


#' Simulation of fractional Brownian motion
#'
#' @description
#' This function simulates a realisation of the fractional Brownian motion over
#' the time interval \code{[t_start,t_end]} for a provided Hurst parameter, which has
#' the initial value \code{x_start}.
#'
#' @param H Hurst parameter which lies between 0 and 1.
#' @param x_start Value of the process at the initial time point (additive constant mean).
#' @param t_start Initial time point.
#' @param t_end Terminal time point.
#' @param N Number of time steps on the interval \code{[t_start,t_end]}.
#' Default set to 1000.
#' @param plot Logical: If \code{TRUE}, the realisation of the fractional Brownian
#' motion is plotted in interactive sessions.
#'
#' @return A data frame where the first column is \code{t} and second
#' column is simulated values of the realisation of fractional Brownian motion
#' with added constant mean.
#'
#' @importFrom ggplot2 ggplot geom_line labs ggtitle aes
#' @importFrom stats toeplitz rnorm
#'
#' @export FBm
#' @references Banna, O., Mishura, Y., Ralchenko, K., & Shklyar, S. (2019). Fractional Brownian motion:
#' Approximations and Projections. John Wiley & Sons. \doi{doi:10.1002/9781119476771.app3}.
#' @seealso \code{\link{FGn}}, \code{\link{Bm}}, \code{\link{GHBMP}}, \code{\link{Bbridge}}, \code{\link{FBbridge}}
#'
#' @examples
#' FBm(H = 0.5, plot = TRUE)
FBm <- function(H, x_start = 0, t_start = 0, t_end = 1, N = 1000, plot = FALSE){

  if (!is.numeric(H) | !(H > 0 & H< 1)) {
    stop("H must be a number between 0 and 1")
  }

  if (!is.numeric(x_start)) {
    stop("x_start must be numeric")
  }

  if (!is.numeric(t_start)) {
    stop("t_start must be numeric")
  } else if ( !(t_start >=0)) {
    stop("Incorrect input for t_start")
  }

  if (!is.numeric(t_end)) {
    stop("t_end must be numeric")
  } else if ( !(t_end > 0)) {
    stop("Incorrect input for t_end")
  }

  if (!(t_start < t_end)) {
    stop("Incorrect inputs for t_start and t_end")
  }

  if (!is.numeric(N)) {
    stop("N must be numeric")
  } else if (!(N %% 1 == 0) | !(N > 0)) {
    stop("N must be a positive integer")
  }

  if (!is.logical(plot)) {
    stop("Plot must have logical inputs either TRUE or FALSE")
  }

  diff <- (t_end - t_start) / N
  t <- seq(t_start, t_end, length.out = N + 1)

  c <- function(k, H){
    0.5 * ((k + 1)^(2*H) + (abs(k - 1))^(2*H) - 2*(k)^(2*H))
  }

  cov_mat <- toeplitz(sapply(0:(N-1), c, H = H))

  L <- t(chol(cov_mat))

  z <- rnorm(N, 0, 1)

  x <- L%*%z

  B <- cumsum(x)
  B <- B * diff^H
  B <- x_start + B
  B <- append(B, x_start, after = 0)

  sim_data <- data.frame(t = t, X = B)

  if (isTRUE(plot) && interactive()) {
    p<- ggplot(sim_data, aes(x = .data$t, y = .data$X)) +
      geom_line() +
      labs(y = "X(t)",x = "t") +
      ggtitle("Realisation of fractional Brownian motion")

    print(p)
  }

  return(sim_data)

}

#' Simulation of fractional Gaussian noise
#'
#' @description
#' This function simulates a realisation of the fractional Gaussian noise over the
#' time interval \code{[t_start,t_end]} for a provided Hurst parameter.
#'
#' @param H Hurst parameter which lies between 0 and 1.
#' @param t_start Initial time point.
#' @param t_end Terminal time point.
#' @param N Number of time steps on the interval \code{[t_start,t_end]}.
#' Default set to 1000.
#' @param plot Logical: If \code{TRUE}, the realisation of the fractional Gaussian noise
#' is plotted in interactive sessions.
#'
#' @return A data frame where the first column is \code{t} and second
#' column is simulated values of the realisation of fractional Gaussian noise.
#'
#' @importFrom ggplot2 ggplot geom_line labs ggtitle aes
#' @importFrom stats toeplitz rnorm
#'
#' @export FGn
#' @references Banna, O., Mishura, Y., Ralchenko, K., & Shklyar, S. (2019). Fractional Brownian motion:
#' Approximations and Projections. John Wiley & Sons. \doi{doi:10.1002/9781119476771.app3}.
#' @seealso \code{\link{FBm}}, \code{\link{Bm}}, \code{\link{GHBMP}}, \code{\link{Bbridge}}, \code{\link{FBbridge}}
#' @examples
#' FGn(H = 0.5,plot = TRUE)
FGn <- function(H, t_start = 0, t_end = 1, N = 1000, plot = FALSE){

  if (!is.numeric(H) | !(H > 0 & H< 1)) {
    stop("H must be a number between 0 and 1")
  }

  if (!is.numeric(t_start)) {
    stop("t_start must be numeric")
  } else if ( !(t_start >= 0)) {
    stop("Incorrect input for t_start")
  }

  if (!is.numeric(t_end)) {
    stop("t_end must be numeric")
  } else if ( !(t_end > 0)) {
    stop("Incorrect input for t_end")
  }

  if (!(t_start<t_end)) {
    stop("Incorrect inputs for t_start and t_end")
  }

  if (!is.numeric(N)) {
    stop("N must be numeric")
  } else if (!(N %% 1 == 0) | !(N > 0)) {
    stop("N must be a positive integer")
  }

  if (!is.logical(plot)) {
    stop("Plot must have logical inputs either TRUE or FALSE")
  }

  t <- seq(t_start, t_end, length.out = N + 1)

  c <- function(k, H){
    0.5 * ((k + 1)^(2*H) + (abs(k - 1))^(2*H) - 2*(k)^(2*H))
  }

  cov_mat <- toeplitz(sapply(0:(N), c, H = H))

  L <- t(chol(cov_mat))

  z <- rnorm(N + 1, 0, 1)

  x <- L%*%z

  sim_data <- data.frame(t = t, X = x)

  if (isTRUE(plot) && interactive()) {
    p<- ggplot(sim_data,aes(x = .data$t,y = .data$X)) +
      geom_line() +
      labs(y = "X(t)",x = "t") +
      ggtitle("Realisation of fractional Guassian noise")

    print(p)
  }

  return(sim_data)

}

#' Simulation of Brownian bridge
#'
#' @description
#' This function simulates a realisation of the Brownian bridge over the
#' time interval \code{[0,t_end]} which has the initial value \code{x_start} and terminates
#' at \code{x_end} with \code{N} time steps.
#'
#' @param x_end Value of the process at the terminating time point.
#' @param t_end Terminal time point.
#' @param x_start Value of the process at the initial time point.
#' @param N Number of time steps on the interval \code{[0,t_end]}.
#' Default set to 1000.
#' @param plot Logical: If \code{TRUE}, the realisation of the Brownian bridge
#' is plotted in interactive sessions.
#' @importFrom ggplot2 ggplot geom_line labs ggtitle aes
#' @return A data frame where the first column is \code{t} and second
#' column is simulated values of the realisation of Brownian bridge.
#' @seealso \code{\link{Bm}}, \code{\link{FBm}}, \code{\link{FBbridge}}, \code{\link{FGn}}, \code{\link{GHBMP}}
#' @export Bbridge
#' @references Bianchi, S., Frezza, M., Pianese, A., Palazzo, A.M. (2022). Modelling
#' H-Volatility with Fractional Brownian Bridge. In: Corazza, M., Perna, C., Pizzi, C.,
#' Sibillo, M. (eds) Mathematical and Statistical Methods for Actuarial Sciences and Finance.
#' MAF 2022. Springer, Cham. \doi{doi:10.1007/978-3-030-99638-3_16}.
#' @examples
#' Bbridge(x_end = 2, t_end = 1, plot = TRUE)
Bbridge <- function(x_end, t_end, x_start = 0, N = 1000, plot = FALSE){

  if (!is.numeric(x_end)) {
    stop("x_end must be numeric")
  }

  if (!is.numeric(t_end)) {
    stop("t_end must be numeric")
  } else if ( !(t_end > 0)) {
    stop("Incorrect input for t_end")
  }

  if (!is.numeric(x_start)) {
    stop("x_end must be numeric")
  }

  if (!is.numeric(N)) {
    stop("N must be numeric")
  } else if (!(N %% 1 == 0) | !(N > 0)) {
    stop("N must be a positive integer")
  }

  if (!is.logical(plot)) {
    stop("Plot must have logical inputs either TRUE or FALSE")
  }

  Bm_sim <- Bm(x_start = x_start, t_start = 0, t_end = t_end, N = N)
  X <- Bm_sim[,2] - ((Bm_sim[,1] / t_end) * (Bm_sim[N + 1, 2] - x_end))
  sim_data <- data.frame(t = Bm_sim[,1], X = X)

  if (isTRUE(plot) && interactive()) {
    p<- ggplot(sim_data, aes(x = .data$t, y = .data$X)) +
      geom_line() +
      labs(y = "X(t)",x = "t") +
      ggtitle("Realisation of Brownian bridge")

    print(p)
  }

  return(sim_data)
}




#' Simulation of fractional Brownian bridge
#'
#' @description
#' This function simulates a realisation of the fractional Brownian bridge
#' for a provided Hurst parameter over the time interval \code{[0,t_end]},
#' which has the initial value \code{x_start} and terminates at \code{x_end} with \code{N} time steps.
#'
#' @param H Hurst parameter which lies between 0 and 1.
#' @param x_end Value of the process at the terminating time point.
#' @param t_end Terminal time point.
#' @param x_start Value of the process at the initial time point.
#' @param N Number of time steps on the interval \code{[0,t_end]}.
#' Default set to 1000.
#' @param plot Logical: If \code{TRUE}, the realisation of the fractional Brownian bridge
#' is plotted in interactive sessions.
#' @importFrom ggplot2 ggplot geom_line labs ggtitle aes
#' @return A data frame where the first column is \code{t} and second
#' column is simulated values of the realisation of fractional Brownian bridge.
#' @seealso \code{\link{FBm}}, \code{\link{FGn}}, \code{\link{Bm}}, \code{\link{GHBMP}}, \code{\link{Bbridge}}
#' @export FBbridge
#' @references Bianchi, S., Frezza, M., Pianese, A., Palazzo, A.M. (2022). Modelling
#' H-Volatility with Fractional Brownian Bridge. In: Corazza, M., Perna, C., Pizzi, C.,
#' Sibillo, M. (eds) Mathematical and Statistical Methods for Actuarial Sciences and Finance.
#' MAF 2022. Springer, Cham. \doi{doi:10.1007/978-3-030-99638-3_16}.
#' @examples
#' FBbridge(H = 0.5, x_end = 2, t_end = 1, plot = TRUE)
FBbridge <- function(H, x_end, t_end, x_start = 0, N = 1000, plot = FALSE){

  if (!is.numeric(H) | !(H > 0 & H< 1)) {
    stop("H must be a number between 0 and 1")
  }

  if (!is.numeric(x_end)) {
    stop("x_end must be numeric")
  }

  if (!is.numeric(t_end)) {
    stop("t_end must be numeric")
  } else if ( !(t_end > 0)) {
    stop("Incorrect input for t_end")
  }

  if (!is.numeric(x_start)) {
    stop("x_end must be numeric")
  }

  if (!is.numeric(N)) {
    stop("N must be numeric")
  } else if (!(N %% 1 == 0) | !(N > 0)) {
    stop("N must be a positive integer")
  }

  if (!is.logical(plot)) {
    stop("Plot must have logical inputs either TRUE or FALSE")
  }

  FBm_sim <- FBm(H = H, x_start = x_start, t_start = 0, t_end = t_end, N = N)
  X <- FBm_sim[,2] - (0.5 * (FBm_sim[N + 1, 2] - x_end) * (1 + (FBm_sim[,1] / t_end)^(2*H) - (1 - (FBm_sim[,1] / t_end))^(2*H)))
  sim_data <- data.frame(t = FBm_sim[,1], X = X)

  if (isTRUE(plot) && interactive()) {
    p<- ggplot(sim_data, aes(x = .data$t, y = .data$X)) +
      geom_line() +
      labs(y = "X(t)", x = "t") +
      ggtitle("Realisation of fractional Brownian bridge")

    print(p)
  }

  return(sim_data)
}

#GHBMP()

test_that("GHBMP fails for nonnumeric t", {
  expect_error(GHBMP("a", H, 1))
})

test_that("GHBMP fails for t not in [0,1]", {
  expect_error(GHBMP(2:3, H, 1))
  expect_error(GHBMP(-1, H, 1))
})

test_that("GHBMP fails for empty t ", {
  expect_error(GHBMP(c(), H, 1))
})

test_that("GHBMP fails for H(t) not in (0,1)", {
  expect_error(GHBMP(0.1, function(t) 2 + 0.5 * t, 1))
  expect_error(GHBMP(0.1, function(t) -2 - 0.5 * t, 1))
})

test_that("GHBMP fails for nonnumeric J ", {
  expect_error(GHBMP(0.1, H, "a"))
})

test_that("GHBMP fails for negative J", {
  expect_error(GHBMP(0.1, H, -1))
})

test_that("GHBMP fails for non integer J", {
  expect_error(GHBMP(0.1, H, 1.5))
})

test_that("GHBMP fails for nonnumeric num.cores", {
  expect_error(GHBMP(0.1, H, 1, "a"))
})

test_that("GHBMP fails for negative num.cores", {
  expect_error(GHBMP(0.1, H, 1, -1))
})

test_that("GHBMP fails for non integer num.cores", {
  expect_error(GHBMP(0.1, H, 1, 1.5))
})


#Bm()

test_that("Bm works", {

  expected_sim <- data.frame(
    t = c(0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0),
    X = c(0.0000000, -0.1772380, -0.2500265, 0.2428804, 0.2651771, 0.3060615, 0.8484126, 0.9941671, 0.5941196, 0.3769177, 0.2359870))

  expect_equal(Bm_sim, expected_sim, tolerance = 1e-6)

})

test_that("Bm fails if x_start is nonnumeric", {
  expect_error(Bm("a", N = 5))
})

test_that("Bm fails if t_start is nonnumeric", {
  expect_error(Bm(t_start = "a", N = 5))
})

test_that("Bm fails if t_start is negative", {
  expect_error(Bm(t_start = -1, N = 5))
})

test_that("Bm fails if t_end is nonnumeric", {
  expect_error(Bm(t_end = "a", N = 5))
})

test_that("Bm fails if t_end is negative", {
  expect_error(Bm(t_end = -1, N = 5))
})

test_that("Bm fails if t_end < t_start", {
  expect_error(Bm(t_start = 5, t_end = 4, N = 5))
})

test_that("Bm fails for nonnumeric N ", {
  expect_error(Bm(N = "a"))
})

test_that("Bm fails for negative N", {
  expect_error(Bm(N = -1))
})

test_that("Bm fails for non integer N", {
  expect_error(Bm(N = 1.5))
})

test_that("Bm fails for non logical plot", {
  expect_error(Bm(plot = 1, N = 5))
  expect_error(Bm(plot = "TRUE", N = 5))
})

#FBm()

test_that("FBm works", {

  expected_sim <- data.frame(
    t = c(0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0),
    X = c(0.0000000, -0.1772380, -0.2500265, 0.2428804, 0.2651771, 0.3060615, 0.8484126, 0.9941671, 0.5941196, 0.3769177, 0.2359870))

  expect_equal(fBm_sim, expected_sim, tolerance = 1e-6)

})

test_that("FBm fails for nonnumeric H ", {
  expect_error(FBm("a", N = 5))
})

test_that("FBm fails for H not in (0,1)", {
  expect_error(FBm(-2, N = 5))
  expect_error(FBm(2, N = 5))
})

test_that("FBm fails if x_start is nonnumeric", {
  expect_error(FBm(0.5, "a", N = 5))
})

test_that("FBm fails if t_start is nonnumeric", {
  expect_error(FBm(0.5, t_start = "a", N = 5))
})

test_that("FBm fails if t_start is negative", {
  expect_error(FBm(0.5, t_start = -1, N = 5))
})

test_that("FBm fails if t_end is nonnumeric", {
  expect_error(FBm(0.5, t_end = "a", N = 5))
})

test_that("FBm fails if t_end is negative", {
  expect_error(FBm(0.5, t_end = -1, N = 5))
})

test_that("FBm fails if t_end < t_start", {
  expect_error(FBm(0.5, t_start = 5, t_end = 4, N = 5))
})

test_that("FBm fails for nonnumeric N ", {
  expect_error(FBm(0.5, N = "a"))
})

test_that("FBm fails for negative N", {
  expect_error(FBm(0.5, N = -1))
})

test_that("FBm fails for non integer N", {
  expect_error(FBm(0.5, N = 1.5))
})

test_that("FBm fails for non logical plot", {
  expect_error(FBm(0.5, plot = 1, N = 5))
  expect_error(FBm(0.5, plot = "TRUE", N = 5))
})

#FGn()

test_that("FGn works", {

  expected_sim <- data.frame(
    t = c(0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0),
    X = c(-0.56047565, -0.23017749, 1.55870831, 0.07050839, 0.12928774,
          1.71506499, 0.46091621, -1.26506123, -0.68685285, -0.44566197, 1.22408180))

  expect_equal(fGn_sim, expected_sim, tolerance = 1e-6)

})

test_that("FGn fails for nonnumeric H ", {
  expect_error(FGn("a", N = 5))
})

test_that("FGn fails for H not in (0,1)", {
  expect_error(FGn(-2, N = 5))
  expect_error(FGn(2, N = 5))
})

test_that("FGn fails if t_start is nonnumeric", {
  expect_error(FGn(0.5, t_start = "a", N = 5))
})

test_that("FGn fails if t_start is negative", {
  expect_error(FGn(0.5, t_start = -1, N = 5))
})

test_that("FGn fails if t_end is nonnumeric", {
  expect_error(FGn(0.5, t_end = "a", N = 5))
})

test_that("FGn fails if t_end is negative", {
  expect_error(FGn(0.5, t_end = -1, N = 5))
})

test_that("FGn fails if t_end < t_start", {
  expect_error(FGn(0.5, t_start = 5, t_end = 4, N = 5))
})

test_that("FGn fails for nonnumeric N ", {
  expect_error(FGn(0.5, N = "a"))
})

test_that("FGn fails for negative N", {
  expect_error(FGn(0.5, N = -1))
})

test_that("FGn fails for non integer N", {
  expect_error(FGn(0.5, N = 1.5))
})

test_that("FGn fails for non logical plot", {
  expect_error(FGn(0.5, plot = 1, N = 5))
  expect_error(FGn(0.5, plot = "TRUE", N = 5))
})

#Bbridge()

test_that("Bbridge works", {

  expected_sim <- data.frame(
    t = c(0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0),
    X = c(0.00000000, -0.10083666, -0.09722388, 0.47208427, 0.57078228,
          0.68806795, 1.30682042, 1.52897622, 1.20533003, 1.06452939, 1.00000000))

  expect_equal(Bbridge_sim, expected_sim, tolerance = 1e-6)

})

test_that("Bbridge fails for nonnumeric x_end", {
  expect_error(Bbridge("a", 1, N = 5))
})

test_that("Bbridge fails for nonnumeric t_end", {
  expect_error(Bbridge(2, "a", N = 5))
})

test_that("Bbridg fails if t_end is negative or 0", {
  expect_error(Bbridge(2, -2, N = 5))
  expect_error(Bbridge(2, 0, N = 5))
})

test_that("Bbridge fails for nonnumeric x_start", {
  expect_error(Bbridge(2, 1, "a", N = 5))
})

test_that("Bbridge fails for nonnumeric N", {
  expect_error(Bbridge(2, 1, N = "a"))
})

test_that("Bbridge fails for negative N", {
  expect_error(Bbridge(2, 1, N = -2))
})

test_that("Bbridge fails for non integer N", {
  expect_error(Bbridge(2, 1, N = 1.5))
})

test_that("Bbridge fails for non logical plot", {
  expect_error(Bbridge(2, 1, N = 5, plot = 1))
  expect_error(Bbridge(2, 1, N = 5, plot = "TRUE"))
})

#FBbridge()

test_that("FBbridge works", {

  expected_sim <- data.frame(
    t = c(0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0),
    X = c(0.0000000000, -0.0008366623, 0.1027761237, 0.7720842710, 0.9707822814,
          1.1880679523, 1.9068204210, 2.2289762224, 2.0053300336, 1.9645293899, 2.0000000000))

  expect_equal(FBbridge_sim, expected_sim, tolerance = 1e-6)

})

test_that("FBbridge fails for nonnumeric H ", {
  expect_error(FBbridge("a", 2, 1, N = 5))
})

test_that("FBbridge fails for H not in (0,1)", {
  expect_error(FBbridge(-2, 2, 1, N = 5))
  expect_error(FBbridge(2, 2, 1, N = 5))
})

test_that("FBbridge fails for nonnumeric x_end", {
  expect_error(FBbridge(0.5, "a", 1, N = 5))
})

test_that("FBbridge fails for nonnumeric t_end", {
  expect_error(FBbridge(0.5, 2, "a", N = 5))
})

test_that("FBbridg fails if t_end is negative or 0", {
  expect_error(FBbridge(0.5, 2, -2, N = 5))
  expect_error(FBbridge(0.5, 2, 0, N = 5))
})

test_that("FBbridge fails for nonnumeric x_start", {
  expect_error(FBbridge(0.5, 1, 1, "a", N = 5))
})

test_that("FBbridge fails for nonnumeric N", {
  expect_error(FBbridge(0.5, 1, 1, N = "a"))
})

test_that("FBbridge fails for negative N", {
  expect_error(FBbridge(0.5, 1, 1, N = -2))
})

test_that("FBbridge fails for non integer N", {
  expect_error(FBbridge(0.5, 1, 1, N = 1.5))
})

test_that("FBbridge fails for non logical plot", {
  expect_error(FBbridge(0.5, 1, 1, N = 5, plot = 1))
  expect_error(FBbridge(0.5, 1, 1, N = 5, plot = "TRUE"))
})

#' Rmfrac: Simulation and analysis of multifractional processes
#'
#' A collection of tools for simulating, analysing and visualising multifractional processes
#' and time series. The package includes estimation techniques for the Hurst function,
#' Local Fractal Dimension and several other geometric statistics. It provides highly customisable
#' plotting functions for simulated realisations, user-provided time series and their statistics.
#'
#' @section Features:
#' \itemize{
#'   \item Simulation of Brownian motion, fractional Brownian motion, fractional Gaussian noise,
#'         Brownian bridge and fractional Brownian bridge.
#'   \item Simulation of Gaussian Haar-based multifractional process (GHBMP).
#'   \item Estimation of Hurst function and Local Fractal Dimension.
#'   \item Customisable plotting functions for GHBMP and user provided time series
#'         with estimates of Hurst function and Local Fractal Dimension.
#'   \item Estimation and visualisation of geometric statistics using realisations of stochastic processes
#'         and time series. Clustering based on the Hurst function. Estimating sojourn measure, excursion area, etc.
#'   \item An interactive Shiny application that provides options to explore and visualise
#'         the core functionalities of the package through simulations and user-provided time series.
#' }
#'
#' @author
#'
#' \itemize{
#'   \item Andriy Olenko \href{https://orcid.org/0000-0002-0917-7000}{ORCID} <a.olenko@latrobe.edu.au>
#'   \item Nemini Samarakoon \href{https://orcid.org/0009-0000-2107-1973}{ORCID} <neminisamarakoon95@gmail.com>
#' }
#'
#' @seealso
#' Useful links:
#' \itemize{
#'   \item \url{https://github.com/Nemini-S/Rmfrac}
#'   \item Report bugs at \url{https://github.com/Nemini-S/Rmfrac/issues}
#' }
#'
#' @docType package
#' @name Rmfrac-package
"_PACKAGE"

#' K-means clustering
#'
#' @description
#' This function performs k-means clustering of realisations
#' based on the estimated Hurst functions.
#'
#' @param X.t A list of data frames. In each data frame, the first column is a numeric time sequence
#' and the second gives the values of the processes or time series. To get reliable results, it is
#' recommended to use at least 500 time points. See Examples for usage.
#'
#' @param k The desired number of clusters.
#' @param ... Optional arguments: \code{iter.max}, \code{nstart} and \code{algorithm}.
#' Refer \code{\link[stats]{kmeans}}.
#' @param N Argument used for the estimation of Hurst functions. Number of sub-intervals on which the estimation is performed on. Default is set to 100 sub-intervals.
#' @param Q Argument used for the estimation of Hurst functions. Fixed integer greater than or equal to 2. Default is set to 2.
#' @param L Argument used for the estimation of Hurst functions. Fixed integer greater than or equal to 2. Default is set to 2.
#'
#' @return An object list of class \code{"k_hurst"} with print and plot methods. The list has following components:
#' \describe{
#'   \item{\code{cluster_info}}{A data frame indicating the cluster number and euclidean distance to cluster center of
#'   each smoothed estimated Hurst function (item) }
#'   \item{\code{cluster}}{A vector of cluster number of each item.}
#'   \item{\code{cluster_sizes}}{Number of item in each cluster.}
#'   \item{\code{centers}}{A data frame of cluster centers. Center obtained as the average of each smoothed estimated Hurst function in the cluster.
#'   Columns denote time points in which estimates were obtained. Row names denote cluster numbers.}
#'   \item{\code{smoothed_Hurst_estimates}}{A data frame of smoothed Hurst estimates. Columns denote time points in which estimates were obtained.
#'   Rows denote estimates for each realisation.}
#'   \item{\code{raw_Hurst_estimates}}{A list of data frames of raw Hurst estimates.}
#'   \item{\code{call}}{Information about the input parameters used.}
#' }
#'
#' @details
#' The Hurst function of each realisation is estimated using \code{\link{Hurst}}. The smoothed Hurst estimates are
#' used for k-means clustering in \code{\link[stats]{kmeans}}. The Hartigan and Wong algorithm is used as the
#' default k-means clustering algorithm.
#'
#' @importFrom proxy dist
#' @importFrom stats loess kmeans
#'
#' @export kmeans_hurst
#'
#' @seealso \code{\link{print.k_hurst}}, \code{\link{plot.k_hurst}}, \code{\link{hclust_hurst}}
#'
#' @examples
#' \donttest{
#' #Simulation of multifractional processes
#' t <- seq(0, 1, by = (1/2)^10)
#' H1 <- function(t) {return(0.1 + 0*t)}
#' H2 <- function(t) {return(0.2 + 0.45*t)}
#' H3 <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
#' X.list.1 <- replicate(3, GHBMP(t, H1), simplify = FALSE)
#' X.list.2 <- replicate(3, GHBMP(t, H2), simplify = FALSE)
#' X.list.3 <- replicate(3, GHBMP(t, H3), simplify = FALSE)
#' X.list <- c(X.list.1, X.list.2, X.list.3)
#'
#' #K-means clustering based on k = 3 clusters
#' KC <- kmeans_hurst(X.list, k = 3)
#' print(KC)
#'
#' #Plot of smoothed Hurst functions in each cluster with cluster centers
#' plot(KC, type = "ec")
#' }
kmeans_hurst <- function(X.t, k, ..., N = 100, Q = 2, L = 2)
{
  if (!is.list(X.t) | !all(sapply(X.t, is.data.frame))) {
    stop("X.t must be a list of numeric data frames")
  }

  if (!is.numeric(k) | !(k %% 1 == 0) | !(k > 0) | !(k <= length(X.t))) {
    stop(paste("k must be a numeric positive integer between 1 and ", length(X.t)))
  }

  if (!is.numeric(N)) {
    stop("N must be numeric")
  } else if (!(N %% 1 == 0) | !(N > 0)) {
    stop("N must be a positive integer")
  }

  if (!is.numeric(Q)) {
    stop("Q must be numeric")
  } else if (!(Q %% 1 == 0) | !(Q > 1)) {
    stop("Q must be a positive integer greater than 1")
  }

  if (!is.numeric(L)) {
    stop("L must be numeric")
  } else if (!(L %% 1 == 0) | !(L > 1)) {
    stop("L must be a positive integer greater than 1")
  }

  H<-list()

  for (i in 1:length(X.t)) {
    H[[i]] <- Hurst(X.t[[i]], N, Q, L)
  }

  loess.fn<-function(df){
    sm<-loess(df[,2] ~ df[,1], data = df, span = 0.3)
    return(sm$fitted)
  }

  Smooth_df0 <- data.frame(rbind(t(sapply(H, loess.fn))))
  Smooth_df <- as.data.frame(lapply(Smooth_df0, function(x) pmax(pmin(x, 1), 0)))

  km <- kmeans(Smooth_df, k, ...)
  clusters <- km$cluster
  n_cl <- length(unique(clusters))

  DF <- data.frame(Item = row.names.data.frame(Smooth_df), Cluster = factor(clusters))
  DF <- DF[order(clusters), ]
  rownames(DF)<-NULL

  cl_df<-list()
  center<-list()
  df1<-list()
  dist_df<-list()
  df2<-list()

  for(i in 1:n_cl){
    cl_df[[i]] <- Smooth_df[clusters == i,]
    center[[i]] <- colMeans(cl_df[[i]])
    df1[[i]] <- rbind(cl_df[[i]], center[[i]])
    dist_df[[i]] <- as.matrix(dist(df1[[i]]))
    df2[[i]] <- as.numeric(dist_df[[i]][1:(nrow(cl_df[[i]])), nrow(df1[[i]])])
  }

  clust_DF <- data.frame(DF, Distance_from_center = unlist(df2))
  structure(list(h = DF, cluster_info = clust_DF, cluster = clusters, cluster_sizes = as.vector(table(clusters)),
                 centers = as.data.frame(do.call(rbind, center)), smoothed_Hurst_estimates = Smooth_df,
                 raw_Hurst_estimates = H, call = match.call()), class = "k_hurst")


}


#' Print method for "k_hurst" class objects
#'
#' @description
#' Prints the results of k-means clustering of realisations of processes.
#'
#' @param x Object of class \code{"k_hurst"}.
#' @param ... Other arguments.
#'
#' @return Prints an object of class \code{"k_hurst"}.
#'
#' @seealso \code{\link{kmeans_hurst}}
#'
#' @exportS3Method Rmfrac::print
print.k_hurst <- function(x, ...)
{
  cat("K-means clustering with ", length(x$cluster_sizes), " clusters of sizes ",
      paste(x$cluster_sizes, collapse = ", "), "\n", sep = "")
  cat("\nClustering information with the distance from the cluster center:\n")
  print(x$cluster_info)
  cat("\nOther available components:\n", sep = "\n")
  print(names(x))
  invisible(x)
}

#' @importFrom ggplot2 autoplot ggplot facet_wrap geom_line labs aes
#' @importFrom rlang .data
#' @importFrom stats aggregate
#' @export
#'
autoplot.k_hurst <- function(object, ..., type = "estimates")
{
  smth_h <- object$smoothed_Hurst_estimates
  raw_h <- object$raw_Hurst_estimates
  cluster <- object$cluster
  cent <- object$centers

  DF <- data.frame(
    clus = rep(cluster, each = ncol(smth_h)),
    item = rep(1:nrow(smth_h), each = ncol(smth_h)),
    t = unlist(lapply(raw_h, function(df) df[[1]])),
    smth_est = as.vector(t(smth_h))
  )

  DF1 <- aggregate(smth_est ~ clus+t, data = DF, FUN = mean)

  DF1["item"] <- DF1["clus"]

  if (type == "estimates")
  {
    p <- ggplot(DF, aes(.data$t, .data$smth_est, group = .data$item)) +
      facet_wrap(~clus, ncol = 2, scales = "free_x") +
      geom_line(color = "black") +
      labs(title = "Smoothed Hurst estimates in each cluster", x = "Time", y = "Smoothed Hurst estimates")


  }
  else if (type == "centers")
  {
    p <- ggplot(DF1, aes(.data$t, .data$smth_est)) +
      facet_wrap(~clus, ncol = 2, scales = "free_x") +
      geom_line(color = "red") +
      labs(title = "Cluster centers", x = "Time", y = "Smoothed Hurst estimates")


  }
  else if (type == "ec")
  {
    p <- ggplot(DF, aes(.data$t, .data$smth_est, group = .data$item)) +
      geom_line(color = "black") +
      geom_line(data = DF1, aes(.data$t, .data$smth_est), color = "red") +
      facet_wrap(~clus, ncol = 2, scales = "free_x") +
      labs(title = "Smoothed Hurst estimates in each cluster and cluster center", x = "Time", y = "Smoothed Hurst estimates")


  }
  else
  {
    message("Invalid type")
  }

}


#' Plot smoothed Hurst functions in each cluster with cluster centers
#'
#' @description
#' Creates a plot of the smoothed Hurst functions of realisations of processes (or time series) separately in each cluster with cluster centers using the return from
#' \code{\link{kmeans_hurst}}. Options to plot only estimates, only centers or both are available.
#'
#' @param x Return from \code{\link{kmeans_hurst}}.
#' @param type The type of plot required.
#' \describe{
#' \item{\code{"estimates"}}{Only the smoothed Hurst functions in each cluster.}
#' \item{\code{"centers"}}{Only the cluster centers. Center denotes average of all smoothed Hurst functions in the cluster.}
#' \item{\code{"ec"}}{Both \code{"estimates"} and \code{"centers"}.}
#' }
#' @param ... Other arguments.
#'
#' @return A ggplot object which is used to plot the relevant \code{type} of plot: \code{"estimates"}, \code{"centers"} or \code{"ec"}.
#' @exportS3Method Rmfrac::plot
#' @importFrom ggplot2 ggplot facet_wrap geom_line labs aes
#' @importFrom rlang .data
#'
#' @seealso \code{\link{kmeans_hurst}}
#'
#' @examples
#' \donttest{
#' #Simulation of multifractional processes
#' t <- seq(0, 1, by = (1/2)^10)
#' H1 <- function(t) {return(0.1 + 0*t)}
#' H2 <- function(t) {return(0.2 + 0.45*t)}
#' H3 <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
#' X.list.1 <- replicate(3, GHBMP(t, H1), simplify = FALSE)
#' X.list.2 <- replicate(3, GHBMP(t, H2), simplify = FALSE)
#' X.list.3 <- replicate(3, GHBMP(t, H3), simplify = FALSE)
#' X.list <- c(X.list.1, X.list.2, X.list.3)
#'
#' #K-means clustering based on k=3 clusters
#' KC <- kmeans_hurst(X.list, k = 3)
#' print(KC)
#'
#' #Plot of smoothed Hurst functions in each cluster with cluster centers
#' plot(KC, type = "ec")
#' }
plot.k_hurst <- function(x, type = "estimates", ...) {

  pp <- (autoplot(x, type = type))

  if (interactive()) {
    print(pp)
  }

  invisible(pp)
}

#' Estimated sojourn measure
#'
#' @description Computes the estimated sojourn measure for a time series \eqn{X(t)} greater or lower than the
#' constant level \code{A} for the provided time interval or its sub-interval.
#'
#' @param X Data frame where the first column is a numeric time sequence \eqn{t}
#' and the second one is the values of the time series \eqn{X(t)}.
#' @param A Constant level as a numeric value.
#' @param N Number of steps on the time interval (or time sub-interval) used for computations. Default set to 10000.
#' @param level A vector of character strings which specifies which sojourn
#' measure required for \code{X}, \code{"greater"} or \code{"lower"} than \code{A}. Default set to \code{"greater"}.
#' @param subI Time sub-interval is a vector, where the lower bound is
#' the first element and the upper bound is the second. Optional: If provided,
#' the estimated sojourn measure for the sub-interval is returned, otherwise the whole time interval is considered.
#' @param plot Logical: If \code{TRUE}, the time series, constant level (in blue) and the sojourn measure (in red) are plotted
#' in interactive sessions.
#'
#' @return Estimated sojourn measure.
#' @export sojourn
#' @importFrom ggplot2 ggplot geom_line geom_hline geom_segment labs aes ggtitle theme element_text
#' @importFrom stats approx aggregate
#' @importFrom rlang .data
#' @seealso \code{\link{exc_Area}}
#' @examples
#' t <- seq(0, 1, length = 1000)
#' TS <- data.frame("t" = t,"X(t)" = rnorm(1000))
#' sojourn(TS, 0.8, level = 'lower', subI = c(0.5, 0.8), plot = TRUE)
sojourn <- function(X, A, N = 10000, level = 'greater', subI = NULL, plot = FALSE){

  if (!is.data.frame(X) | !ncol(X) == 2 | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0)))
  {
    stop("X must be a numeric data frame with time sequence given as the first column")
  }

  if (!is.numeric(A)){
    stop("A must be numeric")
  }

  if (!is.numeric(N)) {
    stop("N must be numeric")
  } else if (!(N %% 1 == 0) | !(N > 0)) {
    stop("N must be a positive integer")
  }

  if (!level %in% c("greater", "lower")) {
    stop("level should be 'greater' or 'lower'")
  }

  if (!is.logical(plot)) {
    stop("plot should have logical inputs either TRUE or FALSE")
  }

  X <- na.omit(X)
  X <- X[order(X[,1]),]
  colnames(X) <- c("x", "y")

  if (is.null(subI)){

    ag_df <- aggregate(X[,2] ~ X[,1], FUN = mean)
    t <- seq(ag_df[1, 1], ag_df[nrow(ag_df), 1], length.out = N + 1)
    int_X <- approx(x = ag_df[,1], y=ag_df[,2], xout = t)$y
    diff<-((ag_df[nrow(ag_df), 1] - ag_df[1, 1]) / N)

    if(level == 'greater'){

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:N){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          S <- S + diff

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 >= A) && (x2 < A)){

          S <- S + (diff * (x1 - A) / (x1 - x2))

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (x1 - A) / (x1 - x2))

        } else if ((x1 < A) && (x2 >= A)){

          S <- S + (diff * (A - x2) / (x1 - x2))

          seg$T_start[i] <- t2 - (diff * (A - x2) / (x1 - x2))
          seg$T_end[i] <- t2

        } else {

          S <- S

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }
    }

    if(level == 'lower'){

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:N){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){

          S <- S + diff

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 <= A) && (x2 > A)){

          S <- S + (diff * (A - x1) / (x2 - x1))

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (A - x1) / (x2 - x1))

        } else if ((x1 > A) && (x2 <= A)){

          S <- S + (diff * (x2 - A) / (x2 - x1))

          seg$T_start[i] <- t2 - (diff * (x2 - A) / (x2 - x1))
          seg$T_end[i] <- t2

        } else {

          S <- S

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }

    }

    seg <- na.omit(seg)
    if (isTRUE(plot) && interactive()) {

      p<- ggplot(X, aes(x = .data$x, y = .data$y)) +
        geom_line() +
        geom_hline(yintercept = A, color="blue", linetype = "dashed") +
        labs(y = "X(t)", x = "t") +
        ggtitle(sprintf("Excursion region where the realisation is over the level %s", A)) +
        theme(plot.title = element_text(size = 10))

      if (nrow(seg) > 0){

        p <- p + geom_segment(data = seg, aes(x = .data$T_start, xend = .data$T_end, y = 0, yend = 0), color="red")
      }

      print(p)
    }

    return(S)

  }
  else{

    if (!is.numeric(subI) | !is.vector(subI) | !length(subI) == 2 | !(all(subI[1] >= X[1, 1] & subI[2] <= X[nrow(X), 1]))){
      stop("subI must be a numeric vector")
    }

    Time<-X[,1]
    X.I<-subset(X, Time >= subI[1] & Time <= subI[2])

    ag_df <- aggregate(X.I[,2]~X.I[,1], FUN = mean)
    t <- seq(ag_df[1, 1], ag_df[nrow(X.I), 1], length.out = N + 1)
    int_X <- approx(x = ag_df[,1], y=ag_df[,2], xout=t)$y
    diff<-((ag_df[nrow(ag_df), 1] - ag_df[1, 1]) / N)

    if(level == 'greater'){

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:N){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          S <- S + diff

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 >= A) && (x2 < A)){

          S <- S + (diff * (x1 - A) / (x1 - x2))

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (x1 - A) / (x1 - x2))

        } else if ((x1 < A) && (x2 >= A)){

          S <- S + (diff * (A - x2) / (x1 - x2))

          seg$T_start[i] <- t2 - (diff * (A - x2) / (x1 - x2))
          seg$T_end[i] <- t2

        } else {

          S <- S

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }}

    if(level == 'lower'){

      S <- 0
      seg <- data.frame(T_start = rep(NA_real_, N), T_end = rep(NA_real_, N))

      for(i in 1:N){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){

          S <- S + diff

          seg$T_start[i] <- t1
          seg$T_end[i] <- t2

        } else if ((x1 <= A) && (x2 > A)){

          S <- S + (diff * (A - x1) / (x2 - x1))

          seg$T_start[i] <- t1
          seg$T_end[i] <- t1 + (diff * (A - x1) / (x2 - x1))

        } else if ((x1 > A) && (x2 <= A)){

          S <- S + (diff * (x2 - A) / (x2 - x1))

          seg$T_start[i] <- t2 - (diff * (x2 - A) / (x2 - x1))
          seg$T_end[i] <- t2

        } else {

          S <- S

          seg$T_start[i] <- NA
          seg$T_end[i] <- NA
        }

      }

    }

    seg <- na.omit(seg)

    if (isTRUE(plot) && interactive()) {

      p<- ggplot(X.I, aes(x = .data$x, y = .data$y)) +
        geom_line() +
        geom_hline(yintercept = A, color="blue", linetype = "dashed") +
        labs(y="X(t)", x="t") +
        ggtitle(sprintf("Excursion region where the realisation is over the level %s", A)) +
        theme(plot.title = element_text(size = 10))

      if (nrow(seg) > 0){

        p <- p + geom_segment(data = seg, aes(x = .data$T_start, xend = .data$T_end, y = 0, yend = 0), color="red")
      }

      print(p)
    }

    return(S)

  }

}


#' Excursion area
#'
#' @description
#' Computes the excursion area where a time series \eqn{X(t)} is greater or lower than the
#' constant level \code{A} for the provided time interval or its sub-interval.
#'
#' @param X Data frame where the first column is a numeric time sequence \eqn{t}
#' and the second one is the values of the time series \eqn{X(t)}.
#' @param A Constant level as a numeric value.
#' @param N Number of steps on the time interval (or time sub-interval) used for computations. Default set to 10000.
#' @param level A vector of character strings which specifies whether the excursion
#' area is required for \code{X}, \code{"greater"} or \code{"lower"} than \code{A}. Default set to \code{"greater"}.
#' @param subI Time sub-interval is a vector, where the lower bound is
#' the first element and the upper bound is the second. Optional: If provided, the excursion area
#' for the sub-interval is returned, otherwise the whole time interval is considered.
#' @param plot Logical: If \code{TRUE}, the time series, constant level and excursion area are plotted in interactive sessions.
#'
#' @return Excursion area.
#' @importFrom ggplot2 ggplot geom_line geom_hline geom_polygon labs aes ggtitle theme element_text
#' @importFrom stats approx aggregate
#' @importFrom rlang .data
#'
#' @export exc_Area
#'
#' @seealso \code{\link{sojourn}}
#'
#' @examples
#' t <- seq(0, 1, length = 1000)
#' TS <- data.frame("t" = t, "X(t)" = rnorm(1000))
#' exc_Area(TS, 0.8, level = 'lower', subI = c(0.5, 0.8), plot = TRUE)
#'
exc_Area <- function(X, A, N = 10000, level = 'greater', subI = NULL, plot = FALSE){

  if (!is.data.frame(X) | !ncol(X) == 2 | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0)))
  {
    stop("X must be a numeric data frame with time sequence given as the first column")
  }

  if (!is.numeric(A)){
    stop("A must be numeric")
  }

  if (!is.numeric(N)) {
    stop("N must be numeric")
  } else if (!(N %% 1 == 0) | !(N > 0)) {
    stop("N must be a positive integer")
  }

  if (!level %in% c("greater", "lower")) {
    stop("level should be 'greater' or 'lower'")
  }

  if (!is.logical(plot)) {
    stop("plot should have logical inputs either TRUE or FALSE")
  }

  X <- na.omit(X)
  X <- X[order(X[,1]),]
  colnames(X) <- c("x", "y")

  if (is.null(subI)){

    ag_df <- aggregate(X[,2] ~ X[,1], FUN = mean)
    t <- seq(ag_df[1, 1], ag_df[nrow(ag_df), 1], length.out = N + 1)
    int_X <- approx(x = ag_df[,1],y = ag_df[,2], xout = t)$y
    diff<-((ag_df[nrow(ag_df), 1] - ag_df[1, 1]) / N)


    if(level == 'greater'){

      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:N){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          Area = Area + (diff * (((x1 - A) + (x2 - A)) / 2))

          DF_Area <- rbind(DF_Area,data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2,X_t = x2))}

        } else if ((x1 >= A) && (x2 < A)){

          Area = Area + ((diff * (x1 - A)/(x1 - x2)) * ((x1 - A) / 2))

          x_cross <- t1 + (diff * (x1 - A) / (x1 - x2))
          DF_Area <- rbind(DF_Area,data.frame(t = t1, X_t = x1), data.frame(t = x_cross, X_t = A), data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 < A) && (x2 >= A)){

          Area = Area + ((diff * (A - x2) / (x1 - x2)) * ((x2 - A) / 2))

          x_cross <- t1 + (diff * (A - x2) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        } else {

          Area = Area

        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = ag_df[nrow(ag_df), 1],X_t = ag_df[nrow(ag_df), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)
    }

    if(level == 'lower'){

      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){

          Area <- Area + (diff * (((A - x1) + (A - x2)) / 2))

          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 <= A) && (x2 > A)){

          Area <- Area + ((diff * (A - x1)/(x2 - x1)) * ((A - x1) / 2))

          x_cross <- t1+ (diff * (A - x1)/(x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = t1,X_t = x1), data.frame(t = x_cross, X_t = A), data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 > A) && (x2 <= A)){

          Area <- Area + ((diff * (x2 - A) / (x2 - x1)) * ((A - x2) / 2))

          x_cross <- t1 + (diff * (x2 - A) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        } else {

          Area <- Area
        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = ag_df[nrow(ag_df), 1],X_t = ag_df[nrow(ag_df), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)

    }

    DF_Area <- na.omit(DF_Area)

    if (isTRUE(plot) && interactive()) {
      p<- ggplot(X, aes(x = .data$x, y = .data$y)) +
        geom_line() +
        geom_hline(yintercept = A,color="blue",linetype = "dashed") +
        labs(y = "X(t)", x = "t") +
        ggtitle(sprintf("Excursion area of the realisation over the level %s", A)) +
        theme(plot.title = element_text(size = 10))


      if (!is.null(DF_Area) && nrow(DF_Area)>0){

        p <- p + geom_polygon(data=DF_Area, aes(x = .data$t,y = .data$X_t, group = .data$G), fill = "lightblue")
      }

      print(p)
    }

    return(Area)

  }
  else{

    if (!is.numeric(subI) | !is.vector(subI) | !length(subI) == 2 | !(all(subI[1] >= X[1,1] & subI[2] <= X[nrow(X), 1]))){
      stop("subI must be a numeric vector")
    }

    Time<-X[,1]
    X.I<-subset(X, Time >= subI[1] & Time <= subI[2])

    ag_df <- aggregate(X.I[,2] ~ X.I[,1], FUN = mean)
    t <- seq(ag_df[1,1], ag_df[nrow(X.I), 1], length.out = N + 1)
    int_X <- approx(x=ag_df[,1], y = ag_df[,2], xout = t)$y
    diff<-((ag_df[nrow(ag_df), 1] - ag_df[1, 1]) / N)

    if(level == 'greater'){
      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 >= A) && (x2 >= A)){

          Area <- Area + (diff * (((x1 - A) + (x2 - A)) / 2))

          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2, X_t = x2))}

        } else if ((x1 >= A) && (x2 < A)){

          Area <- Area + ((diff * (x1 - A) / (x1 - x2)) * ((x1 - A) / 2))

          x_cross <- t1 + (diff * (x1 - A) / (x1 - x2))
          DF_Area <- rbind(DF_Area,data.frame(t = t1, X_t = x1), data.frame(t = x_cross, X_t = A), data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0),X_t = numeric(0))

        } else if ((x1 < A) && (x2 >= A)){

          Area <- Area + ((diff * (A - x2) / (x1 - x2)) * ((x2 - A) / 2))

          x_cross <- t1 + (diff * (A - x2) / (x1 - x2))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        } else {

          Area <- Area

        }

      }

      if (!is.null(DF_Area) && (nrow(DF_Area) > 0)) {
        DF_Area <- rbind(DF_Area, data.frame(t = ag_df[nrow(ag_df), 1], X_t = ag_df[nrow(ag_df), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)
    }

    if(level == 'lower'){

      Area <- 0
      DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))
      G <- 1
      polygon <- list()

      for(i in 1:(N)){

        x1 <- int_X[i]
        x2 <- int_X[i + 1]

        t1 <- t[i]
        t2 <- t[i + 1]

        if((x1 <= A) && (x2 <= A)){

          Area <- Area + (diff * (((A - x1) + (A - x2)) / 2))

          DF_Area <- rbind(DF_Area, data.frame(t = t1, X_t = x1))
          if (i == N) {
            DF_Area <- rbind(DF_Area, data.frame(t = t2,X_t = x2))}

        } else if ((x1 <= A) && (x2 > A)){

          Area <- Area + ((diff * (A - x1)/(x2 - x1)) * ((A - x1) / 2))

          x_cross <- t1 + (diff * (A - x1) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = t1,X_t = x1), data.frame(t = x_cross, X_t = A), data.frame(t = x_cross, X_t = A))

          DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
          DF_Area$G <- G
          polygon[[G]] <- DF_Area
          G <- G + 1
          DF_Area <- data.frame(t = numeric(0), X_t = numeric(0))

        } else if ((x1 > A) && (x2 <= A)){

          Area <- Area + ((diff * (x2 - A) / (x2 - x1)) * ((A - x2) / 2))

          x_cross <- t1 + (diff * (x2 - A) / (x2 - x1))
          DF_Area <- rbind(DF_Area, data.frame(t = x_cross, X_t = A))

        } else {

          Area <- Area
        }

      }

      if (!is.null(DF_Area) && nrow(DF_Area) > 0) {
        DF_Area <- rbind(DF_Area, data.frame(t = ag_df[nrow(ag_df), 1],X_t = ag_df[nrow(ag_df), 2]))
        DF_Area <- rbind(DF_Area, data.frame(t = rev(DF_Area$t), X_t = rep(A, nrow(DF_Area))))
        DF_Area$G <- G
        polygon[[G]] <- DF_Area
      }

      DF_Area <- do.call(rbind, polygon)

    }

    DF_Area <- na.omit(DF_Area)

    if (isTRUE(plot) && interactive()) {

      p<- ggplot(X.I, aes(x = .data$x, y = .data$y)) +
        geom_line() +
        geom_hline(yintercept = A,color="blue",linetype = "dashed") +
        labs(y = "X(t)", x = "t") +
        ggtitle(sprintf("Excursion area of the realisation over the level %s", A)) +
        theme(plot.title = element_text(size = 10))


      if (!is.null(DF_Area) && nrow(DF_Area) > 0){

        p <- p + geom_polygon(data = DF_Area, aes(x = .data$t, y = .data$X_t, group = .data$G), fill = "lightblue")
      }

      print(p)

    }

    return(Area)

  }
}


#' Estimated maximum of a time series
#'
#' @description
#' This function computes the maximum of a time series for the provided
#' time interval or its sub-interval.
#'
#' @param X  Data frame where the first column is a numeric time sequence \eqn{(t)}
#' and the second the values of the time series \eqn{(X(t))}.
#' @param subI Time sub-interval is a vector where the lower bound is
#' the first element and upper bound is the second. Optional: If provided maximum of the
#' sub-interval is returned, otherwise the whole time sequence is considered.
#' @param plot Logical: If \code{TRUE}, the time series, the maximum and corresponding \eqn{t} values are plotted in interactive sessions.
#' @param hline Logical: If \code{TRUE}, a horizontal line is plotted across the maximum.
#' @param vline Logical: If \code{TRUE}, a vertical line is plotted across the maximum.
#'
#' @return A list of numeric vector(s). The first element in the vector is the corresponding \eqn{t} value and second the
#' maximum of the time series.
#' @importFrom ggplot2 ggplot geom_line geom_point geom_vline geom_hline labs aes
#' @importFrom rlang .data
#' @export X_max
#'
#' @seealso \code{\link{X_min}}
#'
#' @examples
#' t <- seq(0, 1, length = 100)
#' TS <- data.frame("t" = t, "X(t)" = rnorm(100))
#' X_max(TS, subI = c(0.5, 0.8), plot = TRUE)
#'
X_max <- function(X, subI = NULL, plot = FALSE, vline = FALSE, hline = FALSE){

  if (!is.data.frame(X) | !ncol(X) == 2 | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0)))
  {
    stop("X must be a numeric data frame with time sequence given as the first column")
  }

  if (!is.logical(plot)) {
    stop("plot should have logical inputs either TRUE or FALSE")
  }

  if (!is.logical(vline))
  {
    stop("vline should have logical inputs either TRUE or FALSE")
  }

  if (!is.logical(hline))
  {
    stop("hline should have logical inputs either TRUE or FALSE")
  }

  X <- na.omit(X)
  X <- X[order(X[,1]), ]
  colnames(X) <- c("x", "y")

  if (is.null(subI)){

    X.maximum <- max(X[,2])
    t.X.maximum <- ((X[,1])[which(X[,2] == X.maximum)])
    max_points_df <- data.frame(t = t.X.maximum, x = rep(X.maximum, length(t.X.maximum)))
    max_return <- apply(max_points_df, 1, function(row) as.numeric(row), simplify = FALSE)

    if (isTRUE(plot) && interactive()) {

      p <- ggplot(X, aes(x = .data$x, y = .data$y)) +
        geom_line() +
        geom_point(data = max_points_df, aes(x = .data$t, y = .data$x), color = "red", size = 1.5) +
        labs(x = "t", y = "X(t)")

      if(vline){
        p <- p + geom_vline(xintercept = t.X.maximum, linetype = "dashed", color = "blue")
      }

      if(hline){
        p <- p + geom_hline(yintercept = X.maximum, linetype = "dashed", color = "blue")
      }

      if (hline && vline){
        p <- p +
          geom_hline(yintercept = X.maximum, linetype = "dashed", color = "blue") +
          geom_vline(xintercept = t.X.maximum, linetype = "dashed", color = "blue")
      }

      print(p)
    }

  }

  else{

    if (!is.numeric(subI) | !is.vector(subI) | !length(subI) == 2 | !(all(subI[1] >= X[1,1] & subI[2] <= X[nrow(X),1]))){
      stop("subI must be a numeric vector")
    }
    Time <- X[,1]
    X.I <- subset(X, Time >= subI[1] & Time <= subI[2])
    X.maximum <- max(X.I[,2])
    t.X.maximum <- ((X.I[,1])[which(X.I[,2] == X.maximum)])
    max_points_df <- data.frame(t = t.X.maximum, x = rep(X.maximum, length(t.X.maximum)))
    max_return <- apply(max_points_df,1, function(row) as.numeric(row),simplify = FALSE)

    if (isTRUE(plot) && interactive()) {

      p <- ggplot(X.I, aes(x = .data$x, y = .data$y)) +
        geom_line() +
        geom_point(data = max_points_df, aes(x = .data$t, y = .data$x), color = "red", size = 1.5) +
        labs(x = "t",y = "X(t)")

      if(vline){
        p <- p + geom_vline(xintercept = t.X.maximum, linetype = "dashed", color = "blue")
      }

      if(hline){
        p <- p + geom_hline(yintercept = X.maximum, linetype = "dashed", color = "blue")
      }

      if (hline && vline){
        p <- p + geom_hline(yintercept = X.maximum, linetype = "dashed", color = "blue") +
          geom_vline(xintercept = t.X.maximum, linetype = "dashed", color = "blue")
      }

      print(p)
    }

  }

  return(max_return)
}


#' Estimated minimum of a time series
#'
#' @description
#' This function computes the minimum of a time series for the provided
#' time interval or its sub-interval.
#'
#' @param X  Data frame where the first column is a numeric time sequence \eqn{t}
#' and the second the values of the time series \eqn{X(t)}.
#' @param subI Time sub-interval is a vector where the lower bound is
#' the first element and upper bound is the second. Optional: If provided minimum of the
#' sub-interval is returned, otherwise the whole time interval is considered.
#' @param plot Logical: If \code{TRUE}, the time series, the minimum and corresponding \eqn{t} values are plotted in interactive sessions.
#' @param hline Logical: If \code{TRUE}, a horizontal line is plotted across the minimum.
#' @param vline Logical: If \code{TRUE}, a vertical line is plotted across the minimum.
#'
#' @return A list of numeric vector(s). The first element in the vector is the corresponding \eqn{t} value and second the
#' minimum of the time series.
#' @importFrom ggplot2 ggplot geom_line geom_point geom_vline geom_hline labs aes
#' @importFrom rlang .data
#'
#' @export X_min
#'
#' @seealso \code{\link{X_max}}
#'
#' @examples
#' t <- seq(0, 1, length = 100)
#' TS <- data.frame("t" = t, "X(t)" = rnorm(100))
#' X_min(TS, subI = c(0.2, 0.8), plot = TRUE)
#'
X_min <- function(X, subI = NULL, plot = FALSE, vline = FALSE, hline = FALSE){

  if (!is.data.frame(X) | !ncol(X) == 2 | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0)))
  {
    stop("X must be a numeric data frame with time sequence given as the first column")
  }

  if (!is.logical(plot)) {
    stop("plot should have logical inputs either TRUE or FALSE")
  }

  if (!is.logical(vline))
  {
    stop("vline should have logical inputs either TRUE or FALSE")
  }

  if (!is.logical(hline))
  {
    stop("hline should have logical inputs either TRUE or FALSE")
  }

  X <- na.omit(X)
  X <- X[order(X[,1]), ]
  colnames(X) <- c("x", "y")

  if (is.null(subI)){

    X.minimum <- min(X[,2])
    t.X.minimum <- ((X[,1])[which(X[,2] == X.minimum)])
    min_points_df <- data.frame(t = t.X.minimum, x = rep(X.minimum, length(t.X.minimum)))
    min_return <- apply(min_points_df, 1, function(row) as.numeric(row),simplify = FALSE)

    if (isTRUE(plot) && interactive()) {

      p <- ggplot(X, aes(x = .data$x, y = .data$y)) +
        geom_line() +
        geom_point(data = min_points_df, aes(x = .data$t, y = .data$x), color = "red", size = 1.5) +
        labs(x = "t", y = "X(t)")

      if(vline){
       p <- p + geom_vline(xintercept = t.X.minimum, linetype = "dashed", color = "blue")
      }

      if(hline){
        p <- p + geom_hline(yintercept = X.minimum, linetype = "dashed", color = "blue")
      }

      if (hline && vline){
        p <- p + geom_hline(yintercept = X.minimum, linetype = "dashed", color = "blue") +
          geom_vline(xintercept = t.X.minimum, linetype = "dashed", color = "blue")
      }

      print(p)
    }

  }

  else{

    if (!is.numeric(subI) | !is.vector(subI) | !length(subI) == 2 | !(all(subI[1] >= X[1,1] & subI[2] <= X[nrow(X),1]))){
      stop("subI must be a numeric vector")
    }
    Time <- X[,1]
    X.I<-subset(X, Time >= subI[1] & Time <= subI[2])
    X.minimum <- min(X.I[,2])
    t.X.minimum <- ((X.I[,1])[which(X.I[,2] == X.minimum)])
    min_points_df <- data.frame(t = t.X.minimum, x = rep(X.minimum, length(t.X.minimum)))
    min_return <- apply(min_points_df, 1, function(row) as.numeric(row),simplify = FALSE)

    if (isTRUE(plot) && interactive()) {
      p <- ggplot(X.I, aes(x = .data$x, y = .data$y)) +
        geom_line() +
        geom_point(data = min_points_df, aes(x = .data$t, y = .data$x), color = "red", size = 1.5) +
        labs(x = "t", y = "X(t)")

      if(vline){
        p <- p + geom_vline(xintercept = t.X.minimum, linetype = "dashed", color = "blue")
      }

      if(hline){
        p <- p + geom_hline(yintercept = X.minimum, linetype = "dashed", color = "blue")
      }

      if (hline && vline){
        p <- p +
          geom_hline(yintercept = X.minimum, linetype = "dashed", color = "blue") +
          geom_vline(xintercept = t.X.minimum, linetype = "dashed", color = "blue")
      }

      print(p)
    }

  }

  return(min_return)
}

#sojourn

test_that("sojourn works", {
  expect_equal(sojourn(TS_Ana_1, 0.8, N = 100), 0.2489572, tolerance = 1e-6)
  expect_equal(sojourn(TS_Ana_1, 0.8, N = 100, subI = c(0.2,0.5)), 0.05664624, tolerance = 1e-6)
  expect_equal(sojourn(TS_Ana_1, 0.8, N = 100, level = 'lower'), 0.7510428, tolerance = 1e-6)
  expect_equal(sojourn(TS_Ana_1, 0.8, N = 100, level = 'lower', subI = c(0.2,0.5)), 0.165576, tolerance = 1e-6)
})

test_that("sojourn fails if X is not a data frame", {
  expect_error(sojourn(c(0.2788, 0.1365, 0.5567), 0.8, N = 10))
})

test_that("sojourn fails if X is a data frame with more than or less than 2 columns", {
  expect_error(sojourn(data.frame(x = c(0.2788, 0.1365, 0.5567)), 0.8, N = 10))
  expect_error(sojourn(data.frame(t = c(1, 2, 3), x = c(0.2788, 0.1365, 0.5567), y = c(0.2788,0.1365,0.5567)), 0.8, N = 10))
})

test_that("sojourn fails if X is a nonnumeric data frame", {
  expect_error(sojourn(data.frame(t = c("A", "B", "C"), x = c(0.2788, 0.1365, 0.5567)), 0.8, N = 10))
  expect_error(sojourn(data.frame(t = c(1, 2, 3), x = c(0.2788, "A", 0.5567)), 0.8, N = 10))
})

test_that("sojourn fails if X[,1] have negative values", {
  expect_error(sojourn(data.frame(t = c(1, -2, 3), x = c(0.2788, 0.1365, 0.5567)), 0.8, N = 10))
})

test_that("sojourn fails if A is nonnumeric", {
  expect_error(sojourn(TS_Ana_1, "A", N = 10))
})

test_that("sojourn fails if N is nonnumeric", {
  expect_error(sojourn(TS_Ana_1, 0.8, N = "A"))
})

test_that("sojourn fails for negative N", {
  expect_error(sojourn(TS_Ana_1, 0.8, N = -10))
})

test_that("sojourn fails for non integer N", {
  expect_error(sojourn(TS_Ana_1, 0.8, N = 2.5))
})

test_that("sojourn fails for irrelavant inpuTS_Ana_1 for level", {
  expect_error(sojourn(TS_Ana_1, 0.8, N = 10, level = "high"))
})

test_that("sojourn fails if subI is not a numeric vector with 2 elemenTS_Ana_1", {
  expect_error(sojourn(TS_Ana_1, 0.8, N = 10, subI = c(0,1, "A")))
  expect_error(sojourn(TS_Ana_1, 0.8, N = 10, subI = c(0,1, 0.2, 0.3)))
})

test_that("sojourn fails if the second element of subI is larger than the first", {
  expect_error(sojourn(TS_Ana_1, 0.8, N = 10, subI = c(0,5, 0.2)))
})

test_that("sojourn fails for non logical plot", {
  expect_error(sojourn(TS_Ana_1, 0.8, N = 10, plot = 1))
  expect_error(sojourn(TS_Ana_1, 0.8, N = 10, plot = "TRUE"))
})

#exc_Area

test_that("exc_Area works", {
  expect_equal(exc_Area(TS_Ana_1, 0.8, N = 100), 0.1052229, tolerance = 1e-6)
  expect_equal(exc_Area(TS_Ana_1, 0.8, N = 100, subI = c(0.2,0.5)), 0.02148899, tolerance = 1e-6)
  expect_equal(exc_Area(TS_Ana_1, 0.8, N = 100, level = 'lower'), 0.7664732, tolerance = 1e-6)
  expect_equal(exc_Area(TS_Ana_1, 0.8, N = 100, level = 'lower', subI = c(0.2,0.5)), 0.09765494, tolerance = 1e-6)
})

test_that("exc_Area fails if X is not a data frame", {
  expect_error(exc_Area(c(0.2788, 0.1365, 0.5567), 0.8, N = 10))
})

test_that("exc_Area fails if X is a data frame with more than or less than 2 columns", {
  expect_error(exc_Area(data.frame(x = c(0.2788, 0.1365, 0.5567)), 0.8, N = 10))
  expect_error(exc_Area(data.frame(t = c(1, 2, 3), x = c(0.2788, 0.1365, 0.5567), y = c(0.2788,0.1365,0.5567)), 0.8, N = 10))
})

test_that("exc_Area fails if X is a nonnumeric data frame", {
  expect_error(exc_Area(data.frame(t = c("A", "B", "C"), x = c(0.2788, 0.1365, 0.5567)), 0.8, N = 10))
  expect_error(exc_Area(data.frame(t = c(1, 2, 3), x = c(0.2788, "A", 0.5567)), 0.8, N = 10))
})

test_that("exc_Area fails if X[,1] have negative values", {
  expect_error(exc_Area(data.frame(t = c(1, -2, 3), x = c(0.2788, 0.1365, 0.5567)), 0.8, N = 10))
})

test_that("exc_Area fails if A is nonnumeric", {
  expect_error(exc_Area(TS_Ana_1, "A", N = 10))
})

test_that("exc_Area fails if N is nonnumeric", {
  expect_error(exc_Area(TS_Ana_1, 0.8, N = "A"))
})

test_that("exc_Area fails for negative N", {
  expect_error(exc_Area(TS_Ana_1, 0.8, N = -10))
})

test_that("exc_Area fails for non integer N", {
  expect_error(exc_Area(TS_Ana_1, 0.8, N = 2.5))
})

test_that("exc_Area fails for irrelavant inpuTS_Ana_1 for level", {
  expect_error(exc_Area(TS_Ana_1, 0.8, N = 10, level = "high"))
})

test_that("exc_Area fails if subI is not a numeric vector with 2 elemenTS_Ana_1", {
  expect_error(exc_Area(TS_Ana_1, 0.8, N = 10, subI = c(0,1, "A")))
  expect_error(exc_Area(TS_Ana_1, 0.8, N = 10, subI = c(0,1, 0.2, 0.3)))
})

test_that("exc_Area fails if the second element of subI is larger than the first", {
  expect_error(exc_Area(TS_Ana_1, 0.8, N = 10, subI = c(0,5, 0.2)))
})

test_that("exc_Area fails for non logical plot", {
  expect_error(exc_Area(TS_Ana_1, 0.8, N = 10, plot = 1))
  expect_error(exc_Area(TS_Ana_1, 0.8, N = 10, plot = "TRUE"))
})

#X_max

test_that("X_max works", {
  expect_equal(X_max(TS_Ana_1), list(c(0.5555556, 1.7150650)), tolerance = 1e-6)
  expect_equal(X_max(TS_Ana_1, subI = c(0.5, 0.8)), list(c(0.5555556, 1.7150650)),  tolerance = 1e-6)
})

test_that("X_max fails if X is not a data frame", {
  expect_error(X_max(c(0.2788, 0.1365, 0.5567)))
})

test_that("X_max fails if X is a data frame with more than or less than 2 columns", {
  expect_error(X_max(data.frame(x = c(0.2788, 0.1365, 0.5567))))
  expect_error(X_max(data.frame(t = c(1, 2, 3), x = c(0.2788, 0.1365, 0.5567), y = c(0.2788,0.1365,0.5567))))
})

test_that("X_max fails if X is a nonnumeric data frame", {
  expect_error(X_max(data.frame(t = c("A", "B", "C"), x = c(0.2788, 0.1365, 0.5567))))
  expect_error(X_max(data.frame(t = c(1, 2, 3), x = c(0.2788, "A", 0.5567))))
})

test_that("X_max fails if X[,1] have negative values", {
  expect_error(X_max(data.frame(t = c(1, -2, 3), x = c(0.2788, 0.1365, 0.5567))))
})

test_that("X_max fails if subI is not a numeric vector with 2 elemenTS_Ana_1", {
  expect_error(X_max(TS_Ana_1, subI = c(0,1, "A")))
  expect_error(X_max(TS_Ana_1, subI = c(0,1, 0.2, 0.3)))
})

test_that("X_max fails if the second element of subI is larger than the first", {
  expect_error(X_max(TS_Ana_1, subI = c(0,5, 0.2)))
})

test_that("X_max fails for non logical plot", {
  expect_error(X_max(TS_Ana_1, plot = 1))
  expect_error(X_max(TS_Ana_1, plot = "TRUE"))
})

test_that("X_max fails for non logical vline", {
  expect_error(X_max(TS_Ana_1, vline = 1))
  expect_error(X_max(TS_Ana_1, vline = "TRUE"))
})

test_that("X_max fails for non logical hline", {
  expect_error(X_max(TS_Ana_1, hline = 1))
  expect_error(X_max(TS_Ana_1, hline = "TRUE"))
})

#X_min

test_that("X_min works", {
  expect_equal(X_min(TS_Ana_1), list(c(0.7777778, -1.2650612)), tolerance = 1e-6)
  expect_equal(X_min(TS_Ana_1, subI = c(0.5, 0.8)), list(c(0.7777778, -1.2650612)),  tolerance = 1e-6)
})

test_that("X_min fails if X is not a data frame", {
  expect_error(X_min(c(0.2788, 0.1365, 0.5567)))
})

test_that("X_min fails if X is a data frame with more than or less than 2 columns", {
  expect_error(X_min(data.frame(x = c(0.2788, 0.1365, 0.5567))))
  expect_error(X_min(data.frame(t = c(1, 2, 3), x = c(0.2788, 0.1365, 0.5567), y = c(0.2788,0.1365,0.5567))))
})

test_that("X_min fails if X is a nonnumeric data frame", {
  expect_error(X_min(data.frame(t = c("A", "B", "C"), x = c(0.2788, 0.1365, 0.5567))))
  expect_error(X_min(data.frame(t = c(1, 2, 3), x = c(0.2788, "A", 0.5567))))
})

test_that("X_min fails if X[,1] have negative values", {
  expect_error(X_min(data.frame(t = c(1, -2, 3), x = c(0.2788, 0.1365, 0.5567))))
})

test_that("X_min fails if subI is not a numeric vector with 2 elemenTS_Ana_1", {
  expect_error(X_min(TS_Ana_1, subI = c(0,1, "A")))
  expect_error(X_min(TS_Ana_1, subI = c(0,1, 0.2, 0.3)))
})

test_that("X_min fails if the second element of subI is larger than the first", {
  expect_error(X_min(TS_Ana_1, subI = c(0,5, 0.2)))
})

test_that("X_min fails for non logical plot", {
  expect_error(X_min(TS_Ana_1, plot = 1))
  expect_error(X_min(TS_Ana_1, plot = "TRUE"))
})

test_that("X_min fails for non logical vline", {
  expect_error(X_min(TS_Ana_1, vline = 1))
  expect_error(X_min(TS_Ana_1, vline = "TRUE"))
})

test_that("X_min fails for non logical hline", {
  expect_error(X_min(TS_Ana_1, hline = 1))
  expect_error(X_min(TS_Ana_1, hline = "TRUE"))
})

set.seed(123)
X.t_Bm <- replicate(10, Bm(N = 10), simplify = FALSE)
X_Bm <- do.call(rbind, lapply(X.t_Bm, function(df) df[, 2]))
t <- seq(0, 1, by = 0.1)
Data_Bm <- data.frame(t, t(X_Bm))

H <- function(t) {return(0.2 + 0.45*t)}

#' Statistical estimation of the Hurst function
#'
#' @description
#' This function computes statistical estimates for the Hurst function.
#'
#' @param X Data frame where the first column is a numeric time sequence and the second the values of the process or time series.
#' @param N Number of sub-intervals on which the estimation is performed on. Default is set to 100 sub-intervals.
#' @param Q Fixed integer greater than or equal to 2. Default is set to 2.
#' @param L Fixed integer greater than or equal to 2. Default is set to 2.
#'
#' @return A data frame of where the first column is a time sequence and second column is estimated values of the Hurst function.
#' @note
#' Since these are estimators of local characteristics, reliable results can only be obtained when a sufficiently large number of points is used.
#' @details
#' Statistical estimation of the Hurst function is done based on the results of Ayache, A.,
#' & Bouly, F. (2023). The pointwise Holder exponent of the process for data considered in
#' the package is equal to the Hurst function. The estimator is built through generalized
#' quadratic variations of the process associated with its increments. The integer parameters
#' Q and L define the generalized quadratic variations and the corresponding estimator.
#'
#'
#' @export Hurst
#'
#' @importFrom zoo rollapply
#' @importFrom matrixStats sum2
#' @importFrom stats na.omit
#'
#' @references Ayache, A. and Bouly, F. (2023). Uniformly and strongly consistent estimation for
#' the random Hurst function of a multifractional process. Latin American Journal of
#' Probability and Mathematical Statistics, 20(2):1587–1614. \doi{doi:10.30757/alea.v20-60}.
#'
#' @seealso \code{\link{LFD}}, \code{H_LFD}, \code{\link{plot.mp}}, \code{\link{plot_tsest}}, \code{\link{plot.H_LFD}}
#'
#' @examples
#' \donttest{
#' #Hurst function of a multifractional process simulated using GHBMP function
#' t <- seq(0, 1, by = (1/2)^10)
#' H <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
#' X <- GHBMP(t, H)
#' Hurst(X)
#' }
#'
#' #Hurst function of a fractional Browian motion simulated using FBm
#' X <- FBm(H = 0.5, x_start = 0, t_start = 0, t_end = 2, N = 1000)
#' Hurst(X)
#'
Hurst <- function(X, N = 100, Q = 2, L = 2)
{
  tmin1 <- min(na.omit(X[,1]))
  tmax1 <- max(na.omit(X[,1]))

  X <- na.omit(X)

  if (!is.data.frame(X) | !ncol(X) == 2 | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0)) ) {
    stop("X must be a numeric data frame with time sequence given as the first column")
  }

  if (!is.numeric(N)) {
    stop("N must be numeric")
  } else if (!(N %% 1 == 0) | !(N > 0)) {
    stop("N must be a positive integer")
  }

  if (!is.numeric(Q)) {
    stop("Q must be numeric")
  } else if (!(Q %% 1 == 0) | !(Q > 1)) {
    stop("Q must be a positive integer greater than 1")
  }

  if (!is.numeric(L)) {
    stop("L must be numeric")
  } else if (!(L %% 1 == 0) | !(L > 1)) {
    stop("L must be a positive integer greater than 1")
  }

  X <- X[order(X[,1]),]

  X_scaled <- X
  X_scaled[,1] <- (X_scaled[,1] - tmin1) / (tmax1 - tmin1)

  tQ <- X_scaled[,1]
  t <- tQ[seq(1, length(tQ), by = Q)]

  l <- 0:L

  al <- (-1)^(L - l) * ((factorial(L)) / ((factorial(l)) * (factorial(L - l))))

  ms <- function(b)
  {
    sum(al * b)
  }

  log_Q <- function(x, y, q)
  {
    (log(x / y)) / log(q^2)
  }

  #val <- ifelse(L<N, "Continue", "Stop")
  #print(val)

  XNQ <- X[,2]
  XN <- XNQ[seq(1, length(XNQ), by = Q)]

  dNk <- rollapply(XN, width = (L + 1), ms)
  dNk2 <- dNk^2
  dNQk <- rollapply(XNQ, width = (L + 1), ms)
  dNQk2 <- dNQk^2

  sub_intervals <- seq(0, 1, 1/N)
  start_points <- sub_intervals[-length(sub_intervals)]
  end_points <- sub_intervals[-1]

  H_est<-function(np)
  {
    I_L <- start_points[np]
    I_U <- end_points[np]
    cx <- floor(I_U * (length(t) - 1)) - ceiling(I_L * (length(t) - 1)) + 1
    cy <- floor(I_U * ((length(tQ) - 1))) - ceiling(I_L * ((length(tQ) - 1))) + 1
    v1 <- ((I_L * (length(t) - 1)) + 1) : (I_U * (length(t) - 1) + 1)
    vx <- (1/cx) * sum2(dNk2, v1)
    v2 <- ((I_L * ((length(tQ) - 1))) + 1) :(I_U*((length(tQ) - 1)) + 1)
    vy <- (1/cy) * sum2(dNQk2, v2)
    a <- min(1, max(log_Q(vx, vy, Q), 0))
    matrix(c(I_L, a))
  }

  H_est_v <- Vectorize(H_est)
  p1 <- H_est_v(1:N)
  est_data <- as.data.frame(t(p1)) #Estimated data for the Hurst function

  est_data[,1] <- ((tmax1 - tmin1) * est_data[,1]) + tmin1

  est_data <- na.omit(est_data)
  colnames(est_data) <- c("Time", "Hurst_estimate")

  return(est_data)

}

#' Estimation of the local fractal dimension
#'
#' @description
#' This function computes the estimates for the local fractal dimension.
#'
#' @param X Data frame where the first column is a numeric time sequence and the second is the values of the process or time series.
#' @param N The same argument that is used for the estimation of Hurst function. Number of sub-intervals on which the estimation is performed on. Default is set to 100 sub-intervals.
#' @param Q The same argument that is used for the estimation of Hurst function. Fixed integer greater than or equal to 2. Default is set to 2.
#' @param L The same argument that is used for the estimation of Hurst function. Fixed integer greater than or equal to 2. Default is set to 2.
#'
#' @return A data frame where the first column is a time sequence and the second column is estimated values of the local fractal dimension.
#' @export LFD
#' @seealso \code{\link{Hurst}}, \code{\link{H_LFD}}, \code{\link{plot.mp}}, \code{\link{plot_tsest}}, \code{\link{plot.H_LFD }}
#' @note
#' Since these are estimators of local characteristics, reliable results can only be obtained when a sufficiently large number of points is used.
#' @details
#' The formula \eqn{\widehat{LFD} = 2-\widehat{H}(t)} is used to compute the estimated local fractal dimension,
#' where \eqn{\widehat{H}(t)} is the estimated Hurst function.
#'
#' @references Gneiting, T., and Schlather, M. (2004). Stochastic models
#' that separate fractal dimension and the Hurst effect. SIAM Review, 46(2):269-282.
#' \doi{doi:10.1137/S0036144501394387}.
#' @examples
#' \donttest{
#' #LFD of a multifractional process simulated using GHBMP function
#' t <- seq(0, 1, by = (1/2)^10)
#' H <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
#' X <- GHBMP(t, H)
#' LFD(X)
#' }
#'
#' #LFD of a fractional Browian motion simulated using FBm
#' X <- FBm(H = 0.5, x_start = 0, t_start = 0, t_end = 2, N = 1000)
#' LFD(X)
#'
LFD <- function(X, N = 100, Q = 2, L = 2)
{
  X <- na.omit(X)

  if (!is.data.frame(X) | !ncol(X) == 2 | !(all(sapply(X, is.numeric))) | !(all(X[[1]] >= 0))) {
    stop("X must be a numeric data frame with time sequence from 0 to 1 given as the first column")
  }

  if (!is.numeric(N)) {
    stop("N must be numeric")
  } else if (!(N %% 1 == 0) | !(N > 0)) {
    stop("N must be a positive integer")
  }

  if (!is.numeric(Q)) {
    stop("Q must be numeric")
  } else if (!(Q %% 1 == 0) | !(Q > 1)) {
    stop("Q must be a positive integer greater than 1")
  }

  if (!is.numeric(L)) {
    stop("L must be numeric")
  } else if (!(L %% 1 == 0) | !(L > 1)) {
    stop("L must be a positive integer greater than 1")
  }

  Hurst_est <- Hurst(X, N, Q, L)

  D <- data.frame(Time = Hurst_est[,1], LFD_estimate = 2 - (Hurst_est[,2]))

  return(D)
}

#Hurst()

test_that("Hurst works", {
  expect_equal(Hurst(df_est)$Hurst_estimate, c(0.0000000, 1.0000000, 0.0000000, 0.6266602, 1.0000000, 1.0000000, 0.4116034), tolerance = 1e-6)
})

test_that("Hurst fails if X is not a data frame", {
  expect_error(Hurst(c(0.2788,0.1365,0.5567)))
})

test_that("Hurst fails if X is a data frame with more than or less than 2 columns", {
  expect_error(Hurst(data.frame(x = c(0.2788,0.1365,0.5567))))
  expect_error(Hurst(data.frame(t = c(1, 2, 3), x = c(0.2788,0.1365,0.5567), y =c(0.2788,0.1365,0.5567))))
})

test_that("Hurst fails if X is a nonnumeric data frame", {
  expect_error(Hurst(data.frame(t = c("A", "B", "C"), x = c(0.2788,0.1365,0.5567))))
  expect_error(Hurst(data.frame(t = c(1, 2, 3), x = c(0.2788,"B",0.5567))))
  })

test_that("Hurst fails if X[,1] have negative values", {
  expect_error(Hurst(data.frame(t = c(1, -2, 3), x = c(0.2788,0.1365,0.5567))))
})

test_that("Hurst fails if N is nonnumeric", {
  expect_error(Hurst(df_est, "a"))
})

test_that("Hurst fails if N is negative", {
  expect_error(Hurst(df_est, -100))
})

test_that("Hurst fails if N is non integer", {
  expect_error(Hurst(df_est, 100.5))
})

test_that("Hurst fails if Q is nonnumeric", {
  expect_error(Hurst(df_est, Q = "a"))
})

test_that("Hurst fails if Q is less than 2", {
  expect_error(Hurst(df_est, Q = -2))
  expect_error(Hurst(df_est, Q = 1))
})

test_that("Hurst fails if Q is non integer", {
  expect_error(Hurst(df_est, Q = 5.5))
})

test_that("Hurst fails if L is nonnumeric", {
  expect_error(Hurst(df_est, L = "a"))
})

test_that("Hurst fails if L is less than 2", {
  expect_error(Hurst(df_est, L = -2))
  expect_error(Hurst(df_est, L = 1))
})

test_that("Hurst fails if L is non integer", {
  expect_error(Hurst(df_est, L = 5.5))
})

#LFD()

test_that("LFD works", {
  expected_est <- data.frame(
      Time = c(0.00, 0.12, 0.24, 0.25, 0.37, 0.49, 0.50),
      LFD_estimate = c(2.000000, 1.000000, 2.000000, 1.373340, 1.000000, 1.000000, 1.588397))
  expect_equal(LFD(df_est), expected_est, tolerance = 1e-5)
})

test_that("LFD fails if X is not a data frame", {
  expect_error(LFD(c(0.2788,0.1365,0.5567)))
})

test_that("Hurst fails if X is a data frame with more than or less than 2 columns", {
  expect_error(LFD(data.frame(x = c(0.2788,0.1365,0.5567))))
  expect_error(LFD(data.frame(t = c(1, 2, 3), x = c(0.2788,0.1365,0.5567), y =c(0.2788,0.1365,0.5567))))
})

test_that("LFD fails if X is a nonnumeric data frame", {
  expect_error(LFD(data.frame(t = c("A", "B", "C"), x = c(0.2788,0.1365,0.5567))))
  expect_error(LFD(data.frame(t = c(1, 2, 3), x = c(0.2788,"B",0.5567))))
})

test_that("LFD fails if X[,1] have negative values", {
  expect_error(LFD(data.frame(t = c(1, -2, 3), x = c(0.2788,0.1365,0.5567))))
})


test_that("LFD fails if N is nonnumeric", {
  expect_error(LFD(df_est, "a"))
})

test_that("LFD fails if N is negative", {
  expect_error(LFD(df_est, -100))
})

test_that("LFD fails if N is non integer", {
  expect_error(LFD(df_est, 100.5))
})

test_that("LFD fails if Q is nonnumeric", {
  expect_error(LFD(df_est, Q = "a"))
})

test_that("LFD fails if Q is less than 2", {
  expect_error(LFD(df_est, Q = -2))
  expect_error(LFD(df_est, Q = 1))
})

test_that("LFD fails if Q is non integer", {
  expect_error(LFD(df_est, Q = 5.5))
})

test_that("LFD fails if L is nonnumeric", {
  expect_error(LFD(df_est, L = "a"))
})

test_that("LFD fails if L is less than 2", {
  expect_error(LFD(df_est, L = -2))
  expect_error(LFD(df_est, L = 1))
})

test_that("LFD fails if L is non integer", {
  expect_error(LFD(df_est, L = 5.5))
})

#hclust_hurst

test_that("hclust_hurst works", {
  expect_equal(sort(HC_actual_1$cluster_sizes), sort(c(3, 1)))
  expect_equal(sort(HC_actual_1$cluster_info[,3]), sort(c(0.0000000, 0.8247233, 0.7271482, 0.7361240)), tolerance = 1e-6)
})

test_that("hclust_hurst works", {
  expect_equal(sort(HC_actual_2$cluster_sizes), sort(c(1, 3)))
  expect_equal(sort(HC_actual_2$cluster_info[,3]), sort(c(0.0000000, 0.8247233, 0.7271482, 0.7361240)), tolerance = 1e-6)
})

test_that("hclust_hurst has the specified class", {
  expect_s3_class(HC_actual_1, c("hc_hurst", "list"))
})

test_that("hclust_hurst fails if X.t is not a list of data frames", {
  expect_error(hclust_hurst(data.frame(t = c(1, 2, 3), x = c(0.2788,0.1365,0.5567)), k = 2))
  expect_error(hclust_hurst(c(0.2788, 0.1365, 0.5567), k = 2))
  expect_error(hclust_hurst(list(0.2788, 0.1365, 0.5567), k = 2))
})

test_that("hclust_hurst fails if any data frame in X.t is not numeric", {
  expect_error(hclust_hurst(list(data.frame(t = c("A", 2, 3), x = c(0.2788,0.1365,0.5567)), data.frame(t = c(1, 2, 3), x = c(-0.2788,0.1365,-0.5567))), k = 1))
  expect_error(hclust_hurst(list(data.frame(t = c(1, 2, 3), x = c("A",0.1365,0.5567)), data.frame(t = c(1, 2, 3), x = c(-0.2788,0.1365,-0.5567))), k = 1))
})

test_that("hclust_hurst fails if any data frame in X.t has less than or more than 2 columns", {
  expect_error(hclust_hurst(list(data.frame(t = c(1, 2, 3)), data.frame(t = c(1, 2, 3), x = c(-0.2788,0.1365,-0.5567))), k = 1))
  expect_error(hclust_hurst(list(data.frame(t = c(1, 2, 3)), data.frame(t = c(1, 2, 3), x = c(-0.2788,0.1365,-0.5567), y = c(0.2788,0.1365,0.5567))), k = 1))
})

test_that("hclust_hurst fails if first column in any data frame in X.t has negative values", {
  expect_error(hclust_hurst(list(data.frame(t = c(-1, 2, 3), x = c(0.2788,0.1365,0.5567)), data.frame(t = c(1, 2, 3), x = c(-0.2788,0.1365,-0.5567))), k = 1))
})

test_that("hclust_hurst fails if either k or h not provided", {
  expect_error(hclust_hurst(X.list))
})

test_that("hclust_hurst fails for irrelevant k", {
  expect_error(hclust_hurst(X.list, k = "A"))
  expect_error(hclust_hurst(X.list, k = 2.5))
  expect_error(hclust_hurst(X.list, k = -2))
  expect_error(hclust_hurst(X.list, k = length(X.list) +2))
})

test_that("hclust_hurst fails if h is nonnumeric", {
  expect_error(hclust_hurst(X.list, h = "A"))
})

test_that("hclust_hurst fails if h is negative", {
  expect_error(hclust_hurst(X.list, h = -1.5))
})

test_that("hclust_hurst fails for incorrect dist.method", {
  expect_error(hclust_hurst(X.list, k = 2, dist.method = "Any"))
})

test_that("hclust_hurst fails for incorrect method", {
  expect_error(hclust_hurst(X.list, k = 2, method = "Any"))
})

test_that("hclust_hurst fails for non logical dendrogram", {
  expect_error(hclust_hurst(X.list, k = 2, dendrogram = 1))
  expect_error(hclust_hurst(X.list, k = 2, dendrogram = "TRUE"))
})

test_that("hclust_hurst fails if N is nonnumeric", {
  expect_error(hclust_hurst(X.list, k = 2, N = "A"))
})

test_that("hclust_hurst fails if N is negative", {
  expect_error(hclust_hurst(X.list, k = 2, N = -100))
})

test_that("hclust_hurst fails if N is non integer", {
  expect_error(hclust_hurst(X.list, k = 2, N = 100.5))
})

test_that("hclust_hurst fails if Q is nonnumeric", {
  expect_error(hclust_hurst(X.list, k = 2, Q = "A"))
})

test_that("hclust_hurst fails if Q is less than 2", {
  expect_error(hclust_hurst(X.list, k = 2, Q = -2))
  expect_error(hclust_hurst(X.list, k = 2, Q = 1))
})

test_that("hclust_hurst fails if Q is non integer", {
  expect_error(hclust_hurst(X.list, k = 2, Q = 5.5))
})

test_that("hclust_hurst fails if L is nonnumeric", {
  expect_error(hclust_hurst(X.list, k = 2, L = "A"))
})

test_that("hclust_hurst fails if L is less than 2", {
  expect_error(hclust_hurst(X.list, k = 2, L = -2))
  expect_error(hclust_hurst(X.list, k = 2, L = 1))
})

test_that("hclust_hurst fails if L is non integer", {
  expect_error(hclust_hurst(X.list, k = 2, L = 5.5))
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Plots_sim_est.R
\name{plot_tsest}
\alias{plot_tsest}
\title{Plot the estimated Hurst functions and local fractal dimension estimates
for a user provided time series}
\usage{
plot_tsest(
  X,
  H_Est = TRUE,
  H_Smooth_Est = TRUE,
  LFD_Est = TRUE,
  LFD_Smooth_Est = TRUE,
  N = 100,
  Q = 2,
  L = 2
)
}
\arguments{
\item{X}{Data frame where the first column is a numeric time sequence \eqn{t}
and the second one is the values of the time series \eqn{X(t)}.
To get reliable results for time series, it is recommended to use at least 500 time points.}

\item{H_Est}{Logical: If \code{TRUE}, the Hurst function estimated by using \code{\link{Hurst}} is plotted.}

\item{H_Smooth_Est}{Logical: If \code{TRUE}, the smoothed estimated Hurst function is plotted.
The estimated Hurst function is smoothed using the LOESS method.}

\item{LFD_Est}{Logical: If \code{TRUE}, the local fractal dimension estimates are plotted.}

\item{LFD_Smooth_Est}{Logical: If \code{TRUE}, the smoothed estimates of local fractal dimension is plotted.
Smoothed using the LOESS method.}

\item{N}{Argument used for the estimation of Hurst functions and LFD. Number of sub-intervals on which the estimation is performed on. Default is set to 100 sub-intervals.}

\item{Q}{Argument used for the estimation of Hurst functions and LFD. Fixed integer greater than or equal to 2. Default is set to 2.}

\item{L}{Argument used for the estimation of Hurst functions and LFD. Fixed integer greater than or equal to 2. Default is set to 2.}
}
\value{
A ggplot object which is used to plot the time series with raw and smoothed estimates of Hurst function
and local fractal dimension.
}
\description{
Creates a plot of the user provided time series with the
Hurst function estimated using \code{\link{Hurst}}, the smoothed estimated
Hurst function and local fractal dimension estimated using \code{\link{LFD}}
and smoothed estimates of local fractal dimension.
}
\details{
Compared to \code{\link{plot.H_LFD}} the function's first argument is a time series,
not \code{H_LFD} object.
}
\examples{
TS <- data.frame("t" = seq(0, 1, length = 1000), "X(t)" = rnorm(1000))
#Plot of time series, estimated and smoothed Hurst and LFD estimates
plot_tsest(TS)

}
\seealso{
\code{\link{Hurst}}, \code{\link{LFD}}, \code{\link{plot.H_LFD}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Simulation.R
\name{Bbridge}
\alias{Bbridge}
\title{Simulation of Brownian bridge}
\usage{
Bbridge(x_end, t_end, x_start = 0, N = 1000, plot = FALSE)
}
\arguments{
\item{x_end}{Value of the process at the terminating time point.}

\item{t_end}{Terminal time point.}

\item{x_start}{Value of the process at the initial time point.}

\item{N}{Number of time steps on the interval \code{[0,t_end]}.
Default set to 1000.}

\item{plot}{Logical: If \code{TRUE}, the realisation of the Brownian bridge
is plotted in interactive sessions.}
}
\value{
A data frame where the first column is \code{t} and second
column is simulated values of the realisation of Brownian bridge.
}
\description{
This function simulates a realisation of the Brownian bridge over the
time interval \code{[0,t_end]} which has the initial value \code{x_start} and terminates
at \code{x_end} with \code{N} time steps.
}
\examples{
Bbridge(x_end = 2, t_end = 1, plot = TRUE)
}
\references{
Bianchi, S., Frezza, M., Pianese, A., Palazzo, A.M. (2022). Modelling
H-Volatility with Fractional Brownian Bridge. In: Corazza, M., Perna, C., Pizzi, C.,
Sibillo, M. (eds) Mathematical and Statistical Methods for Actuarial Sciences and Finance.
MAF 2022. Springer, Cham. \doi{doi:10.1007/978-3-030-99638-3_16}.
}
\seealso{
\code{\link{Bm}}, \code{\link{FBm}}, \code{\link{FBbridge}}, \code{\link{FGn}}, \code{\link{GHBMP}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rmfrac.R
\docType{package}
\name{Rmfrac-package}
\alias{Rmfrac}
\alias{Rmfrac-package}
\title{Rmfrac: Simulation and analysis of multifractional processes}
\description{
A collection of tools for simulating, analysing and visualising multifractional processes
and time series. The package includes estimation techniques for the Hurst function,
Local Fractal Dimension and several other geometric statistics. It provides highly customisable
plotting functions for simulated realisations, user-provided time series and their statistics.
}
\section{Features}{

\itemize{
\item Simulation of Brownian motion, fractional Brownian motion, fractional Gaussian noise,
Brownian bridge and fractional Brownian bridge.
\item Simulation of Gaussian Haar-based multifractional process (GHBMP).
\item Estimation of Hurst function and Local Fractal Dimension.
\item Customisable plotting functions for GHBMP and user provided time series
with estimates of Hurst function and Local Fractal Dimension.
\item Estimation and visualisation of geometric statistics using realisations of stochastic processes
and time series. Clustering based on the Hurst function. Estimating sojourn measure, excursion area, etc.
\item An interactive Shiny application that provides options to explore and visualise
the core functionalities of the package through simulations and user-provided time series.
}
}

\seealso{
Useful links:
\itemize{
\item \url{https://github.com/Nemini-S/Rmfrac}
\item Report bugs at \url{https://github.com/Nemini-S/Rmfrac/issues}
}
}
\author{
\itemize{
\item Andriy Olenko \href{https://orcid.org/0000-0002-0917-7000}{ORCID} \href{mailto:a.olenko@latrobe.edu.au}{a.olenko@latrobe.edu.au}
\item Nemini Samarakoon \href{https://orcid.org/0009-0000-2107-1973}{ORCID} \href{mailto:neminisamarakoon95@gmail.com}{neminisamarakoon95@gmail.com}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Simulation.R
\name{GHBMP}
\alias{GHBMP}
\title{Simulation of Gaussian Haar-based multifractional processes}
\usage{
GHBMP(t, H, J = 15, num.cores = availableCores(omit = 1))
}
\arguments{
\item{t}{Time point or time sequence on the interval \eqn{[0,1]}.}

\item{H}{Hurst function which depends on \code{t} \eqn{(H(t))}. See Examples for usage.}

\item{J}{Positive integer. \code{J} is recommended to be greater than \eqn{\log_2(length(t))}. For large \code{J} values could
be rather time consuming. Default is set to 15.}

\item{num.cores}{Number of cores to set up the clusters for parallel computing.}
}
\value{
A data frame of class \code{"mp"} where the first column is time moments \code{t} and second column is simulated values of \eqn{X(t)}.
}
\description{
This function simulates a realisation of a Gaussian Haar-based multifractional process at any
time point or time sequence on the interval \eqn{[0,1]}.
}
\details{
The following formula defined in Ayache, A., Olenko, A. & Samarakoon, N. (2026) was used in simulating Gaussian Haar-based multifractional process.

\deqn{X(t) := \sum_{j=0}^{+\infty}  \sum_{k=0}^{2^{j}-1}\left(\int_{0}^{1} (t-s)_{+}^{H_{j}(k/{2^j})-{1}/{2}} h_{j,k}(s)ds \right)\varepsilon_{j,k},}

where

\deqn{  \int_{0}^{1} (t-s)_{+}^{H_{j,k}-\frac{1}{2}} h_{j,k} (s) ds = 2^{-j H_{j,k}} h^{[H_{j,k}]} (2^jt-k)}

with \eqn{h^{[\lambda]} (x) =  \int_{\mathbb{R}} (x-s)_{+}^{\lambda-\frac{1}{2}} h(s) ds}.
\eqn{h} is the Haar mother wavelet, \eqn{j} and \eqn{k} are positive integers, \eqn{t} is time, \eqn{H} is the Hurst function and
\eqn{\varepsilon_{j,k}} is a sequence of independent \eqn{\mathcal{N}(0,1)} Gaussian random variables.
For simulations, the truncated version of this formula with first summation up to J is used.
}
\note{
See Examples for the usage of constant, time-varying, piecewise or step Hurst functions.
}
\examples{
\donttest{
#Constant Hurst function
t <- seq(0, 1, by = (1/2)^10)
H <- function(t) {return(0.4 + 0*t)}
GHBMP(t, H)

#Linear Hurst function
t <- seq(0, 1, by = (1/2)^10)
H <- function(t) {return(0.2 + 0.45*t)}
GHBMP(t, H)

#Oscillating Hurst function
t <- seq(0, 1, by = (1/2)^10)
H <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
GHBMP(t, H)

#Piecewise Hurst function
t <- seq(0, 1, by = (1/2)^10)
H <- function(x) {
ifelse(x >= 0 & x <= 0.8, 0.375 * x + 0.2,
      ifelse(x > 0.8 & x <= 1,-1.5 * x + 1.7, NA))
}
GHBMP(t, H)
}
}
\references{
Ayache, A., Olenko, A. and Samarakoon, N. (2026).
On construction, properties and simulation of Haar-based multifractional processes.
Mathematics and Computers in Simulation. 246:311-332. \doi{doi:10.1016/j.matcom.2026.01.033}.
}
\seealso{
\code{\link{Hurst}}, \code{\link{plot.mp}}, \code{\link{Bm}}, \code{\link{FBm}},
\code{\link{FGn}}, \code{\link{Bbridge}}, \code{\link{FBbridge}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hierarchical_clustering.R
\name{print.hc_hurst}
\alias{print.hc_hurst}
\title{Print method for "hc_hurst" class objects}
\usage{
\method{print}{hc_hurst}(x, ...)
}
\arguments{
\item{x}{Object of class \code{"hc_hurst"}.}

\item{...}{Other arguments.}
}
\value{
Prints an object of class \code{"hc_hurst"}.
}
\description{
Prints the results of hierarchical clustering of realisations of processes.
}
\seealso{
\code{\link{hclust_hurst}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hierarchical_clustering.R
\name{hclust_hurst}
\alias{hclust_hurst}
\title{Hierarchical clustering}
\usage{
hclust_hurst(
  X.t,
  k = NULL,
  h = NULL,
  dist.method = "euclidean",
  method = "complete",
  dendrogram = FALSE,
  N = 100,
  Q = 2,
  L = 2
)
}
\arguments{
\item{X.t}{A list of data frames. In each data frame, the first column is a numeric time sequence
and the second gives the values of the processes or time series. To get reliable results, it is
recommended to use at least 500 time points. See Examples for usage.}

\item{k}{The desired number of clusters.}

\item{h}{The height where the dendrogram should be cut into. Either \code{k} or \code{h} must be specified. If both are provided k is used.}

\item{dist.method}{A string which specifies a registered distance from \code{\link[proxy:dist]{proxy::dist()}}. The default
is \code{"euclidean"}.}

\item{method}{A string which specifies the hierarchical method used. Available methods
are \code{"ward.D"}, \code{"ward.D2"}, \code{"single"}, \code{"complete"}, \code{"average"}, \code{"mcquitty"}, \code{"median"} and \code{"centroid"}.
The default method is \code{"complete"}.}

\item{dendrogram}{Logical: If \code{TRUE} the dendrogram is plotted indicating
the clusters in interactive sessions.}

\item{N}{Argument used for the estimation of Hurst functions. Number of sub-intervals on which the estimation is performed on. Default is set to 100 sub-intervals.}

\item{Q}{Argument used for the estimation of Hurst functions. Fixed integer greater than or equal to 2. Default is set to 2.}

\item{L}{Argument used for the estimation of Hurst functions. Fixed integer greater than or equal to 2. Default is set to 2.}
}
\value{
An object list of class \code{"hc_hurst"} with print and plot methods. The list has following components:
\describe{
\item{\code{cluster_info}}{A data frame indicating the cluster number and distance to cluster center from
each smoothed estimated Hurst function (item). Distance is obtained from the \code{dist.method}.}
\item{\code{cluster}}{A vector with cluster number of each item.}
\item{\code{cluster_sizes}}{Number of items in each cluster.}
\item{\code{centers}}{A data frame of cluster centers. Center obtained as the average of each smoothed estimated Hurst function in the cluster.
Columns denote time points in which estimates were obtained. Row names denote cluster numbers.}
\item{\code{smoothed_Hurst_estimates}}{A data frame of smoothed Hurst estimates. Columns denote time points in which estimates were obtained.
Rows denote estimates for each realisation.}
\item{\code{raw_Hurst_estimates}}{A list of data frames of raw Hurst estimates.}
\item{\code{call}}{Information about the input parameters used.}
}
}
\description{
This function performs hierarchical clustering of realisations
based on the estimated Hurst functions.
}
\details{
The Hurst function of each realisation is estimated using the function \code{\link{Hurst}} and the smoothed Hurst estimates are used
for the cluster analysis. The distances between smoothed Hurst estimates are  computed by the \code{dist.method} provided and passed into
the \code{\link[stats]{hclust}} for hierarchical clustering.
}
\examples{
\donttest{
#Simulation of multifractional processes
t <- seq(0, 1, by = (1/2)^10)
H1 <- function(t) {return(0.1 + 0*t)}
H2 <- function(t) {return(0.2 + 0.45*t)}
H3 <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
X.list.1 <- replicate(3, GHBMP(t,H1),simplify = FALSE)
X.list.2 <- replicate(3, GHBMP(t,H2),simplify = FALSE)
X.list.3 <- replicate(3, GHBMP(t,H3),simplify = FALSE)
X.list <- c(X.list.1, X.list.2, X.list.3)

#Hierarchical clustering based on k = 3 clusters with dendrogram plotted
HC <- hclust_hurst(X.list, k=3, dendrogram = TRUE)
print(HC)

#Plot of smoothed Hurst functions in each cluster with cluster centers
plot(HC,type = "ec")
}
}
\seealso{
\code{\link{print.hc_hurst}}, \code{\link{plot.hc_hurst}}, \code{\link{kmeans_hurst}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Estimation.R
\name{LFD}
\alias{LFD}
\title{Estimation of the local fractal dimension}
\usage{
LFD(X, N = 100, Q = 2, L = 2)
}
\arguments{
\item{X}{Data frame where the first column is a numeric time sequence and the second is the values of the process or time series.}

\item{N}{The same argument that is used for the estimation of Hurst function. Number of sub-intervals on which the estimation is performed on. Default is set to 100 sub-intervals.}

\item{Q}{The same argument that is used for the estimation of Hurst function. Fixed integer greater than or equal to 2. Default is set to 2.}

\item{L}{The same argument that is used for the estimation of Hurst function. Fixed integer greater than or equal to 2. Default is set to 2.}
}
\value{
A data frame where the first column is a time sequence and the second column is estimated values of the local fractal dimension.
}
\description{
This function computes the estimates for the local fractal dimension.
}
\details{
The formula \eqn{\widehat{LFD} = 2-\widehat{H}(t)} is used to compute the estimated local fractal dimension,
where \eqn{\widehat{H}(t)} is the estimated Hurst function.
}
\note{
Since these are estimators of local characteristics, reliable results can only be obtained when a sufficiently large number of points is used.
}
\examples{
\donttest{
#LFD of a multifractional process simulated using GHBMP function
t <- seq(0, 1, by = (1/2)^10)
H <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
X <- GHBMP(t, H)
LFD(X)
}

#LFD of a fractional Browian motion simulated using FBm
X <- FBm(H = 0.5, x_start = 0, t_start = 0, t_end = 2, N = 1000)
LFD(X)

}
\references{
Gneiting, T., and Schlather, M. (2004). Stochastic models
that separate fractal dimension and the Hurst effect. SIAM Review, 46(2):269-282.
\doi{doi:10.1137/S0036144501394387}.
}
\seealso{
\code{\link{Hurst}}, \code{\link{H_LFD}}, \code{\link{plot.mp}}, \code{\link{plot_tsest}}, \code{\link{plot.H_LFD }}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Analyses_functions_2.R
\name{cross_mean}
\alias{cross_mean}
\title{Mean time between crossings}
\usage{
cross_mean(X, A, subI = NULL, plot = FALSE)
}
\arguments{
\item{X}{Data frame where the first column is a numeric time sequence \eqn{t}
and the second one is the values of the time series \eqn{X(t)}.}

\item{A}{Constant level as a numeric value.}

\item{subI}{Time sub-interval is a vector, where the lower bound is
the first element and the upper bound is the second. Optional: If provided mean crossing
times for the sub-interval is returned, otherwise the whole time interval is considered.}

\item{plot}{Logical: If \code{TRUE}, the time series, the constant level and crossing
points are plotted in interactive sessions.}
}
\value{
The estimated mean time between crossings.
}
\description{
Computes the mean duration between crossings of a time series
at a specified constant level for the provided time interval or its sub-interval.
}
\examples{
t <- seq(0, 1, length = 100)
TS <- data.frame("t" = t, "X(t)" = rnorm(100))
cross_mean(TS, 0.1, subI = c(0.2, 0.8), plot = TRUE)
}
\seealso{
\code{\link{cross_T}}, \code{\link{cross_rate}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Estimation.R
\name{Hurst}
\alias{Hurst}
\title{Statistical estimation of the Hurst function}
\usage{
Hurst(X, N = 100, Q = 2, L = 2)
}
\arguments{
\item{X}{Data frame where the first column is a numeric time sequence and the second the values of the process or time series.}

\item{N}{Number of sub-intervals on which the estimation is performed on. Default is set to 100 sub-intervals.}

\item{Q}{Fixed integer greater than or equal to 2. Default is set to 2.}

\item{L}{Fixed integer greater than or equal to 2. Default is set to 2.}
}
\value{
A data frame of where the first column is a time sequence and second column is estimated values of the Hurst function.
}
\description{
This function computes statistical estimates for the Hurst function.
}
\details{
Statistical estimation of the Hurst function is done based on the results of Ayache, A.,
& Bouly, F. (2023). The pointwise Holder exponent of the process for data considered in
the package is equal to the Hurst function. The estimator is built through generalized
quadratic variations of the process associated with its increments. The integer parameters
Q and L define the generalized quadratic variations and the corresponding estimator.
}
\note{
Since these are estimators of local characteristics, reliable results can only be obtained when a sufficiently large number of points is used.
}
\examples{
\donttest{
#Hurst function of a multifractional process simulated using GHBMP function
t <- seq(0, 1, by = (1/2)^10)
H <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
X <- GHBMP(t, H)
Hurst(X)
}

#Hurst function of a fractional Browian motion simulated using FBm
X <- FBm(H = 0.5, x_start = 0, t_start = 0, t_end = 2, N = 1000)
Hurst(X)

}
\references{
Ayache, A. and Bouly, F. (2023). Uniformly and strongly consistent estimation for
the random Hurst function of a multifractional process. Latin American Journal of
Probability and Mathematical Statistics, 20(2):1587–1614. \doi{doi:10.30757/alea.v20-60}.
}
\seealso{
\code{\link{LFD}}, \code{H_LFD}, \code{\link{plot.mp}}, \code{\link{plot_tsest}}, \code{\link{plot.H_LFD}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Analyses_functions_2.R
\name{long_streak}
\alias{long_streak}
\title{Longest increasing/decreasing streak}
\usage{
long_streak(X, direction = "increasing", subI = NULL, plot = FALSE)
}
\arguments{
\item{X}{Data frame where the first column is a numeric time sequence \eqn{t}
and the second one is the values of the time series \eqn{X(t)}.}

\item{direction}{A character string which specifies the direction of
the streak: \code{"increasing"} or \code{"decreasing"}.}

\item{subI}{Time sub-interval is a vector, where the lower bound is
the first element and the upper bound is the second. Optional: If provided
longest streak(s) time span of the sub-interval is returned, otherwise the whole time interval is considered.}

\item{plot}{Logical: If \code{TRUE}, the time series and the longest streaks(s) of
increasing/decreasing is plotted in interactive sessions.}
}
\value{
A data frame with one row for each longest streak, containing the time span
and the corresponding values of \eqn{X(t)} at the streak endpoints.
}
\description{
Computes the time span of the longest increasing or decreasing streak(s)
of a time series for the provided time interval or its sub-interval.
}
\examples{
t <- seq(0, 1, length = 100)
TS <- data.frame("t" = t,"X(t)" = rnorm(100))
long_streak(TS, direction = 'decreasing', subI = c(0.2, 0.8), plot = TRUE)
}
\seealso{
\code{\link{mean_streak}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Simulation.R
\name{Bm}
\alias{Bm}
\title{Simulation of Brownian motion}
\usage{
Bm(x_start = 0, t_start = 0, t_end = 1, N = 1000, plot = FALSE)
}
\arguments{
\item{x_start}{Value of the process at the initial time point (additive constant mean).}

\item{t_start}{Initial time point.}

\item{t_end}{Terminal time point.}

\item{N}{Number of time steps on the interval \code{[t_start,t_end]}.
Default set to 1000.}

\item{plot}{Logical: If \code{TRUE}, the realisation of the Brownian
motion is plotted in interactive sessions.}
}
\value{
A data frame where the first column is \code{t} and second
column is simulated values of the realisation of Brownian motion with added constant mean.
}
\description{
This function simulates a realisation of the Brownian motion over the
time interval \code{[t_start,t_end]} with \code{N} time steps and initial value \code{x_start}.
}
\examples{
Bm(t_end = 2, plot = TRUE)
}
\seealso{
\code{\link{GHBMP}}, \code{\link{FBm}}, \code{\link{FGn}}, \code{\link{Bbridge}}, \code{\link{FBbridge}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Kmeans_clustering.R
\name{kmeans_hurst}
\alias{kmeans_hurst}
\title{K-means clustering}
\usage{
kmeans_hurst(X.t, k, ..., N = 100, Q = 2, L = 2)
}
\arguments{
\item{X.t}{A list of data frames. In each data frame, the first column is a numeric time sequence
and the second gives the values of the processes or time series. To get reliable results, it is
recommended to use at least 500 time points. See Examples for usage.}

\item{k}{The desired number of clusters.}

\item{...}{Optional arguments: \code{iter.max}, \code{nstart} and \code{algorithm}.
Refer \code{\link[stats]{kmeans}}.}

\item{N}{Argument used for the estimation of Hurst functions. Number of sub-intervals on which the estimation is performed on. Default is set to 100 sub-intervals.}

\item{Q}{Argument used for the estimation of Hurst functions. Fixed integer greater than or equal to 2. Default is set to 2.}

\item{L}{Argument used for the estimation of Hurst functions. Fixed integer greater than or equal to 2. Default is set to 2.}
}
\value{
An object list of class \code{"k_hurst"} with print and plot methods. The list has following components:
\describe{
\item{\code{cluster_info}}{A data frame indicating the cluster number and euclidean distance to cluster center of
each smoothed estimated Hurst function (item) }
\item{\code{cluster}}{A vector of cluster number of each item.}
\item{\code{cluster_sizes}}{Number of item in each cluster.}
\item{\code{centers}}{A data frame of cluster centers. Center obtained as the average of each smoothed estimated Hurst function in the cluster.
Columns denote time points in which estimates were obtained. Row names denote cluster numbers.}
\item{\code{smoothed_Hurst_estimates}}{A data frame of smoothed Hurst estimates. Columns denote time points in which estimates were obtained.
Rows denote estimates for each realisation.}
\item{\code{raw_Hurst_estimates}}{A list of data frames of raw Hurst estimates.}
\item{\code{call}}{Information about the input parameters used.}
}
}
\description{
This function performs k-means clustering of realisations
based on the estimated Hurst functions.
}
\details{
The Hurst function of each realisation is estimated using \code{\link{Hurst}}. The smoothed Hurst estimates are
used for k-means clustering in \code{\link[stats]{kmeans}}. The Hartigan and Wong algorithm is used as the
default k-means clustering algorithm.
}
\examples{
\donttest{
#Simulation of multifractional processes
t <- seq(0, 1, by = (1/2)^10)
H1 <- function(t) {return(0.1 + 0*t)}
H2 <- function(t) {return(0.2 + 0.45*t)}
H3 <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
X.list.1 <- replicate(3, GHBMP(t, H1), simplify = FALSE)
X.list.2 <- replicate(3, GHBMP(t, H2), simplify = FALSE)
X.list.3 <- replicate(3, GHBMP(t, H3), simplify = FALSE)
X.list <- c(X.list.1, X.list.2, X.list.3)

#K-means clustering based on k = 3 clusters
KC <- kmeans_hurst(X.list, k = 3)
print(KC)

#Plot of smoothed Hurst functions in each cluster with cluster centers
plot(KC, type = "ec")
}
}
\seealso{
\code{\link{print.k_hurst}}, \code{\link{plot.k_hurst}}, \code{\link{hclust_hurst}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hierarchical_clustering.R
\name{plot.hc_hurst}
\alias{plot.hc_hurst}
\title{Plot smoothed Hurst functions in each cluster with cluster centers}
\usage{
\method{plot}{hc_hurst}(x, type = "estimates", ...)
}
\arguments{
\item{x}{Return from \code{\link{hclust_hurst}}.}

\item{type}{The type of plot required:
\describe{
\item{\code{"estimates"}}{Only the smoothed Hurst functions in each cluster.}
\item{\code{"centers"}}{Only the cluster centers. Center denotes average of all smoothed Hurst functions in the cluster.}
\item{\code{"ec"}}{Both \code{"estimates"} and \code{"centers"}.}
}}

\item{...}{Other arguments.}
}
\value{
A ggplot object which is used to plot the relevant \code{type} of plot: \code{"estimates"}, \code{"centers"} or \code{"ec"}.
}
\description{
Creates a plot of the smoothed Hurst functions of realisations of processes (or time series) separately in each cluster with cluster centers using the return from
\code{\link{hclust_hurst}}. Options to plot only estimates, only centers or both are available.
}
\examples{
\donttest{
#Simulation of multifractional processes
t <- seq(0, 1, by = (1/2)^10)
H1 <- function(t) {return(0.1 + 0*t)}
H2 <- function(t) {return(0.2 + 0.45*t)}
H3 <- function(t) {return(0.5 - 0.4*sin(6*3.14*t))}
X.list.1 <- replicate(3, GHBMP(t,H1), simplify = FALSE)
X.list.2 <- replicate(3, GHBMP(t,H2), simplify = FALSE)
X.list.3 <- replicate(3, GHBMP(t,H3), simplify = FALSE)
X.list <- c(X.list.1, X.list.2, X.list.3)

#Hierarchical clustering based on k=3 clusters with dendrogram plotted
HC<- hclust_hurst(X.list, k = 3, dendrogram = TRUE)
print(HC)

#Plot of smoothed Hurst functions in each cluster with cluster centers
plot(HC, type = "ec")
}
}
\seealso{
\code{\link{hclust_hurst}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Analyses_functions_1.R
\name{sojourn}
\alias{sojourn}
\title{Estimated sojourn measure}
\usage{
sojourn(X, A, N = 10000, level = "greater", subI = NULL, plot = FALSE)
}
\arguments{
\item{X}{Data frame where the first column is a numeric time sequence \eqn{t}
and the second one is the values of the time series \eqn{X(t)}.}

\item{A}{Constant level as a numeric value.}

\item{N}{Number of steps on the time interval (or time sub-interval) used for computations. Default set to 10000.}

\item{level}{A vector of character strings which specifies which sojourn
measure required for \code{X}, \code{"greater"} or \code{"lower"} than \code{A}. Default set to \code{"greater"}.}

\item{subI}{Time sub-interval is a vector, where the lower bound is
the first element and the upper bound is the second. Optional: If provided,
the estimated sojourn measure for the sub-interval is returned, otherwise the whole time interval is considered.}

\item{plot}{Logical: If \code{TRUE}, the time series, constant level (in blue) and the sojourn measure (in red) are plotted
in interactive sessions.}
}
\value{
Estimated sojourn measure.
}
\description{
Computes the estimated sojourn measure for a time series \eqn{X(t)} greater or lower than the
constant level \code{A} for the provided time interval or its sub-interval.
}
\examples{
t <- seq(0, 1, length = 1000)
TS <- data.frame("t" = t,"X(t)" = rnorm(1000))
sojourn(TS, 0.8, level = 'lower', subI = c(0.5, 0.8), plot = TRUE)
}
\seealso{
\code{\link{exc_Area}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Simulation.R
\name{FBm}
\alias{FBm}
\title{Simulation of fractional Brownian motion}
\usage{
FBm(H, x_start = 0, t_start = 0, t_end = 1, N = 1000, plot = FALSE)
}
\arguments{
\item{H}{Hurst parameter which lies between 0 and 1.}

\item{x_start}{Value of the process at the initial time point (additive constant mean).}

\item{t_start}{Initial time point.}

\item{t_end}{Terminal time point.}

\item{N}{Number of time steps on the interval \code{[t_start,t_end]}.
Default set to 1000.}

\item{plot}{Logical: If \code{TRUE}, the realisation of the fractional Brownian
motion is plotted in interactive sessions.}
}
\value{
A data frame where the first column is \code{t} and second
column is simulated values of the realisation of fractional Brownian motion
with added constant mean.
}
\description{
This function simulates a realisation of the fractional Brownian motion over
the time interval \code{[t_start,t_end]} for a provided Hurst parameter, which has
the initial value \code{x_start}.
}
\examples{
FBm(H = 0.5, plot = TRUE)
}
\references{
Banna, O., Mishura, Y., Ralchenko, K., & Shklyar, S. (2019). Fractional Brownian motion:
Approximations and Projections. John Wiley & Sons. \doi{doi:10.1002/9781119476771.app3}.
}
\seealso{
\code{\link{FGn}}, \code{\link{Bm}}, \code{\link{GHBMP}}, \code{\link{Bbridge}}, \code{\link{FBbridge}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Plots_sim_est.R
\name{plot.H_LFD}
\alias{plot.H_LFD}
\title{Plot the estimated Hurst functions and local fractal dimension estimates
for objects of class \code{H_LFD}}
\usage{
\method{plot}{H_LFD}(
  x,
  H_Est = TRUE,
  H_Smooth_Est = TRUE,
  LFD_Est = TRUE,
  LFD_Smooth_Est = TRUE,
  ...
)
}
\arguments{
\item{x}{Return from \code{\link{H_LFD}}.}

\item{H_Est}{Logical: If \code{TRUE}, the Hurst function estimated by using \code{\link{Hurst}} is plotted.}

\item{H_Smooth_Est}{Logical: If \code{TRUE}, the smoothed estimated Hurst function is plotted.
The estimated Hurst function is smoothed using the LOESS method.}

\item{LFD_Est}{Logical: If \code{TRUE}, the local fractal dimension estimates are plotted.}

\item{LFD_Smooth_Est}{Logical: If \code{TRUE}, the smoothed estimates of local fractal dimension is plotted.
Smoothed using the LOESS method.}

\item{...}{Other arguments.}
}
\value{
A ggplot object which is used to plot the time series with theoretical, raw and smoothed estimates of Hurst function
and raw and smoothed estimates of local fractal dimension.
}
\description{
Creates a plot of the user provided time series with the
Hurst function estimated using \code{\link{Hurst}}, the smoothed estimated
Hurst function and local fractal dimension estimated using \code{\link{LFD}}
and smoothed estimates of local fractal dimension for objects of class \code{"H_LFD"}.
}
\details{
Compared to \code{\link{plot_tsest}}, the function's argument is a \code{"H_LFD"}
object, not a time series.
}
\examples{
TS <- data.frame("t" = seq(0, 1, length = 1000), "X(t)" = rnorm(1000))
Object <- H_LFD(TS)
#Plot of time series, estimated and smoothed Hurst and LFD estimates
plot(Object)

}
\seealso{
\code{\link{H_LFD}}, \code{\link{Hurst}}, \code{\link{LFD}}, \code{\link{plot_tsest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Covariance_fn.R
\name{est_cov}
\alias{est_cov}
\title{Empirical covariance function}
\usage{
est_cov(X, theta = 0.1, plot = FALSE)
}
\arguments{
\item{X}{A data frame where the first column is the numeric time sequence and the remaining
columns are the values of each realisation of the process.}

\item{theta}{Smoothing parameter.}

\item{plot}{Logical: If TRUE, a 3D surface plot of the covariance function is plotted in interactive sessions.}
}
\value{
An \eqn{m \times m} matrix, where \eqn{m} is the number of time points.
Each element represents the estimated value of covariance function for the
corresponding time points. Time points are arranged in ascending order.
}
\description{
Computes the empirical covariance function of a process, for each pair of time points in the
time sequence using M realisations of the process.
}
\details{
The smoothing parameter \code{theta} can help to better visualise changes between
neighbour estimated values.
}
\examples{
\donttest{
#Matrix of empirical covariance estimates of the GHBMP with Hurst function H.
t <- seq(0, 1, by = (1/2)^8)
H <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
#Only 5 realisations of GHBMP are used in this example to reduce the computational time.
X.t <- replicate(5, GHBMP(t, H), simplify = FALSE)
X <- do.call(rbind, lapply(X.t, function(df) df[, 2]))
Data <- data.frame(t, t(X))
cov.mat <- est_cov(Data, theta = 0.2, plot = TRUE)
cov.mat
}
}
\seealso{
\code{\link{cov_GHBMP}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Analyses_functions_2.R
\name{cross_T}
\alias{cross_T}
\title{Estimated crossing times}
\usage{
cross_T(X, A, subI = NULL, plot = FALSE, vline = FALSE)
}
\arguments{
\item{X}{Data frame where the first column is a numeric time sequence \eqn{t}
and the second one is the values of the time series \eqn{X(t)}.}

\item{A}{Constant level as a numeric value.}

\item{subI}{Time sub-interval as a vector, where the lower bound is
the first element and the upper bound is the second. Optional: If provided estimated crossing
times of the sub-interval is returned, otherwise the whole time interval is considered.}

\item{plot}{Logical: If \code{TRUE}, the time series, the constant level and corresponding \eqn{t} values are plotted
in interactive sessions.}

\item{vline}{Logical: If \code{TRUE}, a vertical line is plotted at the crossing point(s).}
}
\value{
The estimated crossing times at a given level.
}
\description{
Computes the estimated \eqn{t} value(s), in which a time series crosses a specific
constant level for the provided time interval or its sub-interval.
}
\examples{
t <- seq(0, 1, length = 100)
TS <- data.frame("t" = t, "X(t)" = rnorm(100))
cross_T(TS, 0.1, subI = c(0.2, 0.8), plot = TRUE, vline = TRUE)

}
\seealso{
\code{\link{cross_rate}}, \code{\link{cross_mean}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Analyses_functions_2.R
\name{mean_streak}
\alias{mean_streak}
\title{Mean time span of increasing/decreasing streaks}
\usage{
mean_streak(X, direction = "increasing", subI = NULL, plot = FALSE)
}
\arguments{
\item{X}{Data frame where the first column is a numeric time sequence \eqn{t}
and the second one is the values of the time series \eqn{X(t)}.}

\item{direction}{A character string which specifies the direction of
the streak: \code{"increasing"} or \code{"decreasing"}.}

\item{subI}{Time sub-interval is a vector, where the lower bound is
the first element and the upper bound is the second. Optional: If provided mean time span
of the sub-interval is returned, otherwise the whole time interval is considered.}

\item{plot}{Logical: If \code{TRUE}, the time series and the increasing/decreasing
streaks are plotted in interactive sessions.}
}
\value{
Mean time span of the increasing/decreasing streaks.
}
\description{
Computes the mean time span of the increasing/decreasing streaks
for the provided time interval or its sub-interval.
}
\examples{
t <- seq(0, 1 ,length = 100)
TS <- data.frame("t" = t,"X(t)" = rnorm(100))
mean_streak(TS, direction = 'decreasing', subI = c(0.2, 0.8), plot = TRUE)

}
\seealso{
\code{\link{long_streak}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Shinyapp.R
\name{shinyapp_sim}
\alias{shinyapp_sim}
\title{Shiny app to visualise and analyse processes}
\usage{
shinyapp_sim()
}
\value{
An interactive Shiny app with the following user interface controls.

\strong{GHBMP simulation}
\describe{
\item{\code{Hurst function}}{Input the Hurst function in terms of \code{t}.
The default is set to \code{0.5 + 0*t}.}
\item{\code{Time sequence}}{Input the time sequence which belongs to the interval \eqn{[0,1]}. The default is set to \code{seq(0, 1, by = (1/2)^10)}.}
\item{\code{J}}{Input or select a positive integer. For large \code{J} could be rather time consuming. Default is set to 15.}}
\strong{Hurst function and LFD estimation}
\describe{
\item{\code{Number of sub-intervals for estimation}}{Default is set to 100.}
\item{\code{Q}}{Input or select an integer greater than or equal to 2. Default is set to 2.}
\item{\code{L}}{Input or select an integer greater than or equal to 2. Default is set to 2.}
\item{\code{Plot}}{Choose the required from: \code{Theoretical Hurst function}, \code{Raw estimate of Hurst function}, \code{Smoothed estimate of Hurst function}, \code{Raw estimate of Local Fractal Dimension}, \code{Smoothed estimate of Local Fractal Dimension}.}}
\strong{Excursion set and area}
\describe{
\item{\code{Number of time steps}}{Input the number of steps the time interval needs to be split into.}
\item{\code{Constant level}}{Input the constant level.}
\item{\code{Compare to level}}{\code{Greater}, \code{Lower}.}
\item{\code{Plot}}{Select: \code{Excursion set}, \code{Excursion area}.}}
\strong{Longest streak}
\describe{
\item{\code{Longest streak to plot}}{Select: \code{Increasing}, \code{Decreasing}.}}
\strong{Maximum and minimum}
\describe{
\item{\code{Plot}}{Select: \code{Maximum}, \code{Minimum}.}}
}
\description{
Launches a Shiny app to visualise and analyse realisations of Brownian motion (see \code{\link{Bm}}),
Brownian bridge (see \code{\link{Bbridge}}), fractional Brownian motion (see \code{\link{FBm}}),
fractional Brownian bridge (see \code{\link{FBbridge}}), fractional Gaussian noise (see \code{\link{FGn}}),
Gaussian Haar-based multifractional processes (see \code{\link{GHBMP}}) and user-provided time series data.
}
\details{
For \code{Input time series}, provide a \code{.csv} file that has headers of its columns.
It must have two columns: the first column should contain the numeric time sequence \eqn{t},
and the second the corresponding time series values \eqn{X(t)}. To get reliable estimation
results for time series, it is recommended to use at least 500 time points.
Make sure there are no extra header rows or footnotes.
}
\examples{
if (interactive()) {
  shinyapp_sim()
}

}
\seealso{
\code{\link{Bm}}, \code{\link{FBm}}, \code{\link{FGn}}, \code{\link{Bbridge}}, \code{\link{FBbridge}},
\code{\link{GHBMP}}, \code{\link{Hurst}}, \code{\link{LFD}},  \code{\link{sojourn}},  \code{\link{exc_Area}}
\code{\link{long_streak}}, \code{\link{X_max}}, \code{\link{X_min}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Analyses_functions_2.R
\name{RS_Index}
\alias{RS_Index}
\title{Relative strength index}
\usage{
RS_Index(X, period = 14, plot = FALSE, overbought = 70, oversold = 30)
}
\arguments{
\item{X}{A numeric vector.}

\item{period}{Period length used for smoothing. Default is set to 14.}

\item{plot}{Logical: If \code{TRUE}, the time series and the
RSI are plotted (with \code{overbought} and
\code{oversold} levels) in the same window in interactive sessions.}

\item{overbought}{Horizontal line which indicates an overbought level in the RSI plot. Default is set to 70.}

\item{oversold}{Horizontal line which indicates an oversold level in the RSI plot. Default is set to 30.}
}
\value{
A list or vector of the RSI values.
}
\description{
This function computes the Relative Strength Index (RSI) for a time series.
}
\details{
To compute the RSI,

\deqn{\text{RSI} = 100 \dfrac{\text{Average\_gain}}{\text{Average\_gain}+\text{Average\_loss}}}

formula is used. Average gain and average loss are computed using the Wilders's smoothing method.
}
\examples{
X <- c(74.44,74.19,74.25,73.65,74.37,74.73,75.15,75.46,75.88,76.78,
            75.81,76.53,75.11,76.28,76.68,76.08,76.53,76.11,76.42,75.58,
            75.44,75.46,74.98)
RS_Index(X, plot = TRUE)

}
\references{
Wilder, J. W. (1978). New concepts in technical trading systems. Greensboro, NC.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Covariance_fn.R
\name{cov_GHBMP}
\alias{cov_GHBMP}
\title{Covariance of Gaussian Haar-based multifractional processes}
\usage{
cov_GHBMP(
  t,
  H,
  J = 8,
  theta = NULL,
  plot = FALSE,
  num.cores = availableCores(omit = 1)
)
}
\arguments{
\item{t}{Time point or time sequence on the interval \eqn{[0,1]}.}

\item{H}{Hurst function \eqn{H(t)} which depends on \code{t}.}

\item{J}{Positive integer. For large J values could be rather time consuming. Default is set to 8.}

\item{theta}{Optional: Smoothing parameter.}

\item{plot}{Logical: If TRUE, a 3D surface plot of the covariance function is plotted in interactive sessions.}

\item{num.cores}{Number of cores to set up the clusters for parallel computing.}
}
\value{
An \eqn{m \times m} matrix, where \eqn{m} is the length of \code{t}.
}
\description{
Computes the theoretical covariance matrix of a Gaussian Haar-based multifractional process.
}
\details{
To make it comparable with the empirical covariance function the same smoothing parameter
\code{theta} can be used if needed.
}
\examples{
\donttest{
t <- seq(0, 1, by = 0.01)
H <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}

#Smoothed covariance function
cov_GHBMP(t, H, theta = 0.1, plot = TRUE)

#Non-smoothed covariance function
cov_GHBMP(t, H, plot = TRUE)
}
}
\references{
Ayache, A., Olenko, A. and Samarakoon, N. (2026).
On construction, properties and simulation of Haar-based multifractional processes.
Mathematics and Computers in Simulation. 246:311-332. \doi{doi:10.1016/j.matcom.2026.01.033}.
}
\seealso{
\code{\link{GHBMP}}, \code{\link{est_cov}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Plots_sim_est.R
\name{plot.mp}
\alias{plot.mp}
\title{Plot Gaussian Haar-based multifractional processes with their
theoretical and estimated Hurst functions and local fractal dimension}
\usage{
\method{plot}{mp}(
  x,
  H = NULL,
  H_Est = TRUE,
  H_Smooth_Est = TRUE,
  LFD_Est = TRUE,
  LFD_Smooth_Est = TRUE,
  N = 100,
  Q = 2,
  L = 2,
  ...
)
}
\arguments{
\item{x}{Return from \code{\link{GHBMP}}. To get reliable results for simulated trajectories, it is recommended
to use at least 500 time points.}

\item{H}{Theoretical Hurst function. Optional: If provided, the theoretical Hurst function is plotted.}

\item{H_Est}{Logical: If \code{TRUE}, the Hurst function estimated by using \code{\link{Hurst}} is plotted.}

\item{H_Smooth_Est}{Logical: If \code{TRUE}, the smoothed estimated Hurst function is plotted.
The estimated Hurst function is smoothed using the LOESS method.}

\item{LFD_Est}{Logical: If \code{TRUE}, the local fractal dimension estimates are plotted.}

\item{LFD_Smooth_Est}{Logical: If \code{TRUE}, the smoothed estimates of local fractal dimension is plotted.
Smoothed using the LOESS method.}

\item{N}{Argument used for the estimation of Hurst functions and LFD. Number of sub-intervals on which the estimation is performed on. Default is set to 100 sub-intervals.}

\item{Q}{Argument used for the estimation of Hurst functions and LFD. Fixed integer greater than or equal to 2. Default is set to 2.}

\item{L}{Argument used for the estimation of Hurst functions and LFD. Fixed integer greater than or equal to 2. Default is set to 2.}

\item{...}{Other arguments.}
}
\value{
A ggplot object which is used to plot the multifractional process with theoretical, raw and smoothed estimates of Hurst function
and raw and smoothed estimates of local fractal dimension.
}
\description{
Creates a plot of the Gaussian Haar-based multifractional process
simulated by using \code{\link{GHBMP}} with theoretical Hurst function (if provided),
Hurst function estimated using \code{\link{Hurst}}, the smoothed estimated
Hurst function and local fractal dimension estimated using \code{\link{LFD}}
and smoothed estimates of local fractal dimension.
}
\examples{
\donttest{
#Simulation of the multifractional process and estimation of the Hurst function
t <- seq(0, 1, by = (1/2)^10)
H <- function(t) {return(0.5 - 0.4 * sin(6 * 3.14 * t))}
X <- GHBMP(t, H)

#Plot of process, theoretical Hurst function, estimated and smoothed Hurst and LFD estimates
plot(X, H = H)

#Plot of process, estimated and smoothed Hurst and LFD estimates
plot(X)
}
}
\seealso{
\code{\link{GHBMP}}, \code{\link{Hurst}}, \code{\link{LFD}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Analyses_functions_1.R
\name{X_max}
\alias{X_max}
\title{Estimated maximum of a time series}
\usage{
X_max(X, subI = NULL, plot = FALSE, vline = FALSE, hline = FALSE)
}
\arguments{
\item{X}{Data frame where the first column is a numeric time sequence \eqn{(t)}
and the second the values of the time series \eqn{(X(t))}.}

\item{subI}{Time sub-interval is a vector where the lower bound is
the first element and upper bound is the second. Optional: If provided maximum of the
sub-interval is returned, otherwise the whole time sequence is considered.}

\item{plot}{Logical: If \code{TRUE}, the time series, the maximum and corresponding \eqn{t} values are plotted in interactive sessions.}

\item{vline}{Logical: If \code{TRUE}, a vertical line is plotted across the maximum.}

\item{hline}{Logical: If \code{TRUE}, a horizontal line is plotted across the maximum.}
}
\value{
A list of numeric vector(s). The first element in the vector is the corresponding \eqn{t} value and second the
maximum of the time series.
}
\description{
This function computes the maximum of a time series for the provided
time interval or its sub-interval.
}
\examples{
t <- seq(0, 1, length = 100)
TS <- data.frame("t" = t, "X(t)" = rnorm(100))
X_max(TS, subI = c(0.5, 0.8), plot = TRUE)

}
\seealso{
\code{\link{X_min}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Plots_sim_est.R
\name{H_LFD}
\alias{H_LFD}
\title{Creates objects of class \code{H_LFD}}
\usage{
H_LFD(X, N = 100, Q = 2, L = 2)
}
\arguments{
\item{X}{Data frame where the first column is a numeric time sequence \eqn{t}
and the second is the values of the time series \eqn{X(t)}.
To get reliable results, it is recommended to use at least 500 time points.}

\item{N}{Argument used for the estimation of Hurst functions and LFD. Number of sub-intervals on which the estimation is performed on. Default is set to 100 sub-intervals.}

\item{Q}{Argument used for the estimation of Hurst functions and LFD. Fixed integer greater than or equal to 2. Default is set to 2.}

\item{L}{Argument used for the estimation of Hurst functions and LFD. Fixed integer greater than or equal to 2. Default is set to 2.}
}
\value{
The return from \code{\link{H_LFD}} is an object list of class \code{"H_LFD"} with the following components:
\describe{
\item{\code{Raw_Hurst_estimates}}{A data frame of where the first column is a time sequence and second column is estimated values of the Hurst function.}
\item{\code{Smoothed_Hurst_estimates}}{A data frame of where the first column is a time sequence and second column is smoothed estimates of the Hurst function. Smoothed using the LOESS method.}
\item{\code{Raw_LFD_estimates}}{A data frame of where the first column is a time sequence and second column is Local fractal dimension estimates.}
\item{\code{Smoothed_LFD_estimates}}{A data frame of where the first column is a time sequence and second column is smoothed estimates of Local fractal dimension. Smoothed using the LOESS method.}
\item{\code{Data}}{User provided time series.}}
}
\description{
For user provided time series creates objects of class \code{"H_LFD"} with the
Hurst function estimated using \code{\link{Hurst}}, local fractal dimension
estimated using \code{\link{LFD}} and smoothed estimated Hurst function and LFD added.
}
\note{
Since these are estimators of local characteristics, reliable results can only be obtained when a sufficiently large number of points is used.
}
\examples{
TS <- data.frame("t" = seq(0, 1, length = 1000),"X(t)" = rnorm(1000))
Object <- H_LFD(TS)
#Plot of time series, estimated and smoothed Hurst and LFD estimates
plot(Object)

}
\seealso{
\code{\link{plot.H_LFD}}, \code{\link{Hurst}}, \code{\link{LFD}}
}
